/**********************************************************************
 *    Copyright Torquil Gault 2005+
 * #       This program is free software; you can redistribute it and/or
 * #       modify it under the terms of the GNU General Public License
 * #       as published by the Free Software Foundation; either version 2
 * #       of the License, or (at your option) any later version.
 * #
 * #       This program is distributed in the hope that it will be useful,
 * #       but WITHOUT ANY WARRANTY; without even the implied warranty of
 * #       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * #       GNU General Public License for more details.
 * #
 * #       You should have received a copy of the GNU General Public License
 * #       along with this program; if not, write to the Free Software
 * #       Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * #
 * #
 * ###############################################################################
 **********************************************************************/

/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#ifndef VISUALC
#include <dirent.h>
#include <unistd.h>
#else
#include <windows.h>
#include <direct.h>
#include <stdlib.h>
#define strncasecmp strnicmp
#endif

#include "pDBlibrary.h"


/******************************************************************************/
#include <FL/Fl.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Browser.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Check_Browser.H>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Tile.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Counter.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Help_View.H>
#include <FL/Fl_Text_Editor.H>
#include <FL/Fl_Text_Buffer.H>

#include "ADMPdata.h"
#include "ADMPexternals.h"

/****************************************************************************/
#ifdef WINDOWS
#ifdef VISUALC
#define S_IRUSR 00400
#define S_IWUSR 00200
#endif
#define S_IRGRP 00040
#define S_IROTH 00004
void fchmod(int f, int m) {
}
#endif

/****************************************************************************/
int seed1, seed2, seed3;

ADMPsystem config;
ADMPplayer  players[MAX_MEMBERS+4], qkmonsters[MAX_QKMONSTERS+2];

char identifiers[64] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
char mybuf1[2048], mybuf2[2048];

//int i,j,k;

int i_EORanswer, i_WeaponUsed, i_prevWeaponEditted, i_monsterType, i_monsterCount, i_monsterNoteID, i_srdMonsterID;
int qkMonster, qkMonsterWeapon;

float ADMPversion;

FILE *logfile=NULL, *openf=NULL;

Fl_Input *w_Initiatives[MAX_MEMBERS];

Fl_Double_Window *a_EORWindow=(Fl_Double_Window *)0;
Fl_Browser *a_EORMessageList=(Fl_Browser *)0;
Fl_Button *a_EORMessageYes=(Fl_Button *)0;
Fl_Button *a_EORMessageNo=(Fl_Button *)0;
Fl_Button *AYSButton[3]={NULL,NULL,NULL};

ADMPcombatmods combatmods[] = {
{"Attacker flanking defender", {{+2,0},{+0,0}}},
{"Attacker on higher ground", {{+1,0},{+0,0}}},
{"Attacker prone", {{-4,0},{+0,0}}},
{"Attacker invisible", {{+2,0},{+2,0}}},
{"Defender kneeling", {{-2,1},{-2,1}}},
{"Defender prone", {{-4,1},{-4,1}}},
{"Defender stunned/climbing", {{-2,1},{+2,1}}},
{"Defender surprised/flat-footed", {{+0,1},{+2,1}}},
{"Defender running", {{+0,1},{-2,1}}},
{"Defender pinned", {{-4,1},{-4,1}}},
{"Cover 25%", {{+2,1},{2,1}}}, 				//|+1,2
{"Cover 50%", {{+4,1},{4,1}}}, 				//|+2,2}},
{"Cover 75%", {{+7,1},{7,1}}}, 				//|+3,2}},
{"Cover 90%", {{+10,1},{10,1}}},
{"", {{+0,0},{0,0}}}};  			//|+4,2}},

ADMPadjustments adjustments[] = {
{"To Hit", MOD_BAB},
{"Armor Class", MOD_AC},
{"All Saves", MOD_SAVE},
{"Damage", MOD_DAMAGE},
{"Initiative", MOD_INIT},
{"Hit Points", MOD_HP},
{"Skill Checks", MOD_SKILL},
{"Save Fortitude", MOD_SAVE_FORTITUDE},
{"Save Reflex", MOD_SAVE_REFLEX},
{"Save Will", MOD_SAVE_WILL},
{"", -1}};

#define MAX_ADJEFFECTTYPES 9		// this must match how many type are below
char *adjusteffecttype[] = {"Enhancement", "Morale", "Luck", "Haste", "Armor", "Shield", "Deflection", "Dodge", "Other"};

int xp_cr[21][21] = {
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,300,600,900,1350,1800,2700,3600,5400,7200,10800,0,0,0,0,0,0,0,0,0,0},
{0,300,600,900,1350,1800,2700,3600,5400,7200,10800,0,0,0,0,0,0,0,0,0,0},
{0,300,600,900,1350,1800,2700,3600,5400,7200,10800,0,0,0,0,0,0,0,0,0,0},
{0,300,600,800,1200,1600,2400,3200,4800,6400,9600,12800,0,0,0,0,0,0,0,0,0},
{0,300,500,750,1000,1500,2250,3000,4500,6000,9000,12000,18000,0,0,0,0,0,0,0,0},
{0,300,450,600,900,1200,1800,2700,3600,5400,7200,10800,14400,21600,0,0,0,0,0,0,0},
{0,263,394,525,700,1050,1400,2100,3150,4200,6300,8400,12600,16800,25200,0,0,0,0,0,0},
{0,200,300,450,600,800,1200,1600,2400,3600,4800,7200,9600,14400,19200,28800,0,0,0,0,0},
{0,0,225,338,506,675,900,1350,1800,2700,4050,5400,8100,10800,16200,21600,32400,0,0,0,0},
{0,0,0,250,375,563,750,1000,1500,2000,3000,4500,6000,9000,12000,18000,24000,36000,0,0,0},
{0,0,0,0,275,413,619,825,1100,1650,2200,3300,4950,6600,9900,13200,19800,26400,39600,0,0},
{0,0,0,0,0,300,450,675,900,1200,1800,2400,3600,5400,7200,10800,14400,21600,28800,43200,0},
{0,0,0,0,0,0,325,488,731,975,1300,1950,2600,3900,5850,7800,11700,15600,23400,31200,46800},
{0,0,0,0,0,0,0,350,525,788,1050,1400,2100,2800,4200,6300,8400,12600,16800,25200,33600},
{0,0,0,0,0,0,0,0,375,563,844,1125,1500,2250,3000,4500,6750,9000,13500,18000,27000},
{0,0,0,0,0,0,0,0,0,400,600,900,1200,1600,2400,3200,4800,7200,9600,14400,19200},
{0,0,0,0,0,0,0,0,0,0,425,638,956,1275,1700,2550,3400,5100,7650,10200,15300},
{0,0,0,0,0,0,0,0,0,0,0,450,675,1013,1350,1800,2700,3600,5400,8100,10800},
{0,0,0,0,0,0,0,0,0,0,0,475,713,1069,1425,1900,2850,3800,5700,8550,0},
{0,0,0,0,0,0,0,0,0,0,0,500,750,1000,1500,2000,3000,4000,6000,0,0}};

ADMPweapon stdweapons[MAX_STDWEAPONS];

char *dam_reduct_types[] = { "None", "Magic Weapon", "Any", "Adamantine", "Bludgeoning", "Chaotic",
							"Cold Iron", "Epic", "Evil", "Good", "Lawful", "Magic", "Silver", "Slashing"};

Fl_Text_Buffer *notes_buffer, *edit_notes_buffer, *detailed_buffer[2];

int adjustValues[5][3], prev_part_selected;

char systemDir[256];

char *flagTypes[] = {"Stabilised", "Disabled", "Ignore Crits", "Auto Stablise", "Evasion", "NPC", NULL};

char directorySep[2] = "/";

	// if you change this, update the code in initialiseSystem
char *actionTypes[] = {"Melee Attack", "Missile/Thrown Attack", "Spell - Damage", "Spell - WILL Save",
				"Spell - REF Save", "Spell - FORT Save", "Spell - Touch Attk", "Spell - Ranged Touch Attk",
				"Heal/Stabilise", "Tuning", "Skill Check"};

char prevHtmlPage[5][256];
int htmlLevel = -1;

char *environment[] = {" Any", "A chaos-aligned plane", "A chaotic-aligned plane", "A chaotic evil-aligned plane", "A chaotic good-aligned plane",
"A evil-aligned plane", "A good-aligned plane.", "A lawful-aligned plane", "A lawful evil-aligned plane", "A lawful good-aligned plane",
"An evil-aligned plane", "Any", "Any good-aligned plane", "Any land and underground", "Cold aquatic",
"Cold desert", "Cold forests", "Cold hills", "Cold marshes", "Cold mountains",
"Cold plains", "Elemental Plane of Air", "Elemental Plane of Earth", "Elemental Plane of Fire",
"Elemental Plane of Water", "Ethereal Plane", "Ever-Changing Chaos of Limbo", "Plane of Shadow", "Positive Energy Plane",
"Temperate aquatic", "Temperate deserts", "Temperate forests", "Temperate hills", "Temperate marshes",
"Temperate mountains", "Temperate plains", "Underground", "Warm aquatic", "Warm deserts",
"Warm forests", "Warm hills", "Warm marshes", "Warm mountains", "Warm plains", NULL};

int i_spellEntries[128];

/******************************************************************************/
extern Fl_Menu_Item races[];
extern Fl_Menu_Item classes[];
extern Fl_Double_Window *a_WeaponWindow, *a_Exit;

/******************************************************************************/
void setCombatDisplay(int );
void doSystemAlert(char *);
int doAreYouSure(char *);
void saveAllData();
int parseDice(char *dice); //, int *mult, int *die, int *plus);
int parseCritical(char *crit, int dice, int *from, int *to, int *mult);
void loadAllData();
void doInitiativeAction(int );
int getRND(int );
int getHPs(int );
int getHPpercentage(int );
void actionQkMonsters(int );
int doEndOfRoundMessages(int );
void init16();
int getAdjustment(int, int);
void doOtherAction(int );
void calcCombatMods(int );
void writeLog(char *);
void mytolower(char *);
int whichChecked(Fl_Check_Browser *);
void doAutoDamageAction(int );
int calcAutoDamage(int , int);
int calcAction(int );
int getMaxHPs(int );
void reloadNames(int );
int getWeaponAttackBonus(int , int );
void setSystemFlag(int );
void doTriggerAction(int );
void doAdjustmentAction(int );
void loadStdModifiers();
void rollRNDDice();
void qkMonsterAction(int );
int writeAllPlayerData(char *);
int writeAllMonsterData(char *);
int writeAllStdMonsterData(char *);
int writeAllSRDMonsterData(char *);
int readAllPlayerData(char *);
int readAllMonsterData(char *, ADMPplayer *);
int readAllStdMonsterData(char *);
void convertToVersion();
void loadMonsterDescription(char *);
void loadSRDspellEntries(char *);
int readAllStdWeaponData();
int writeSystemData(char *);
int readSystemData(char *);
void doNotesAction(int, int);

/******************************************************************************/
/******************************************************************************/
void initialiseSystem(int argc, char **argv) {
	int i;

	struct tm *tp;
	time_t t;

	init16();

#ifdef WINDOWS
	strcpy(directorySep, "\\");
#else
	strcpy(directorySep, "/");
	umask(000);
#endif

	getcwd(systemDir, 255); strcat(systemDir, directorySep);		// get the current working directory

	i = 0;
	while (systemDir[i] != '\0') {
		if (systemDir[i] == '\\') { systemDir[i] = '/'; }
		i++;
	}

	printf("System Directory = %s\n", systemDir);

	a_CMessage->value("Version 1.12\nCopyright Torquil Gault 2005+\nPortions copyright Wizards of the Coast (see the Open Gaming Licence)");
	ADMPversion = 1.13;

	a_TMain->value(a_TCombat);

	setCombatDisplay(0);
	a_CBActionType->textsize(13);
	a_CTAttacksLeft->textsize(24);
	a_IActions->textsize(13);
	a_CTACBreakdown->textsize(10);
	a_CTHelp->labelsize(20);
	a_ODRList1->textsize(14); a_ODRList2->textsize(14); a_ODRList3->textsize(14); a_ODRList4->textsize(14);
	a_ODRList5->textsize(14); a_ODRList6->textsize(14); a_ODRList7->textsize(14); a_ODRList8->textsize(14);
	a_OTAdjustPartSelect->textsize(16);
	a_CTWeaponDetails->textsize(10);
	aSys_flags->textsize(10);
	a_OStatusList->textsize(12);
	aGN_NameList1->textsize(14); aGN_NameList2->textsize(14); aGN_NameList3->textsize(14);
	aEdit_Notes->textsize(12);
	a_EText->textsize(16);
	aLF_Logfile->textsize(10);
	//a_CTCombatMods->textsize(11);

		// blank everything out just incase ..
	memset(&players[0].c_name[0], '\0', sizeof(ADMPplayer)*MAX_MEMBERS);
	memset(&qkmonsters[0].c_name[0], '\0', sizeof(ADMPplayer)*MAX_QKMONSTERS+2);
	loadAllData();

	sprintf(mybuf2, "%d", config.i_roundCount); aSys_Rounds->value(mybuf2);
	aSys_Bleeding->value(config.i_whenBleed);
	//aSys_XPAllocation->value(config.i_xpAllocate);
	//aSys_GroupInitiatives->value(config.i_doGroupInitiatives);
	//aSys_MultipleAttacks->value(config.i_attackGroup);
	//sprintf(mybuf2, "%d", config.); aSys_AttkSep1->value(config.|);
	//sprintf(mybuf2, "%d", config.); aSys_AttkSep2->value(config.|);
	//sprintf(mybuf2, "%d", config.); aSys_AttkSep3->value(config.|);
	//sprintf(mybuf2, "%d", config.); aSys_AttkSep4->value(config.|);
	//sprintf(mybuf2, "%d", config.); aSys_AttkSep5->value(config.|);
	//sprintf(mybuf2, "%d", config.); aSys_AttkSep6->value(config.|);
	sprintf(mybuf2, "%d", config.i_roundsSave); aSys_saveRounds->value(mybuf2);
	aSys_playerFilename->value(config.c_partyFilename);
	aSys_monsterFilename->value(config.c_monstFilename);
	sprintf(mybuf2, "%.2f", config.f_version); aSys_Version->value(mybuf2);

	for (i=0; i<MAX_MODIFIERS; i++) {
		if (config.system_mods[i].c_message[0] == '\0') { sprintf(config.system_mods[i].c_message, "-- empty slot %d", i); }
		a_OTAdjustSystemList->add(config.system_mods[i].c_message);
	}

	for (i=0; ; i++) {
		if (combatmods[i].c_message[0] == '\0') { break; }
		a_CTCombatMods->add(combatmods[i].c_message);
	}

	for (i=0; ; i++) {
		if (adjustments[i].title[0] == '\0') { break; }
		a_OTAdjustType->add(adjustments[i].title);
	}
	for (i=0; i<MAX_ADJEFFECTTYPES; i++) {
		a_OTAdjustEffectsType->add(adjusteffecttype[i]);
	}
	aEdit_Flags->clear();
	for (i=0; flagTypes[i] != NULL; i++) {
		aEdit_Flags->add(flagTypes[i]);
	}
	for (i=0; ; i++) {
		if (environment[i] == NULL) { break; }
		a_EdtQkMonCREnvironment->add(environment[i]);
	}
	a_EdtQkMonCREnvironment->value(0);

	a_OTTriggerList->clear();
	for (i=0; i<MAX_TRIGGERS; i++) {
		a_OTTriggerList->add(config.action_msgs[i].c_message);
	}
	a_OTAdjustList->clear();
	for (i=0; i<MAX_MODIFIERS; i++) {
		a_OTAdjustList->add(config.action_mods[i].c_message);
	}

	a_CBActionType->clear();
	for (i=0; i<11; i++) {
		a_CBActionType->add(actionTypes[i]);
	}

	if (config.flags.f_logfile != 0) {
		if (time(&t) != -1) {
			tp = localtime(&t);
			sprintf(mybuf1, "ADMP Started (%2.2d/%2.2d/%4.4d %2.2d:%2.2d) in %s\n", tp->tm_mday, tp->tm_mon+1, tp->tm_year+1900, tp->tm_hour, tp->tm_min, systemDir);
			writeLog("       ********************");
			writeLog(mybuf1);
		}
	}
	memset(&stdweapons[0].c_available, '\0', sizeof(ADMPweapon) * MAX_STDWEAPONS);

	for (i=0; i<14; i++) {
		aEdit_DamReductionType->add(dam_reduct_types[i]);
		aQkAdd_DamReductionType->add(dam_reduct_types[i]);
	}

	notes_buffer = new Fl_Text_Buffer();
	a_NotesArea->buffer(notes_buffer);

	edit_notes_buffer = new Fl_Text_Buffer();
	aEdit_Notes->buffer(edit_notes_buffer);

	detailed_buffer[0] = new Fl_Text_Buffer();
	detailed_buffer[1] = new Fl_Text_Buffer();
	a_EDInitiatorDetails2->buffer(detailed_buffer[0]);
	a_EDDefenderDetails2->buffer(detailed_buffer[1]);

	sprintf(mybuf2, "%snotes.txt", systemDir);
	notes_buffer->loadfile(mybuf2);

	sprintf(mybuf2, "%ssrd/index.html", systemDir);
	if (a_SRDText->load(mybuf2) == -1) { sprintf(mybuf2, "%ssrd/readme.txt", systemDir); a_SRDText->load(mybuf2); }

	sprintf(mybuf2, "%shelp.txt", systemDir);
	a_HelpList->load(mybuf2);

	doAutoDamageAction(0);

	if (strncasecmp(config.system_mods[0].c_message, "-- empty slot 0", 15) == 0) {
		loadStdModifiers();
		doAdjustmentAction(9);		// refresh
	}

	doInitiativeAction(2);
	doInitiativeAction(3);
	doOtherAction(20);		// update the status tab

	a_OTabs->value(a_OTStatus);

	setSystemFlag(0);

	prevHtmlPage[0][0] = '\0';

	doAreYouSure(NULL);

	a_OTabs->value(a_OTTriggers);

//	aEdit_Dump->activate();
//	a_OTAdjustDump->activate();

	rollRNDDice();
#ifdef VISUALC
	loadSRDspellEntries("srd\\spells");
#else
	loadSRDspellEntries("srd/spells");
#endif
}

/****************************************************************************/
void searchWeaponDetails(int pos, char type) {
	int a=0,i=0;

	if (stdweapons[0].c_description[0] == '\0') {
		if (config.f_version < 1.11) {
			sprintf(mybuf2, "%sweapon.dat", systemDir);
			if ((openf = fopen(mybuf2, "r")) != NULL) {
				fread(&stdweapons[0].c_available, sizeof(ADMPweapon) * MAX_STDWEAPONS, 1, openf);
				fclose(openf);
			} else {
				doSystemAlert("Unable to read the standard weapon file ... !!!");
				return;
			}
		} else {
			readAllStdWeaponData();
		}
	}
	a_WSearchList->clear();
	config.i_idList[ID_WSCH][a] = -1;
	if (type == 0) {
		for (i=0; i<MAX_STDWEAPONS; i++) {
			a_WSearchList->add(stdweapons[i].c_description);
			config.i_idList[ID_WSCH][a++] = i;
		}
	} else {
		for (i=56; i<MAX_STDWEAPONS; i++) {
			a_WSearchList->add(stdweapons[i].c_description);
			config.i_idList[ID_WSCH][a++] = i;
		}
		for (i=0; i<MAX_STDWEAPONS; i++) {
			if (stdweapons[i].c_attackType != type) {
				a_WSearchList->add(stdweapons[i].c_description);
				config.i_idList[ID_WSCH][a++] = i;
			}
		}
	}
	config.i_idList[ID_WSCH][a] = -1;
	i_WeaponUsed = pos;
	if (pos == 1) { i_prevWeaponEditted = -10; }
	if (pos == 2) { i_prevWeaponEditted = -11; }

	a_WeaponWindow->show();
}

/****************************************************************************/
void reloadNames(int type) {
	int i, j, k;
	char fmt[4];

	switch (type) {
		case 0:  aEdit_players->clear();
					aEdit_monsters->clear();
					aEdit_players->add("New Player");
					aEdit_monsters->add("New Monster");

					for (j=0; j<MAX_MEMBERS; j++) {
						config.i_idList[ID_EDIT][j] = -255;
					}
					config.i_idList[ID_EDIT][0] = -1;
					k=1;

					for (j=0; j<MAX_PLAYERS; j++) {
						if (players[j].c_name[0] != '\0' && players[j].i_hp[HP_MAX] > 0) {
							aEdit_players->add(players[j].c_name);
							config.i_idList[ID_EDIT][k++] = j;
						}
					}

					k = MAX_PLAYERS;
					config.i_idList[ID_EDIT][k++] = -2;
					for (j=MAX_PLAYERS; j<MAX_MONSTERS; j++) {
						if (players[j].c_name[0] != '\0' && players[j].i_hp[HP_MAX] > 0) {
							aEdit_monsters->add(players[j].c_name);
							config.i_idList[ID_EDIT][k++] = j;
						}
					}
					break;

		case 1:
					a_CBInitiators->clear();
#ifdef VISUALC
					for (i=0; i<MAX_MEMBERS; i++) {
#else
					for (int i=0; i<MAX_MEMBERS; i++) {
#endif
						if (config.i_idList[ID_INIT][i] > -1) {
							mybuf2[0] = '\0'; fmt[0] = '\0';
							j = config.i_idList[ID_INIT][i];
							k = getHPs(j);
								// set the various identifiers, color etc
							if (players[j].i_doneAttacks > 0) { strcpy(fmt, "@B88@."); }
							if (players[j].i_stun > 0) { strcat(mybuf2, "S"); strcpy(fmt, "@B95@."); }
							if (k <= players[j].i_unconciousAt) { strcat(mybuf2, "U"); strcpy(fmt, "@B95@."); }
							if (players[j].i_inactive > 0) { strcat(mybuf2, "I"); strcpy(fmt, "@B95@."); }
							if (getHPs(j) < -9) { strcat(mybuf2, "D"); }
							if (strlen(mybuf2) > 0) {
								sprintf(mybuf1, "%s%s (%d=%d%% *%s %d:%d)", fmt, players[j].c_name, k, getHPpercentage(j), mybuf2, players[j].i_noAttacks, players[j].i_doneAttacks);
							} else {
								sprintf(mybuf1, "%s%s (%d=%d%% %d:%d)", fmt, players[j].c_name, k, getHPpercentage(j), players[j].i_noAttacks, players[j].i_doneAttacks);
							}
							a_CBInitiators->add(mybuf1);
if (config.flags.f_debug != 0) { printf("rL: SL1 %d:%d (%s)\n", i, j, mybuf1); }
						}
					}
					a_CBInitiators->redraw();
					break;
		case 9:
					if (a_OTTriggerList->active() == 1) { doTriggerAction(9); }
					if (a_OTAdjustList->active() == 1) { doAdjustmentAction(9); }
					if (a_OTADList->active() == 1) { doAutoDamageAction(9); }
					break;
	}
}

/****************************************************************************/
void editPlayer(int pid) {
	char mybuf[20];
	int i=0, j=0;

	if (pid < 0 || pid > (MAX_MEMBERS+2) || config.i_idList[ID_EDIT][pid] == -255) {
		return;
	}

if (config.flags.f_debug != 0) { printf("eP 1: %d:%d\n", pid, config.i_idList[ID_EDIT][pid]); }

		// check to see if it's an ADD NEW request
	if (config.i_idList[ID_EDIT][pid] == -1) {
		  for (pid=0; pid<MAX_PLAYERS; pid++) {
				 if (players[pid].c_name[0] == '\0' && players[pid].i_hp[HP_MAX] == 0)
						break;
		  }	// did we find a empty slot
		 if (players[pid].c_name[0] != '\0' && players[pid].i_hp[HP_MAX] != 0)
				 return;
if (config.flags.f_debug != 0) { printf("adding new player %d\n", pid); }
		 		// set various defaults
		 players[pid].i_inGroup = -1;
		 players[pid].i_unconciousAt = -1;
		 aEdit_players->deactivate();
		 aEdit_monsters->deactivate();
		 players[pid].i_whomAttacked[0][0] = players[pid].i_whomAttacked[1][0] = players[pid].i_whomAttacked[2][0] = -1;
	} else if (config.i_idList[ID_EDIT][pid] == -2) {
		 for (pid=MAX_PLAYERS; pid<MAX_MONSTERS; pid++) {
				 if (players[pid].c_name[0] == '\0' && players[pid].i_hp[HP_MAX] == 0)
							break;
		 }	// did we find a empty slot
		 if (players[pid].c_name[0] != '\0' && players[pid].i_hp[HP_MAX] != 0)
				 return;
if (config.flags.f_debug != 0) { printf("adding new monster %d\n", pid); }
		 aEdit_SpecialActions->activate();
		  		// set various defaults
		 players[pid].i_inGroup = -1;
		 players[pid].i_unconciousAt = -1;
		 players[pid].flags.f_disabled = 0;
		 for (j=0; j<MAX_D20ROLLS; j++) {
			players[pid].i_d20Rolls[j] = getRND(20);
			players[pid].i_whomAttacked[j][0] = -1;
		 }
		 aEdit_players->deactivate();
		 aEdit_monsters->deactivate();
	} else {
		if (pid < MAX_MEMBERS) {
			reloadNames(0);
			if (pid >= MAX_PLAYERS) {
				sprintf(mybuf1, "@B63@.%s", aEdit_monsters->text(pid-MAX_PLAYERS+1));
				aEdit_monsters->text(pid-MAX_PLAYERS+1, mybuf1);
			} else {
				sprintf(mybuf1, "@B63@.%s", aEdit_players->text(pid+1));
				aEdit_players->text(pid+1, mybuf1);
			}
			pid = config.i_idList[ID_EDIT][pid];
		}
if (config.flags.f_debug != 0) { printf("eP: 2: %d\n", i); }
	}

	if (pid >= MAX_PLAYERS && pid <= MAX_MEMBERS) {
		aEdit_SpecialActions->activate();
	} else {
		aEdit_SpecialActions->deactivate();
	}

	aEdit_Name->value(players[pid].c_name);
	aEdit_Race->value(players[pid].i_race);
	aEdit_Alignment->value(players[pid].i_alignment);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	aEdit_Class1->value(players[pid].i_classes[0]);
	aEdit_Class2->value(players[pid].i_classes[1]);
	aEdit_Class3->value(players[pid].i_classes[2]);
	aEdit_Class4->value(players[pid].i_classes[3]);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_levels[0]); aEdit_Level1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_levels[1]); aEdit_Level2->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_levels[2]); aEdit_Level3->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_levels[3]); aEdit_Level4->value(mybuf);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_hp[HP_MAX]); aEdit_HPMax->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_hp[HP_CUR]); aEdit_HPCurrent->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_hp[HP_NL]); aEdit_NonLethal->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_hp[HP_TMP]); aEdit_TempHP->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bleeding); aEdit_Bleeding->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_regeneration); aEdit_Regeneration->value(mybuf);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_disabledAt); aEdit_Disabled->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_unconciousAt); aEdit_Unconcious->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_savingThrows[SAVE_FORTITUDE]); aEdit_SaveFort->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_savingThrows[SAVE_REFLEX]); aEdit_SaveReflex->value(mybuf);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_savingThrows[SAVE_WISDOM]); aEdit_SaveWisdom->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClass[AC_NORMAL]); aEdit_ACNormal->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClass[AC_TOUCH]); aEdit_ACTouch->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClass[AC_FLATFOOTED]); aEdit_ACFlatFoot->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClassSplit[AC_SECTION_DEX]); aEdit_ACSecDex->value(mybuf);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_armorClassSplit[AC_SECTION_SIZE]); aEdit_ACSecSize->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClassSplit[AC_SECTION_DEFLECTION]); aEdit_ACSecDeflection->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClassSplit[AC_SECTION_NATURAL]); aEdit_ACSecNatural->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClassSplit[AC_SECTION_ARMOR]); aEdit_ACSecArmor->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_armorClassSplit[AC_SECTION_SHIELD]); aEdit_ACSecShield->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_MELEE][0]); aEdit_BABMelee1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_RANGED][0]); aEdit_BABRanged1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_GRAPPLE][0]); aEdit_BABGrapple1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_MELEE][1]); aEdit_BABMelee2->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_RANGED][1]); aEdit_BABRanged2->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_GRAPPLE][1]); aEdit_BABGrapple2->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_MELEE][2]); aEdit_BABMelee3->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_RANGED][2]); aEdit_BABRanged3->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_bab[BAB_GRAPPLE][2]); aEdit_BABGrapple3->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_initiative[0]); aEdit_InitBonus->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_initiative[1]); aEdit_InitRoll->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_speed); aEdit_Speed->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_inactive); aEdit_Inactive->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_stun); aEdit_Stun->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_noAttacks); aEdit_NoAttacks->value(mybuf);

		// add weapon stuff
	aEdit_WeaponSelector->activate(); aEdit_WeaponSelector->clear();
	for (i=0; i<6; i++) {
		if (players[pid].w_weapons[i].c_description[0] != '\0') {
			aEdit_WeaponSelector->add(players[pid].w_weapons[i].c_description);
		} else {
			sprintf(mybuf2, "Melee Empty Slot #%d", i); aEdit_WeaponSelector->add(mybuf2);
		}
	}
	for (i=6; i<MAX_WEAPONS; i++) {
		if (players[pid].w_weapons[i].c_description[0] != '\0') {
			aEdit_WeaponSelector->add(players[pid].w_weapons[i].c_description);
		} else {
			sprintf(mybuf2, "Ranged Empty Slot #%d", i); aEdit_WeaponSelector->add(mybuf2);
		}
	}
	i_prevWeaponEditted = -1;
	aEdit_WeaponAvailable->deactivate();
	aEdit_WeaponName->deactivate();
	aEdit_WeaponSearch->deactivate();
	aEdit_WeaponAttkBonus1->deactivate();
	aEdit_WeaponAttkBonus2->deactivate();
	aEdit_WeaponAttkBonus3->deactivate();
	aEdit_WeaponAttkBonus4->deactivate();
	aEdit_WeaponDamage->deactivate();
	aEdit_WeaponCritical->deactivate();
	aEdit_WeaponMagicBonus->deactivate();
	aEdit_WeaponRange->deactivate();
	aEdit_WeaponReach->deactivate();
	aEdit_WeaponNoAttacks->deactivate();
	aEdit_WeaponSpecialMessage->deactivate();
	aEdit_WeaponType->deactivate();

	sprintf(mybuf, "%.1f", players[pid].f_challengeRating); aEdit_CRating->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[0]); aEdit_d201->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[1]); aEdit_d202->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[2]); aEdit_d203->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[3]); aEdit_d204->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[4]); aEdit_d205->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[5]); aEdit_d206->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_d20Rolls[6]); aEdit_d207->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[0]); aEdit_SpellDC0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[1]); aEdit_SpellDC1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[2]); aEdit_SpellDC2->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[3]); aEdit_SpellDC3->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[4]); aEdit_SpellDC4->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[5]); aEdit_SpellDC5->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_spellDC[6]); aEdit_SpellDC6->value(mybuf);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_spellFailure); aEdit_SpellFailure->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[0][0]); aEdit_StatsSTR0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[1][0]); aEdit_StatsSTR1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[0][1]); aEdit_StatsDEX0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[1][1]); aEdit_StatsDEX1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[0][2]); aEdit_StatsCON0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[1][2]); aEdit_StatsCON1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[0][3]); aEdit_StatsINT0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[1][3]); aEdit_StatsINT1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[0][4]); aEdit_StatsWIS0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[1][4]); aEdit_StatsWIS1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[0][5]); aEdit_StatsCHA0->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_abilityStats[1][5]); aEdit_StatsCHA1->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_SEARCH]); aEdit_SkillSearch->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_SOH]); aEdit_SkillSOH->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_SPOT]); aEdit_SkillSpot->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_OPENLOCKS]); aEdit_SkillOLocks->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_MOVESILENTLY]); aEdit_SkillMoveSilently->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_LISTEN]); aEdit_SkillListen->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_HIDE]); aEdit_SkillHide->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_skills[SKILL_CONCENTRATION]); aEdit_SkillConcentration->value(mybuf);
if (config.flags.f_debug != 0) { printf("eP: 3: %d\n", i); }
	sprintf(mybuf, "%d", players[pid].i_spellResistance); aEdit_SpellResistance->value(mybuf);
	aEdit_DamReductionType->value(players[pid].i_damageReduction[0]);
	sprintf(mybuf, "%d", players[pid].i_damageReduction[1]); aEdit_DamReductionHP->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_damageReduction[2]); aEdit_DamReductionWeapon->value(mybuf);
	aEdit_Size->value(players[pid].i_size);
	sprintf(mybuf, "%d", players[pid].i_xp[MAX]); aEdit_XPCurr->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_xp[TMP]); aEdit_XPTemp->value(mybuf);
	sprintf(mybuf, "%d", players[pid].i_inGroup); aEdit_InGroup->value(mybuf);
	aEdit_HitDice->value(players[pid].c_hitDice);

	aEdit_Flags->check_none();
	aEdit_Flags->checked(1, players[pid].flags.f_isStabilised);
	aEdit_Flags->checked(2, players[pid].flags.f_disabled);
	aEdit_Flags->checked(3, players[pid].flags.f_ignoreCriticals);
	aEdit_Flags->checked(4, players[pid].flags.f_autoStablise);
	aEdit_Flags->checked(5, players[pid].flags.f_evasion);
	aEdit_Flags->checked(6, players[pid].flags.f_npc);

	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel0); aEdit_SpellSlotMax0->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel1); aEdit_SpellSlotMax1->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel2); aEdit_SpellSlotMax2->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel3); aEdit_SpellSlotMax3->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel4); aEdit_SpellSlotMax4->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel5); aEdit_SpellSlotMax5->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel6); aEdit_SpellSlotMax6->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel7); aEdit_SpellSlotMax7->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel8); aEdit_SpellSlotMax8->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_maxLevel9); aEdit_SpellSlotMax9->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel0); aEdit_SpellSlotCur0->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel1); aEdit_SpellSlotCur1->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel2); aEdit_SpellSlotCur2->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel3); aEdit_SpellSlotCur3->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel4); aEdit_SpellSlotCur4->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel5); aEdit_SpellSlotCur5->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel6); aEdit_SpellSlotCur6->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel7); aEdit_SpellSlotCur7->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel8); aEdit_SpellSlotCur8->value(mybuf1);
	sprintf(mybuf1, "%d", players[pid].spellsAvailable.i_curLevel9); aEdit_SpellSlotCur9->value(mybuf1);

	config.i_editID = pid;

	if (config.flags.f_debug != 0) printf("Editting %d\n", pid);

//	aEdit_players->deactivate();
//	aEdit_monsters->deactivate();
	aEdt_Accept->activate();
	aEdt_Cancel->activate();
	aEdt_QkMonsterAdd->deactivate();
	aEdt_Tabs->activate();
	aEdt_Tabs->value(aEdt_Main);

//	sprintf(mybuf2, "%s%s.note", systemDir, players[pid].c_name);
//	edit_notes_buffer->loadfile(mybuf2);
	doNotesAction(1, pid);
}

/******************************************************************************/
void cancelEdit() {
	aEdit_players->activate();
	aEdit_monsters->activate();
	aEdt_Accept->deactivate();
	aEdt_QkMonsterAdd->activate();
	aEdt_Cancel->deactivate();
	aEdt_Tabs->deactivate();
	aEdit_SpecialActions->deactivate();
	a_EdtQkMonList->activate();
	a_EdtQkMonstSearch->activate();
	aEdit_AddQkMonsters->activate();
	aEdit_EditQkMonsters->activate();
	aEdit_SaveQkMonsters->activate();
	a_EdtQkMonCRSearch->activate();
	aEdit_QkMonstersWildSearch->activate();
	a_EdtQkMonAddCtr->activate();
	config.i_editID = -1;
	reloadNames(0);
}

/******************************************************************************/
void savePlayer(int pid) {
	char mybuf[40], ok=0;

	if (pid < 0 || pid > (MAX_MEMBERS+1)) {
		return;
	}

	strncpy(players[pid].c_name, aEdit_Name->value(), 29);
if (config.flags.f_debug != 0) printf("savePlayer %d = %s\n", pid, players[pid].c_name);
	if (pid != MAX_MEMBERS+1) {
		if (players[pid].c_name[2] == '-') {
			players[pid].c_name[0] = identifiers[pid];
		} else {
			sprintf(mybuf, "%c - %s", identifiers[pid], players[pid].c_name);
			strncpy(players[pid].c_name, mybuf, 29);
		}
	}
	players[pid].i_race = aEdit_Race->value();
	players[pid].i_alignment = aEdit_Alignment->value();
	players[pid].i_classes[0] = aEdit_Class1->value();
	players[pid].i_classes[1] = aEdit_Class2->value();
	players[pid].i_classes[2] = aEdit_Class3->value();
	players[pid].i_classes[3] = aEdit_Class4->value();
	players[pid].i_levels[0] = atoi(aEdit_Level1->value());
	players[pid].i_levels[1] = atoi(aEdit_Level2->value());
	players[pid].i_levels[2] = atoi(aEdit_Level3->value());
	players[pid].i_levels[3] = atoi(aEdit_Level4->value());
	players[pid].i_hp[HP_MAX] = atoi(aEdit_HPMax->value());
	players[pid].i_hp[HP_CUR] = atoi(aEdit_HPCurrent->value());
	players[pid].i_hp[HP_NL] = atoi(aEdit_NonLethal->value());
	players[pid].i_hp[HP_TMP] = atoi(aEdit_TempHP->value());
	players[pid].i_bleeding = atoi(aEdit_Bleeding->value());
	players[pid].i_disabledAt = atoi(aEdit_Disabled->value());
	players[pid].i_unconciousAt = atoi(aEdit_Unconcious->value());
	players[pid].i_regeneration = atoi(aEdit_Regeneration->value());
	players[pid].i_savingThrows[SAVE_FORTITUDE] = atoi(aEdit_SaveFort->value());
	players[pid].i_savingThrows[SAVE_REFLEX] = atoi(aEdit_SaveReflex->value());
	players[pid].i_savingThrows[SAVE_WISDOM] = atoi(aEdit_SaveWisdom->value());
	players[pid].i_armorClass[AC_NORMAL] = atoi(aEdit_ACNormal->value());
	players[pid].i_armorClass[AC_TOUCH] = atoi(aEdit_ACTouch->value());
	players[pid].i_armorClass[AC_FLATFOOTED] = atoi(aEdit_ACFlatFoot->value());
	players[pid].i_armorClassSplit[AC_SECTION_DEX] = atoi(aEdit_ACSecDex->value());
	players[pid].i_armorClassSplit[AC_SECTION_SIZE] = atoi(aEdit_ACSecSize->value());
	players[pid].i_armorClassSplit[AC_SECTION_DEFLECTION] = atoi(aEdit_ACSecDeflection->value());
	players[pid].i_armorClassSplit[AC_SECTION_NATURAL] = atoi(aEdit_ACSecNatural->value());
	players[pid].i_armorClassSplit[AC_SECTION_ARMOR] = atoi(aEdit_ACSecArmor->value());
	players[pid].i_armorClassSplit[AC_SECTION_SHIELD] = atoi(aEdit_ACSecShield->value());
	players[pid].i_bab[BAB_MELEE][0] = atoi(aEdit_BABMelee1->value());
	players[pid].i_bab[BAB_RANGED][0] = atoi(aEdit_BABRanged1->value());
	players[pid].i_bab[BAB_GRAPPLE][0] = atoi(aEdit_BABGrapple1->value());
	players[pid].i_bab[BAB_MELEE][1] = atoi(aEdit_BABMelee2->value());
	players[pid].i_bab[BAB_RANGED][1] = atoi(aEdit_BABRanged2->value());
	players[pid].i_bab[BAB_GRAPPLE][1] = atoi(aEdit_BABGrapple2->value());
	players[pid].i_bab[BAB_MELEE][2] = atoi(aEdit_BABMelee3->value());
	players[pid].i_bab[BAB_RANGED][2] = atoi(aEdit_BABRanged3->value());
	players[pid].i_bab[BAB_GRAPPLE][2] = atoi(aEdit_BABGrapple3->value());
	players[pid].i_initiative[0] = atoi(aEdit_InitBonus->value());
	players[pid].i_initiative[1] = atoi(aEdit_InitRoll->value());
	players[pid].i_speed = atoi(aEdit_Speed->value());
	players[pid].i_inactive = atoi(aEdit_Inactive->value());
	players[pid].i_stun = atoi(aEdit_Stun->value());
	players[pid].i_noAttacks = atoi(aEdit_NoAttacks->value());
	
	if (i_prevWeaponEditted != -1) {		// save the previous weapon details
		aEdit_WeaponAvailable->value(players[pid].w_weapons[i_prevWeaponEditted].c_available);
		strncpy(players[pid].w_weapons[i_prevWeaponEditted].c_description, aEdit_WeaponName->value(), 29);
		players[pid].w_weapons[i_prevWeaponEditted].i_attackBonus[0] = atoi(aEdit_WeaponAttkBonus1->value());
		players[pid].w_weapons[i_prevWeaponEditted].i_attackBonus[1] = atoi(aEdit_WeaponAttkBonus2->value());
		players[pid].w_weapons[i_prevWeaponEditted].i_attackBonus[2] = atoi(aEdit_WeaponAttkBonus3->value());
		players[pid].w_weapons[i_prevWeaponEditted].i_attackBonus[3] = atoi(aEdit_WeaponAttkBonus4->value());
		strncpy(players[pid].w_weapons[i_prevWeaponEditted].c_damage, aEdit_WeaponDamage->value(), 19);
		strncpy(players[pid].w_weapons[i_prevWeaponEditted].c_critical, aEdit_WeaponCritical->value(), 19);
		players[pid].w_weapons[i_prevWeaponEditted].i_magicalBonus = atoi(aEdit_WeaponMagicBonus->value());
		players[pid].w_weapons[i_prevWeaponEditted].i_range = atoi(aEdit_WeaponRange->value());
		players[pid].w_weapons[i_prevWeaponEditted].i_reach = atoi(aEdit_WeaponReach->value());
		players[pid].w_weapons[i_prevWeaponEditted].i_noAttacks = atoi(aEdit_WeaponNoAttacks->value());
		strncpy(players[pid].w_weapons[i_prevWeaponEditted].c_message, aEdit_WeaponSpecialMessage->value(), 29);
		players[pid].w_weapons[i_prevWeaponEditted].c_description[29] = '\0';
		players[pid].w_weapons[i_prevWeaponEditted].c_damage[19] = '\0';
		players[pid].w_weapons[i_prevWeaponEditted].c_critical[19] = '\0';
		players[pid].w_weapons[i_prevWeaponEditted].c_attackType = aEdit_WeaponType->value();
	}

	players[pid].f_challengeRating = (float )atof(aEdit_CRating->value());
	players[pid].i_d20Rolls[0] = atoi(aEdit_d201->value());
	players[pid].i_d20Rolls[1] = atoi(aEdit_d202->value());
	players[pid].i_d20Rolls[2] = atoi(aEdit_d203->value());
	players[pid].i_d20Rolls[3] = atoi(aEdit_d204->value());
	players[pid].i_d20Rolls[4] = atoi(aEdit_d205->value());
	players[pid].i_d20Rolls[5] = atoi(aEdit_d206->value());
	players[pid].i_d20Rolls[6] = atoi(aEdit_d207->value());
	players[pid].i_spellDC[0] = atoi(aEdit_SpellDC0->value());
	players[pid].i_spellDC[1] = atoi(aEdit_SpellDC1->value());
	players[pid].i_spellDC[2] = atoi(aEdit_SpellDC2->value());
	players[pid].i_spellDC[3] = atoi(aEdit_SpellDC3->value());
	players[pid].i_spellDC[4] = atoi(aEdit_SpellDC4->value());
	players[pid].i_spellDC[5] = atoi(aEdit_SpellDC5->value());
	players[pid].i_spellDC[6] = atoi(aEdit_SpellDC6->value());
	players[pid].i_spellFailure = atoi(aEdit_SpellFailure->value());
	players[pid].i_abilityStats[0][0] = atoi(aEdit_StatsSTR0->value());
	players[pid].i_abilityStats[1][0] = atoi(aEdit_StatsSTR1->value());
	players[pid].i_abilityStats[0][1] = atoi(aEdit_StatsDEX0->value());
	players[pid].i_abilityStats[1][1] = atoi(aEdit_StatsDEX1->value());
	players[pid].i_abilityStats[0][2] = atoi(aEdit_StatsCON0->value());
	players[pid].i_abilityStats[1][2] = atoi(aEdit_StatsCON1->value());
	players[pid].i_abilityStats[0][3] = atoi(aEdit_StatsINT0->value());
	players[pid].i_abilityStats[1][3] = atoi(aEdit_StatsINT1->value());
	players[pid].i_abilityStats[0][4] = atoi(aEdit_StatsWIS0->value());
	players[pid].i_abilityStats[1][4] = atoi(aEdit_StatsWIS1->value());
	players[pid].i_abilityStats[0][5] = atoi(aEdit_StatsCHA0->value());
	players[pid].i_abilityStats[1][5] = atoi(aEdit_StatsCHA1->value());
	players[pid].i_skills[SKILL_SEARCH] = atoi(aEdit_SkillSearch->value());
	players[pid].i_skills[SKILL_SOH] = atoi(aEdit_SkillSOH->value());
	players[pid].i_skills[SKILL_SPOT] = atoi(aEdit_SkillSpot->value());
	players[pid].i_skills[SKILL_OPENLOCKS] = atoi(aEdit_SkillOLocks->value());
	players[pid].i_skills[SKILL_MOVESILENTLY] = atoi(aEdit_SkillMoveSilently->value());
	players[pid].i_skills[SKILL_LISTEN] = atoi(aEdit_SkillListen->value());
	players[pid].i_skills[SKILL_HIDE] = atoi(aEdit_SkillHide->value());
	players[pid].i_skills[SKILL_CONCENTRATION] = atoi(aEdit_SkillConcentration->value());
	players[pid].i_spellResistance = atoi(aEdit_SpellResistance->value());
	players[pid].i_damageReduction[0] = aEdit_DamReductionType->value();
	players[pid].i_damageReduction[1] = atoi(aEdit_DamReductionHP->value());
	players[pid].i_damageReduction[2] = atoi(aEdit_DamReductionWeapon->value());
	players[pid].i_size = aEdit_Size->value();
	players[pid].i_xp[MAX] = atoi(aEdit_XPCurr->value());
	players[pid].i_xp[TMP] = atoi(aEdit_XPTemp->value());
	players[pid].i_inGroup = atoi(aEdit_InGroup->value());
	strncpy(players[pid].c_hitDice, aEdit_HitDice->value(), 19); players[pid].c_hitDice[19] = '\0';
	players[pid].flags.f_isStabilised = aEdit_Flags->checked(1);
	players[pid].flags.f_disabled = aEdit_Flags->checked(2);
	players[pid].flags.f_ignoreCriticals = aEdit_Flags->checked(3);
	players[pid].flags.f_autoStablise = aEdit_Flags->checked(4);
	players[pid].flags.f_evasion = aEdit_Flags->checked(5);
	players[pid].flags.f_npc = aEdit_Flags->checked(6);

	players[pid].spellsAvailable.i_maxLevel0 = atoi(aEdit_SpellSlotMax0->value());
	players[pid].spellsAvailable.i_maxLevel1 = atoi(aEdit_SpellSlotMax1->value());
	players[pid].spellsAvailable.i_maxLevel2 = atoi(aEdit_SpellSlotMax2->value());
	players[pid].spellsAvailable.i_maxLevel3 = atoi(aEdit_SpellSlotMax3->value());
	players[pid].spellsAvailable.i_maxLevel4 = atoi(aEdit_SpellSlotMax4->value());
	players[pid].spellsAvailable.i_maxLevel5 = atoi(aEdit_SpellSlotMax5->value());
	players[pid].spellsAvailable.i_maxLevel6 = atoi(aEdit_SpellSlotMax6->value());
	players[pid].spellsAvailable.i_maxLevel7 = atoi(aEdit_SpellSlotMax7->value());
	players[pid].spellsAvailable.i_maxLevel8 = atoi(aEdit_SpellSlotMax8->value());
	players[pid].spellsAvailable.i_maxLevel9 = atoi(aEdit_SpellSlotMax9->value());
	players[pid].spellsAvailable.i_curLevel0 = atoi(aEdit_SpellSlotCur0->value());
	players[pid].spellsAvailable.i_curLevel1 = atoi(aEdit_SpellSlotCur1->value());
	players[pid].spellsAvailable.i_curLevel2 = atoi(aEdit_SpellSlotCur2->value());
	players[pid].spellsAvailable.i_curLevel3 = atoi(aEdit_SpellSlotCur3->value());
	players[pid].spellsAvailable.i_curLevel4 = atoi(aEdit_SpellSlotCur4->value());
	players[pid].spellsAvailable.i_curLevel5 = atoi(aEdit_SpellSlotCur5->value());
	players[pid].spellsAvailable.i_curLevel6 = atoi(aEdit_SpellSlotCur6->value());
	players[pid].spellsAvailable.i_curLevel7 = atoi(aEdit_SpellSlotCur7->value());
	players[pid].spellsAvailable.i_curLevel8 = atoi(aEdit_SpellSlotCur8->value());
	players[pid].spellsAvailable.i_curLevel9 = atoi(aEdit_SpellSlotCur9->value());

//printf("%s %d\n", mybuf2, edit_notes_buffer->length());
	if (edit_notes_buffer->length() > 0) {
//		sprintf(mybuf2, "%s%s.note", systemDir, players[pid].c_name);
//		edit_notes_buffer->savefile(mybuf2);
		doNotesAction(0, pid);
	}

	reloadNames(0);

	aEdit_players->activate();
	aEdit_monsters->activate();
	aEdt_Accept->deactivate();
	aEdt_Cancel->deactivate();
	aEdt_QkMonsterAdd->activate();
	aEdt_Tabs->deactivate();
	aEdit_SpecialActions->deactivate();
	a_EdtQkMonList->activate();
	a_EdtQkMonstSearch->activate();
	aEdit_AddQkMonsters->activate();
	aEdit_EditQkMonsters->activate();
	aEdit_SaveQkMonsters->activate();
	a_EdtQkMonCRSearch->activate();
	aEdit_QkMonstersWildSearch->activate();
	a_EdtQkMonAddCtr->activate();
		// TODO - why this bit & do we need it
//	if (pid == MAX_MEMBERS+1) {
//		actionQkMonsters(2);
//	}

	// do some quick sanity checks
	if (a_EORMessageList == NULL) { doEndOfRoundMessages(0); }
	a_EORMessageList->clear();
	a_EORMessageList->add("");
	a_EORMessageList->add("@B63@.The following are possible problems with this player/monster");
	a_EORMessageList->add("");
	if (players[pid].i_hp[HP_MAX] == 0 || players[pid].i_hp[HP_CUR] == 0) { ok = 1; a_EORMessageList->add("  HP's are zero"); }
	if (players[pid].i_armorClass[AC_NORMAL] == 0) { ok = 1; a_EORMessageList->add("  A/C is zero"); }
	if (players[pid].i_savingThrows[SAVE_FORTITUDE] == 0 && players[pid].i_savingThrows[SAVE_REFLEX] == 0 && players[pid].i_savingThrows[SAVE_WISDOM] == 0) { ok = 1; a_EORMessageList->add("  Saving Throws have issues"); }
	if (players[pid].i_noAttacks == 0) { ok = 1; a_EORMessageList->add("  #Attks is zero"); }
	if (players[pid].w_weapons[0].i_attackBonus[0] == 0 && players[pid].w_weapons[1].i_attackBonus[0] == 0 &&
players[pid].w_weapons[2].i_attackBonus[0] == 0 && players[pid].w_weapons[3].i_attackBonus[0] == 0) { ok = 1; a_EORMessageList->add("  Weapon BAB's dont seem to be set"); }
	if (players[pid].f_challengeRating == 0 && pid >= MAX_PLAYERS) { ok = 1; a_EORMessageList->add("  CR value is zero"); }

	if (ok == 1) {
		doEndOfRoundMessages(0);
	}

}

/******************************************************************************/
void setCombatDisplay(int flag) {
	int i,j,k;
	char *cp;

if (config.flags.f_debug != 0) { printf("sCD: %d\n", flag); }
	switch (flag) {
		case 0: a_CBInitiators->activate();
				  a_CBActionType->deactivate();
				  a_CTActions->deactivate();
				  a_CTMelee->deactivate();
				  a_CTSpell->deactivate();
				  a_CTOther->deactivate();
				  a_CTCancel->deactivate();
				  a_CBActionType->clear();
				  for (i=0; i<11; i++) {
			  			a_CBActionType->add(actionTypes[i]);
				  }
				  break;

		case 1: a_CBInitiators->activate();
				  a_CBActionType->activate();
				  a_CBActionType->clear();
				  for (i=0; i<11; i++) {
			  			a_CBActionType->add(actionTypes[i]);
				  }
//				  a_CTActions->deactivate();
				  a_CTActions->activate();
				  a_CTMelee->deactivate();
				  a_CTSpell->deactivate();
				  a_CTOther->deactivate();
				  a_CTMonsterDetails->activate();
				  a_CTSpellDetails->activate();
				  a_CTNotes->activate();
				  a_CTActions->value(a_CTMelee);
				  a_CTCancel->deactivate();
				  a_CTMessage->clear();
				  a_CTMessage->add(""); a_CTMessage->add(""); a_CTMessage->add(""); a_CTMessage->add(""); a_CTMessage->add(""); a_CTMessage->add(""); a_CTMessage->add(""); a_CTMessage->add("");
//				  for (i=0; i<MAX_MEMBERS; i++) { printf("sCD: 1: %d = %d\n", i, config.i_idList[ID_INIT][i]); }
				  config.i_actionInitID = config.i_idList[ID_INIT][a_CBInitiators->value()-1];
				  config.i_actionRecvID = -1;
if (config.flags.f_debug != 0) { printf("sCD: 1: %d:%d:%d\n", flag, config.i_actionInitID, a_CBInitiators->value()-1); }
				  a_CTACBreakdown->value("");
				  a_CTInitiatorDetails->clear(); a_EDInitiatorDetails1->clear();
				  a_CTDefenderDetails->clear(); a_EDDefenderDetails1->clear();
				  sprintf(mybuf2, "Name: %s", players[config.i_actionInitID].c_name); a_CTInitiatorDetails->add(mybuf2); a_EDInitiatorDetails1->add(mybuf2);
				  sprintf(mybuf2, "Speed: %d", players[config.i_actionInitID].i_speed); a_CTInitiatorDetails->add(mybuf2); a_EDInitiatorDetails1->add(mybuf2);
				  if (players[config.i_actionInitID].spellsAvailable.i_maxLevel0 > 0) {
				  		sprintf(mybuf2, "@b@.Spells Left: %d/%d/%d/%d/%d/%d/%d/%d/%d/%d", players[config.i_actionInitID].spellsAvailable.i_curLevel0, players[config.i_actionInitID].spellsAvailable.i_curLevel1, players[config.i_actionInitID].spellsAvailable.i_curLevel2, 
						players[config.i_actionInitID].spellsAvailable.i_curLevel3, players[config.i_actionInitID].spellsAvailable.i_curLevel4, players[config.i_actionInitID].spellsAvailable.i_curLevel5, players[config.i_actionInitID].spellsAvailable.i_curLevel6, 
						players[config.i_actionInitID].spellsAvailable.i_curLevel7, players[config.i_actionInitID].spellsAvailable.i_curLevel8, players[config.i_actionInitID].spellsAvailable.i_curLevel9); 
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel0 > 0) { j = 0; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel1 > 0) { j = 1; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel2 > 0) { j = 2; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel3 > 0) { j = 3; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel4 > 0) { j = 4; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel5 > 0) { j = 5; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel6 > 0) { j = 6; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel7 > 0) { j = 7; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel8 > 0) { j = 8; }
						if (players[config.i_actionInitID].spellsAvailable.i_maxLevel9 > 0) { j = 9; }
						cp = strstr(mybuf2, "/"); j--;
						while (j >= 0) {
							cp = strstr(cp, "/");
							cp++;
							j--;
				  		}
						if (cp != NULL) { cp++; *cp = '\0'; }
						a_CTInitiatorDetails->add(mybuf2); a_EDInitiatorDetails1->add(mybuf2);
				  }
				  sprintf(mybuf2, "   %d", (players[config.i_actionInitID].i_noAttacks - players[config.i_actionInitID].i_doneAttacks)); a_CTAttacksLeft->value(mybuf2);
				  sprintf(mybuf2, "%s%s.note", systemDir, players[config.i_actionInitID].c_name); detailed_buffer[0]->loadfile(mybuf2);
				  if (players[config.i_actionInitID].i_stun > 0) {
						sprintf(mybuf2, "@B95@.!!! They are stunned for %d rounds.... !!!", players[config.i_actionInitID].i_stun);
						a_CTMessage->text(1, mybuf2);
				  }		// mark the selected one as WHITE, makes them easier to find
				  j = a_CBInitiators->value();
				  reloadNames(1);
				  sprintf(mybuf2, "@B255@.%s (%d=%d%% %d:%d)", players[config.i_actionInitID].c_name, getHPs(config.i_actionInitID), getHPpercentage(config.i_actionInitID), players[config.i_actionInitID].i_noAttacks, players[config.i_actionInitID].i_doneAttacks);
				  a_CBInitiators->text(j, mybuf2);
				  config.i_actionWeaponSelected = -1;
				  a_CTMonsterDetails->deactivate();
				  if (config.i_actionInitID >= MAX_PLAYERS) {
				  		a_CTMonsterDetails->activate();
				  		a_CTActions->value(a_CTMonsterDetails);
						loadMonsterDescription(players[config.i_actionInitID].c_name);
				  }
				  doNotesAction(2, config.i_actionInitID);

				  		// display the weapon details
				  a_CTWeaponUsed->clear();
				  for (j=0; j<MAX_WEAPONS; j++) {
						if (players[config.i_actionInitID].w_weapons[j].c_available == 1 || players[config.i_actionInitID].w_weapons[j].c_description[0] != '\0') {
							sprintf(mybuf1, "%s (%d/%d)", players[config.i_actionInitID].w_weapons[j].c_description, players[config.i_actionInitID].w_weapons[j].i_noAttacks, players[config.i_actionInitID].w_weapons[j].i_doneAttacks);
							a_CTWeaponUsed->add(mybuf1);
						}
				  }
				  break;

		case 2: a_CBInitiators->deactivate();
				  a_CBActionType->deactivate();
				  a_CTActions->activate();
				  a_CTMelee->deactivate(); // a_CTallAttacksDone->deactivate();
				  a_CTSpell->deactivate(); a_CTSpellAllDone->deactivate();
				  a_CTOther->deactivate();
				  a_CTCancel->activate();
				  a_CTACBreakdown->value("");
				  a_CTMeleeACAdjustments->value("0");
				  i = a_CBActionType->value() - 1;
				  config.i_actionSelected = a_CBActionType->value() - 1;
				  sprintf(mybuf2, "@B32@.%s", actionTypes[config.i_actionSelected]);
				  a_CBActionType->text(i+1, mybuf2);
				  a_CTOtherFlags->check_none(); a_CTOtherFlags->checked(1,1); a_CTOtherFlags->checked(2,1);

				  		// should they bleed now
				  if (config.i_whenBleed == 0 && players[config.i_actionInitID].i_doneAttacks == 0) {
						if ((players[config.i_actionInitID].i_hp[HP_CUR] <= players[config.i_actionInitID].i_disabledAt && players[config.i_actionInitID].flags.f_isStabilised == 0) || players[config.i_actionInitID].i_bleeding != 0) {
							if (getRND(100) < 11 && (i >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0)) {		// did they stabilise ?
								players[i].flags.f_isStabilised = 1;
								sprintf(mybuf2, "Bleed: %s (HP:0 Sb:1) Cur:%d", players[config.i_actionInitID].c_name, players[config.i_actionInitID].i_hp[HP_CUR]);
							} else {
								AYSButton[2]->activate();
								AYSButton[2]->label("Stabilised");
								sprintf(mybuf2, "They are bleeding @ %d/rnd or are disabled.\n Shall I remove HP's now ?", players[config.i_actionInitID].i_bleeding);
								j = doAreYouSure(mybuf2);
								a_CTOtherFlags->checked(2,0);		// uncheck the Remove from Spell Spot option
								AYSButton[2]->label("Maybe");
								AYSButton[2]->deactivate();
								if (j == 1) {
									if (players[config.i_actionInitID].i_bleeding != 0) {
										players[config.i_actionInitID].i_hp[HP_CUR] -= players[config.i_actionInitID].i_bleeding;
										sprintf(mybuf2, "Bleed: %s (HP:%d) Cur:%d", players[config.i_actionInitID].c_name, players[config.i_actionInitID].i_bleeding, players[config.i_actionInitID].i_hp[HP_CUR]);
										writeLog(mybuf2);
									} else {
										players[config.i_actionInitID].i_hp[HP_CUR] --;
										sprintf(mybuf2, "Bleed: %s (HP:1) Cur:%d", players[config.i_actionInitID].c_name, players[config.i_actionInitID].i_hp[HP_CUR]);
										writeLog(mybuf2);
									}
									players[config.i_actionInitID].i_doneAttacks = 1;
								} else if (j == -2) {
									players[config.i_actionInitID].flags.f_isStabilised = 1;
									players[config.i_actionInitID].i_doneAttacks = 1;
									sprintf(mybuf2, "Bleed: %s (HP:0 Sb:1) Cur:%d", players[config.i_actionInitID].c_name, players[config.i_actionInitID].i_hp[HP_CUR]);
									writeLog(mybuf2);
								}
							}
						}
				  }
				  	// are there any following adjustments that finish now
				  if (a_EORMessageList == NULL) { doEndOfRoundMessages(0); }
				  a_EORMessageList->clear();
				  a_EORMessageList->add("Modifiers that finish now:");
				  k=0;
				  for (j=0; j<MAX_MODIFIERS; j++) {
						if (config.action_mods[j].i_noRounds == 1 && config.action_mods[j].i_idFollows == config.i_actionInitID) {
				  			a_EORMessageList->add(config.action_mods[j].c_message);
				  			sprintf(mybuf1, "ADjust: %s", config.action_mods[j].c_message); writeLog(mybuf1);
							sprintf(config.action_mods[j].c_message, "-- empty slot %d", i);
							config.action_mods[j].i_noRounds = 0;
							config.action_mods[j].i_whom[0] = config.action_mods[j].i_whom[1] = config.action_mods[j].i_idFollows = -255;
							config.action_mods[j].i_data[0][0] = config.action_mods[j].i_data[0][2] = 0; config.action_mods[j].i_data[0][1] = -1;
							config.action_mods[j].i_data[1][0] = config.action_mods[j].i_data[1][2] = 0; config.action_mods[j].i_data[1][1] = -1;
							config.action_mods[j].i_data[2][0] = config.action_mods[j].i_data[2][2] = 0; config.action_mods[j].i_data[2][1] = -1;
							config.action_mods[j].i_data[3][0] = config.action_mods[j].i_data[3][2] = 0; config.action_mods[j].i_data[3][1] = -1;
							config.action_mods[j].i_data[4][0] = config.action_mods[j].i_data[4][2] = 0; config.action_mods[j].i_data[4][1] = -1;
							k=1;
						}
				  }
				  if (k == 1) {
						doEndOfRoundMessages(0);
				  }
if (config.flags.f_debug != 0) { printf("sCD: 2: %d:%d\n", flag, i); }
						// check for any auto-damage that should be applied
				  for (i=0; i<MAX_AUTODAMAGE; i++) {
						if (config.auto_damage[i].i_noRounds > 0 && config.auto_damage[i].flags.f_flag1 != 0 && config.auto_damage[i].i_initiatorID == config.i_actionInitID && config.auto_damage[i].flags.f_flag2 == 0) {
								calcAutoDamage(i, -1);
								config.auto_damage[i].flags.f_flag2 = 1;
						}
				  }
						// now do all the display configuration, clearing of fields etc
				  sprintf(mybuf2, "%d", getAdjustment(config.i_actionInitID, MOD_BAB)); a_CTMeleeBABAdjustments->value(mybuf2);
				  sprintf(mybuf2, "%d", getAdjustment(config.i_actionInitID, MOD_DAMAGE)); a_CTMeleeDmgAdjustments->value(mybuf2); a_CTSpellDmgMods->value(mybuf2);
				  if (config.i_actionSelected == 0 || config.i_actionSelected == 1) {
						a_CTMelee->activate(); a_CTActions->value(a_CTMelee);
						a_CTWeaponUsed->clear();
						a_CTWeaponDetails->clear();
						a_CTCombatMods->check_none();
						a_CTSpecialAttacks->check_none();
						k=0;
						for (j=0; j<MAX_WEAPONS; j++) {
							if (players[config.i_actionInitID].w_weapons[j].c_available == 1 || players[config.i_actionInitID].w_weapons[j].c_description[0] != '\0') {
								sprintf(mybuf1, "%s (%d/%d)", players[config.i_actionInitID].w_weapons[j].c_description, players[config.i_actionInitID].w_weapons[j].i_noAttacks, players[config.i_actionInitID].w_weapons[j].i_doneAttacks);
								a_CTWeaponUsed->add(mybuf1);
								config.i_idList[ID_WEAP][k++] = j;
							}
							config.i_idList[ID_WEAP][k] = -1;
						}
				   } else if (config.i_actionSelected > 7) { 		// other type action: healing/turning etc
						a_CTOtherHealHP->deactivate(); a_CTOtherHealBleeding->deactivate(); 
//						a_CTOtherStabilise->deactivate(); 
						a_CTOtherHealSkillMods->deactivate(); a_CTOtherSpellLevel->deactivate(); a_CTOtherFlags->deactivate();

						if (config.i_actionSelected != 9) {		// a turning could mark them as Inactive/Stunned
							a_CTOtherHealInactive->deactivate(); a_CTOtherHealStun->deactivate();
						}

						a_CTOther->activate(); a_CTActions->value(a_CTOther); 
						a_CTOtherActions->clear();
						a_CTOtherActions->check_none();
						a_CTOtherMessages->clear();
				  		a_CTOtherHealSkillMods->value("");
						if (config.i_actionSelected == 9) {
							a_CTOtherMessages->add("Tuning Range: 60 feet, line of sight");
							a_CTOtherMessages->add("@bRoll 1d20 + Cha mod; table below shows highest HD you can turn");
							a_CTOtherMessages->add("00-00 Cleric level - 4");
							a_CTOtherMessages->add("01-03 Cleric level - 3");
							a_CTOtherMessages->add("04-06 Cleric level - 2");
							a_CTOtherMessages->add("07-09 Cleric level - 1");
							a_CTOtherMessages->add("10-12 Cleric level");
							a_CTOtherMessages->add("13-15 Cleric level + 1");
							a_CTOtherMessages->add("16-18 Cleric level + 2");
							a_CTOtherMessages->add("19-21 Cleric level + 3");
							a_CTOtherMessages->add("22+ Cleric level + 4");
							a_CTOtherMessages->add("@bRoll 2d6 + Cleric level + Cha mod, gives number of HD turned/rebuked");
							a_CTOtherMessages->add("Undead with 1/2 HD of cleric's level are destroyed/commanded");
							a_CTOtherMessages->add("Turned undead flee for 10 rounds, if they can't, they cower");
							a_CTOtherMessages->add("Rebuked undead cower in awe for 10 rounds");
							a_CTOtherMessages->add("If cleric gets closer than 10', turned undead will return to normal");
							a_CTOtherMessages->add("Commanded undead may be ordered as a standard action");
						} else if (config.i_actionSelected == 10) {
							sprintf(mybuf2, "@bd20 Rolls: %2.2d %2.2d %2.2d %2.2d %2.2d %2.2d %2.2d", players[config.i_actionInitID].i_d20Rolls[0], players[config.i_actionInitID].i_d20Rolls[1], players[config.i_actionInitID].i_d20Rolls[2], players[config.i_actionInitID].i_d20Rolls[3], players[config.i_actionInitID].i_d20Rolls[4], players[config.i_actionInitID].i_d20Rolls[5], players[config.i_actionInitID].i_d20Rolls[6]);
							a_CTOtherMessages->add(mybuf2);
							a_CTOtherMessages->add("");
							sprintf(mybuf2, "Spot: %d",players[config.i_actionInitID].i_skills[SKILL_SPOT]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Search: %d", players[config.i_actionInitID].i_skills[SKILL_SEARCH]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Open Locks: %d",players[config.i_actionInitID].i_skills[SKILL_OPENLOCKS]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Move Silently: %d",players[config.i_actionInitID].i_skills[SKILL_MOVESILENTLY]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Listen: %d",players[config.i_actionInitID].i_skills[SKILL_LISTEN]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Hide: %d",players[config.i_actionInitID].i_skills[SKILL_HIDE]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Concentration: %d",players[config.i_actionInitID].i_skills[SKILL_CONCENTRATION]); a_CTOtherMessages->add(mybuf2);
							sprintf(mybuf2, "Sleight of Hand: %d",players[config.i_actionInitID].i_skills[SKILL_SOH]); a_CTOtherMessages->add(mybuf2);
							a_CTOtherHealSkillMods->activate();
				  			sprintf(mybuf2, "%d", getAdjustment(config.i_actionInitID, MOD_SKILL)); a_CTOtherHealSkillMods->value(mybuf2);
						} else {
							a_CTOtherHealHP->activate(); a_CTOtherHealBleeding->activate(); a_CTOtherHealStun->activate();
//							a_CTOtherStabilise->activate(); 
							a_CTOtherHealInactive->activate(); a_CTOtherFlags->activate(); a_CTOtherSpellLevel->activate();
						}
				   } else { 		// a spell action
						a_CTSpell->activate(); a_CTActions->value(a_CTSpell); 
						a_CTSpellEffects->check_none();
						a_CTSpellModifiers->check_none();
						a_CTSpellSaved->check_none();
						a_CTSpellModifiers->check_none(); a_CTSpellModifiers->checked(1,1); // assume SAVE = 50%
						a_CTSpellModifiers->checked(4,1); a_CTSpellModifiers->checked(5,1);
						sprintf(mybuf2, "%d", players[config.i_actionInitID].i_spellDC[0]);
						a_CTSpellDC->value(mybuf2);
						a_CTSpellLevel->value(1.0);
						if (players[config.i_actionInitID].i_spellFailure > 0) {
							if (config.i_actionInitID < MAX_PLAYERS) {
								sprintf(mybuf2, "!!! The Initiator selected has a Spell Failure of %d%% !!!", players[config.i_actionInitID].i_spellFailure);
							} else {
								if (getRND(100) <= players[config.i_actionInitID].i_spellFailure) {
									sprintf(mybuf2, "!!! The Initiator selected has a Spell Failure of %d%% and has FAILED !!!", players[config.i_actionInitID].i_spellFailure);
								} else {
									sprintf(mybuf2, "!!! The Initiator selected has a Spell Failure of %d%% and has SUCCEEDED !!!", players[config.i_actionInitID].i_spellFailure);
								}
							}
							a_CTMessage->text(2, mybuf2);
						}
				   }		// fill in all the check lists
					j=0;
					a_CTSpellEffects->clear();
					a_CTSpellSaved->clear();
					a_CTDefender->clear();
					a_CTOtherActions->clear();
					if (config.i_actionInitID < MAX_PLAYERS) {
						for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
							if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
								sprintf(mybuf1, "%s (%d=%d%%)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i));
								a_CTDefender->add(mybuf1);
								a_CTOtherActions->add(mybuf1);
								if (players[i].i_spellResistance > 0) {
									sprintf(mybuf1, "%s (%d=%d%% SR:%d)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i), players[i].i_spellResistance);
								}
								a_CTSpellEffects->add(mybuf1);
								a_CTSpellSaved->add(mybuf1);
								config.i_idList[ID_DEFD][j++] = i;
							}
						}
						for (i=0; i<MAX_PLAYERS; i++) {
							if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
								sprintf(mybuf1, "%s (%d=%d%%)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i));
								a_CTDefender->add(mybuf1);
								a_CTOtherActions->add(mybuf1);
								if (players[i].i_spellResistance > 0) {
									sprintf(mybuf1, "%s (%d=%d%% SR:%d)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i), players[i].i_spellResistance);
								}
								a_CTSpellEffects->add(mybuf1);
								a_CTSpellSaved->add(mybuf1);
								config.i_idList[ID_DEFD][j++] = i;
							}
						}
					} else {
						for (i=0; i<MAX_MEMBERS; i++) {
							if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
								sprintf(mybuf1, "%s (%d=%d%%)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i));
								a_CTDefender->add(mybuf1);
								a_CTOtherActions->add(mybuf1);
								if (players[i].i_spellResistance > 0) {
									sprintf(mybuf1, "%s (%d=%d%% SR:%d)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i), players[i].i_spellResistance);
								}
								a_CTSpellEffects->add(mybuf1);
								a_CTSpellSaved->add(mybuf1);
								config.i_idList[ID_DEFD][j++] = i;
							}
						}
					}
					for (; j < MAX_MEMBERS; j++) {
						config.i_idList[ID_DEFD][j] = -1;
					}
					a_CTMeleeAC->value(""); a_CTMeleeBAB->value(""); a_CTMeleeRoll->value(""); a_CTMeleeDamage->value("");
					a_CTMeleeInactive->value(""); a_CTMeleeBleeding->value(""); a_CTMeleeStun->value(""); a_CTMeleeInactive->value("");
					if (config.i_actionInitID >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0) {
						i = atoi(a_CTMeleeRoll->value());
						sprintf(mybuf1, "%d", getRND(20)); a_CTMeleeRoll->value(mybuf1);
							// just make sure the attack roll aint the same as the previous
						if (i == atoi(a_CTMeleeRoll->value())) {
							sprintf(mybuf1, "%d", getRND(20)); a_CTMeleeRoll->value(mybuf1);
						}
					}

					a_CTSpellDamage->value(""); a_CTSpellStunned->value(""); a_CTSpellBleeding->value("");
					a_CTSpellInactive->value("");

					a_CTOtherHealHP->value(""); a_CTOtherHealBleeding->value(""); a_CTOtherHealStun->value("");
//					a_CTOtherStabilise->value(1); 
					a_CTOtherSpellLevel->value(1); a_CTOtherHealInactive->value("");
					if (config.i_actionSelected == 6) {				// spell Touch attack
						a_CTSpellDC->label("Attack Roll");
						sprintf(mybuf2, "%d", (players[config.i_actionInitID].i_bab[BAB_MELEE][0] + getAdjustment(config.i_actionInitID, MOD_BAB))); a_CTSpellMods->value(mybuf2);
						a_CTSpellLevel->deactivate();
						a_CTSpellUSA->deactivate();
						a_CTSpellSAP->deactivate();
						a_CTSpellSAM->deactivate();
						if (config.flags.f_rollAllRolls != 0 || config.i_actionInitID >= MAX_PLAYERS) {
							sprintf(mybuf2, "%d", getRND(20)); a_CTSpellDC->value(mybuf2);
						} else {
							a_CTSpellDC->value("");
						}
					} else if (config.i_actionSelected == 7) {	// spell Ranged Touch attack
						a_CTSpellDC->label("Attack Roll");
						sprintf(mybuf2, "%d", (players[config.i_actionInitID].i_bab[BAB_RANGED][0] + getAdjustment(config.i_actionInitID, MOD_BAB))); a_CTSpellMods->value(mybuf2);
						a_CTSpellLevel->deactivate();
						a_CTSpellUSA->deactivate();
						a_CTSpellSAP->deactivate();
						a_CTSpellSAM->deactivate();
						if (config.flags.f_rollAllRolls != 0 || config.i_actionInitID >= MAX_PLAYERS) {
							sprintf(mybuf2, "%d", getRND(20)); a_CTSpellDC->value(mybuf2);
						} else {
							a_CTSpellDC->value("");
						}
				   } else if (config.i_actionSelected == 0 || config.i_actionSelected == 1) {
							// preset the defender & weapon details to that of the previous attack
						i = config.i_actionRecvID = players[config.i_actionInitID].i_whomAttacked[players[config.i_actionInitID].i_doneAttacks][0];
						j = config.i_actionWeaponSelected = players[config.i_actionInitID].i_whomAttacked[players[config.i_actionInitID].i_doneAttacks][1];
						if (i != -1) {
							for (k=0; config.i_idList[ID_WEAP][k] != -1; k++) {
								if (config.i_idList[ID_WEAP][k] == j) {
									sprintf(mybuf2, "@B255@.%s (%d/%d)", players[config.i_actionInitID].w_weapons[j].c_description, players[config.i_actionInitID].w_weapons[j].i_noAttacks, players[config.i_actionInitID].w_weapons[j].i_doneAttacks);
									a_CTWeaponUsed->text(k+1, mybuf2);
									break;
								}
							}
							for (k=0; config.i_idList[ID_DEFD][k] != -1; k++) {
								if (config.i_idList[ID_DEFD][k] == i) {
									sprintf(mybuf2, "@B255@.%s (%d=%d%%)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i));
									a_CTDefender->text(k+1, mybuf2);
									break;
								}
							}
							setCombatDisplay(7);		// set the weapon details
							setCombatDisplay(8);		// set the defender details
							if (config.flags.f_rollAllRolls != 0 || config.i_actionInitID >= MAX_PLAYERS) {
								calcAction(0);			// calc the attack results
							}
						}
					} else {
						a_CTSpellDC->label("Spell DC");
						a_CTSpellMods->value("0");
						a_CTSpellLevel->activate();
						a_CTSpellUSA->activate();
						a_CTSpellSAP->activate();
						a_CTSpellSAM->activate();
					}
				  break;

		case 3: 	// weapon selected
				  j = a_CTWeaponUsed->value();
				  for (k=0; config.i_idList[ID_WEAP][k] != -1; k++) {
						i = config.i_idList[ID_WEAP][k];
						sprintf(mybuf2, "%s (%d/%d)", players[config.i_actionInitID].w_weapons[i].c_description, players[config.i_actionInitID].w_weapons[i].i_noAttacks, players[config.i_actionInitID].w_weapons[i].i_doneAttacks);
						a_CTWeaponUsed->text(k+1, mybuf2);
				  }
				  i = config.i_idList[ID_WEAP][j-1];
				  sprintf(mybuf2, "@B255@.%s (%d/%d)", players[config.i_actionInitID].w_weapons[i].c_description, players[config.i_actionInitID].w_weapons[i].i_noAttacks, players[config.i_actionInitID].w_weapons[i].i_doneAttacks);
				  a_CTWeaponUsed->text(j, mybuf2);
				  config.i_actionWeaponSelected = i;
				  setCombatDisplay(8);
				  break;

		case 8:
				  i = config.i_actionWeaponSelected;
if (config.flags.f_debug != 0) { printf("sCD: 3: %d:%d\n", i, a_CTWeaponUsed->value()); }
				  a_CTWeaponDetails->clear();
				  sprintf(mybuf1, "@b# Attacks: %d", players[config.i_actionInitID].w_weapons[i].i_noAttacks); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "@b# Attks Done: %d", players[config.i_actionInitID].w_weapons[i].i_doneAttacks); a_CTWeaponDetails->add(mybuf1);
				  mybuf2[0] = '\0';
				  if (players[config.i_actionInitID].w_weapons[i].i_attackBonus[1] != 0) {
						sprintf(mybuf2, "%s/%d", mybuf2, players[config.i_actionInitID].w_weapons[i].i_attackBonus[1]);
				  }
				  if (players[config.i_actionInitID].w_weapons[i].i_attackBonus[2] != 0) {
						sprintf(mybuf2, "%s/%d", mybuf2, players[config.i_actionInitID].w_weapons[i].i_attackBonus[2]);
				  }
				  if (players[config.i_actionInitID].w_weapons[i].i_attackBonus[3] != 0) {
						sprintf(mybuf2, "%s/%d", mybuf2, players[config.i_actionInitID].w_weapons[i].i_attackBonus[3]);
				  }
				  sprintf(mybuf1, "Attk Bonuses: %d%s", players[config.i_actionInitID].w_weapons[i].i_attackBonus[0], mybuf2); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "Damage : %s", players[config.i_actionInitID].w_weapons[i].c_damage); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "Critical : %s", players[config.i_actionInitID].w_weapons[i].c_critical); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "Range : %d", players[config.i_actionInitID].w_weapons[i].i_range); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "Magic Bonus : %d", players[config.i_actionInitID].w_weapons[i].i_magicalBonus); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "Message : %s", players[config.i_actionInitID].w_weapons[i].c_message); a_CTWeaponDetails->add(mybuf1);
				  sprintf(mybuf1, "%d", getWeaponAttackBonus(config.i_actionInitID, config.i_actionWeaponSelected)); a_CTMeleeBAB->value(mybuf1);
				  a_CTMeleeDamage->value("");
						// show any damage reduction details
				  if (config.i_actionRecvID != -1) {
					 if (players[config.i_actionRecvID].i_damageReduction[1] != 0) {
						if (players[config.i_actionRecvID].i_damageReduction[0] != 1) {
							sprintf(mybuf2, "@B88@.Damage Reduction: %d/%d %s ... Please adjust manually !!!!", players[config.i_actionRecvID].i_damageReduction[1], players[config.i_actionRecvID].i_damageReduction[2], dam_reduct_types[players[config.i_actionRecvID].i_damageReduction[0]]);
						} else {
							sprintf(mybuf2, "@B95@.Damage Reduction: %d/%d %s will be applied !!!!", players[config.i_actionRecvID].i_damageReduction[1], players[config.i_actionRecvID].i_damageReduction[2], dam_reduct_types[players[config.i_actionRecvID].i_damageReduction[0]]);
						}
						a_CTMessage->text(3, mybuf2);
					 } else {
						a_CTMessage->text(3, "");
					 }
					a_CTMessage->redraw();
					i = atoi(a_CTMeleeAC->value()) + atoi(a_CTMeleeACAdjustments->value()) - atoi(a_CTMeleeBAB->value()) - atoi(a_CTMeleeBABAdjustments->value());
					if (i < 1) { i = 1; }
					if (i > 20) { i = 20; }
					sprintf(mybuf2, "@b@B95@.You require atleast %d to hit them", i);
					a_CTMessage->text(1, mybuf2);
				  } else {
					a_CTMessage->text(1, "");
				  }
				  break;

		case 4: 	// defender selected
				  j = a_CTDefender->value();
				  for (i=0; i<MAX_MEMBERS && config.i_idList[ID_DEFD][i] != -1; i++) {
						k = config.i_idList[ID_DEFD][i];
						sprintf(mybuf2, "%s (%d=%d%%)", players[k].c_name, players[k].i_hp[HP_CUR], getHPpercentage(k));
						a_CTDefender->text(i+1, mybuf2);
				  }
				  i = config.i_idList[ID_DEFD][j-1];
				  sprintf(mybuf2, "@B255@.%s (%d=%d%%)", players[i].c_name, players[i].i_hp[HP_CUR], getHPpercentage(i));
				  a_CTDefender->text(j, mybuf2);
if (config.flags.f_debug != 0) { printf("sCD: 3: %d:%d:%d\n", i, a_CTDefender->value(), j); }
				  config.i_actionRecvID = i;
				  setCombatDisplay(7);
				  if (config.i_actionWeaponSelected != -1) {
				  	setCombatDisplay(8);
				  }
				  break;

		case 7:
				  i = config.i_actionRecvID;
				  a_CTDefenderDetails->clear(); a_EDDefenderDetails1->clear();
				  sprintf(mybuf2, "Name: %s", players[config.i_actionRecvID].c_name); a_CTDefenderDetails->add(mybuf2); a_EDDefenderDetails1->add(mybuf2);
				  sprintf(mybuf2, "Speed: %d", players[config.i_actionRecvID].i_speed); a_CTDefenderDetails->add(mybuf2); a_EDDefenderDetails1->add(mybuf2);
				  sprintf(mybuf1, "Touch:%d FFoot:%d (Arm:%d Shd:%d Nat:%d Dex:%d Def:%d Siz:%d)", players[i].i_armorClass[AC_TOUCH], players[i].i_armorClass[AC_FLATFOOTED], players[i].i_armorClassSplit[AC_SECTION_ARMOR], players[i].i_armorClassSplit[AC_SECTION_SHIELD], players[i].i_armorClassSplit[AC_SECTION_NATURAL], players[i].i_armorClassSplit[AC_SECTION_DEX], players[i].i_armorClassSplit[AC_SECTION_DEFLECTION], players[i].i_armorClassSplit[AC_SECTION_SIZE]); a_CTACBreakdown->value(mybuf1); a_EDDefenderDetails1->add(mybuf1);
				  sprintf(mybuf1, "%d", players[i].i_armorClass[AC_NORMAL]); a_CTMeleeAC->value(mybuf1);
				  sprintf(mybuf2, "%d", getAdjustment(config.i_actionRecvID, MOD_AC)); a_CTMeleeACAdjustments->value(mybuf2);
				  sprintf(mybuf2, "%s%s.note", systemDir, players[config.i_actionRecvID].c_name); detailed_buffer[1]->loadfile(mybuf2);
				  	// if stunned/uncon set combat mods
					a_CTCombatMods->check_none();
					if (players[config.i_actionRecvID].i_stun > 0) { a_CTCombatMods->checked(7, 1); }
					if (getHPs(config.i_actionRecvID) <= players[config.i_actionRecvID].i_unconciousAt) { a_CTCombatMods->checked(6, 1); }
					calcCombatMods(0);
					a_CTMessage->text(3, "");		// clear damage reduction line
					if (players[config.i_actionRecvID].i_damageReduction[1] != 0) {
						if (players[config.i_actionRecvID].i_damageReduction[0] != 1) {
							sprintf(mybuf2, "@B88@.Damage Reduction: %d/%d %s ... Please adjust manually !!!!", players[config.i_actionRecvID].i_damageReduction[1], players[config.i_actionRecvID].i_damageReduction[2], dam_reduct_types[players[config.i_actionRecvID].i_damageReduction[0]]);
						} else {
							sprintf(mybuf2, "@B95@.Damage Reduction: %d/%d %s will be applied !!!!", players[config.i_actionRecvID].i_damageReduction[1], players[config.i_actionRecvID].i_damageReduction[2], dam_reduct_types[players[config.i_actionRecvID].i_damageReduction[0]]);
						}
						a_CTMessage->text(3, mybuf2);
					} else {
						a_CTMessage->text(3, "");		// clear damage reduction line
					}
					a_CTMessage->redraw();
				  break;

		case 5: 	// other selected
//				  j = a_CTOtherActions->value();
//				  i = config.i_idList[ID_DEFD][j-1];
//				  config.i_actionRecvID = i;
//if (config.flags.f_debug != 0) { printf("sCD: 3: %d:%d\n", i, a_CTOtherActions->value()); }
//				  a_CTOtherActions->check_none();
//				  a_CTOtherActions->checked(j, 1);
				  break;

		case 6:	// spell effects selected
				  if (config.i_actionSelected == 6 || config.i_actionSelected == 7) {	// spell touch attack
				  		j = a_CTSpellEffects->value();
				  		a_CTSpellEffects->check_none();
				  		a_CTSpellEffects->checked(j, 1);
				  }
				  break;
	}
}

/***********************************************************************/
void init16()     /* initialise the RANDOM No. generator */
{
time_t ltime;
struct tm *ntime;

	time(&ltime);
	ntime = gmtime(&ltime);

	seed1 = (ntime->tm_sec*1000)+(ntime->tm_min*100)+ntime->tm_mday;
	seed2 = (ntime->tm_min*1000)+(ntime->tm_sec*100)+ntime->tm_year;
	seed3 = (ntime->tm_sec*1000)+(ntime->tm_hour*100)+ntime->tm_mon;
}

/****************************************************************************/
double uniform16()   /* get a RANDOM No. */
{
int z, k;

    k = seed1 / 206;
    seed1 = 157 * (seed1 - k * 206) - k * 21;
    if (seed1 < 0)
        seed1 += 32363;

    k = seed2 / 217;
    seed2 = 146 * (seed2 - k * 217) - k * 45;
    if (seed2 < 0)
        seed2 += 31727;

    k = seed3 / 222;
    seed3 = 142 * (seed3 - k * 222) - k * 133;
    if (seed3 < 0)
        seed3 += 31657;

    z = seed1 - seed2;
    if (z > 706)
        z -= 32362;

    z += seed3;
    if (z < 1)
        z += 32362;

    return(z * 3.0899E-5);
}

/******************************************************************************/
int getRND(int val)
{
   if (val < 1) {
      val = getRND(20);
      fl_alert("Possible data issue ... 0 in GET-RND");
   }

   val = ((int )(uniform16() * 32767.99)) % val;

//if (config.flags.f_debug != 0) { printf("gRND: %d\n", val+1); }
   return(val+1);
} 
	
/******************************************************************************/
void endOfRound() {
	int i=0,j=0,k=0;
	char mybuf[70];

	if (a_EORMessageList == NULL) { doEndOfRoundMessages(0); }

	j = 0;
		// check for any players/monsters that have not done all their actions
	a_EORMessageList->clear();
	a_EORMessageList->add("@bThe following Players/Monsters have not done all their actions:");
	for (i=0; i<(MAX_MEMBERS); i++) {
		if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_CUR] > players[i].i_unconciousAt && players[i].i_doneAttacks < players[i].i_noAttacks && players[i].flags.f_disabled == 0) {
			a_EORMessageList->add(players[i].c_name);
			j = 1;
					// do some HP boundary checking and the like
			if (players[i].i_regeneration != 0) { players[i].i_hp[HP_CUR] += players[i].i_regeneration; }
			if (players[i].i_hp[HP_CUR] > players[i].i_hp[HP_MAX]) { players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX]; }
			if (players[i].i_hp[HP_CUR] < -20) { players[i].i_hp[HP_CUR] = -20; }
			if (players[i].i_hp[HP_CUR] > 0) { players[i].flags.f_isStabilised = 0; }
		}
	}
		// check for any Messages to be shown
	a_EORMessageList->add("");
	a_EORMessageList->add("@bThe following Alerts have occurred:");
	for (i=0; i<MAX_TRIGGERS; i++) {
		if (config.action_msgs[i].i_noRounds > 0) {
			config.action_msgs[i].i_noRounds--;
			if (config.action_msgs[i].i_noRounds > 0 && config.action_msgs[i].i_display == 1) {
				a_EORMessageList->add(config.action_msgs[i].c_message);
				j = 1;
			} else if (config.action_msgs[i].i_noRounds == 0) {
				sprintf(mybuf1, "@B63@.%s", config.action_msgs[i].c_message);
				a_EORMessageList->add(mybuf1);
				if (strlen(config.action_msgs[i].c_every) > 0) {
					config.action_msgs[i].i_noRounds = parseDice(config.action_msgs[i].c_every);
				} else {
					config.action_msgs[i].i_display = 0;
					sprintf(config.action_msgs[i].c_message, "-- empty slot %d", i);
				}
				j = 1;
			}
		}
	}
		// check for any Modifiers that are finishing
	a_EORMessageList->add("");
	a_EORMessageList->add("@bThe following Adjustments have expired:");
	for (i=0; i<MAX_MODIFIERS; i++) {
		if (config.action_mods[i].i_noRounds > 0) {
			config.action_mods[i].i_noRounds--;
			if (config.action_mods[i].i_noRounds == 0) {
				sprintf(mybuf1, "@B63@.%s", config.action_mods[i].c_message);
				a_EORMessageList->add(mybuf1);
				sprintf(config.action_mods[i].c_message, "-- empty slot %d", i);
				config.action_mods[i].i_whom[0] = config.action_mods[i].i_whom[1] = config.action_mods[i].i_idFollows = -255;
				config.action_mods[i].i_data[0][0] = config.action_mods[i].i_data[0][2] = 0; config.action_mods[i].i_data[0][1] = -1;
				config.action_mods[i].i_data[1][0] = config.action_mods[i].i_data[1][2] = 0; config.action_mods[i].i_data[1][1] = -1;
				config.action_mods[i].i_data[2][0] = config.action_mods[i].i_data[2][2] = 0; config.action_mods[i].i_data[2][1] = -1;
				config.action_mods[i].i_data[3][0] = config.action_mods[i].i_data[3][2] = 0; config.action_mods[i].i_data[3][1] = -1;
				config.action_mods[i].i_data[4][0] = config.action_mods[i].i_data[4][2] = 0; config.action_mods[i].i_data[4][1] = -1;
				j = 1;
			}
		}
	}
		// check for any Auto-Damages that are finishing
	a_EORMessageList->add("");
	a_EORMessageList->add("@bThe following Auto-Damages have expired:");
	for (i=0; i<MAX_AUTODAMAGE; i++) {
//if (config.flags.f_debug != 0) { printf("eOR: AD %d:%s:%d:%d\n", i, config.auto_damage[i].c_description, config.auto_damage[i].i_noRounds, config.auto_damage[i].flags.f_flag2); }
		if (config.auto_damage[i].i_noRounds > 0) {
			config.auto_damage[i].i_noRounds--;
			config.auto_damage[i].i_doneRounds++;
			if (config.auto_damage[i].flags.f_flag2 == 0) {		// has it been done this round
				for (k=0; k<MAX_MEMBERS; k++) {
					if (players[k].c_name[0] != '\0' && players[k].i_hp[HP_MAX] > 0) {
//if (config.flags.f_debug != 0) { printf("eOR: AD %d\n", k); }
						calcAutoDamage(i, k);							// apply it to all 
					}
				}
			}
			if (config.auto_damage[i].i_noRounds == 0) {
				a_EORMessageList->add(config.auto_damage[i].c_description);
				config.auto_damage[i].i_initiatorID = config.auto_damage[i].i_whom[0] = -1;
				config.auto_damage[i].i_noRounds = config.auto_damage[i].i_type = 0;
				config.auto_damage[i].c_damage[0] = '\0';
				sprintf(config.auto_damage[i].c_description, "-- empty slot %d", i);
				config.auto_damage[i].flags.f_flag1 = config.auto_damage[i].flags.f_flag2 = 0;
				config.auto_damage[i].flags.f_flag3 = config.auto_damage[i].flags.f_flag4 = 0;
				config.auto_damage[i].flags.f_flag5 = config.auto_damage[i].flags.f_flag6 = 0;
				config.auto_damage[i].flags.f_flag7 = config.auto_damage[i].flags.f_flag8 = 0;
				config.auto_damage[i].i_doneRounds = 0;
				j = 1;
			}
			config.auto_damage[i].flags.f_flag2 = 0;
		}
	}

	if (j != 0) {		// something to display, so display it
		i_EORanswer = -1;
		if (doEndOfRoundMessages(0) == -1) {
			return;
		}
	}

	sprintf(mybuf, "%d", ++config.i_roundCount); aSys_Rounds->value(mybuf);

		// do they BLEED now
	if (config.i_whenBleed == 1) {
		for (i=0; i<MAX_MEMBERS; i++) {
			if (players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
				if ((players[i].i_hp[HP_CUR] <= players[i].i_disabledAt && players[i].flags.f_isStabilised == 0) || players[i].i_bleeding != 0) {
if (config.flags.f_debug != 0) { printf("eOR: Bl %d:%d\n", i, players[i].i_bleeding); }
						if (players[i].i_bleeding != 0) {
							players[i].i_hp[HP_CUR] -= players[i].i_bleeding;
							if (players[i].i_hp[HP_CUR] > players[i].i_hp[HP_MAX]) { players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX]; }
							sprintf(mybuf2, "Bleed: %s (HP:%d) Cur:%d", players[i].c_name, players[i].i_bleeding, players[i].i_hp[HP_CUR]);
						} else {
							if (getRND(100) < 11 && (i >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0)) {		// did they stabilise ?
								players[i].flags.f_isStabilised = 1;
								sprintf(mybuf2, "Bleed: %s (HP:0 Sb:1) Cur:%d", players[i].c_name, players[i].i_hp[HP_CUR]);
							} else {
								players[i].i_hp[HP_CUR]--;
								sprintf(mybuf2, "Bleed: %s (HP:1) Cur:%d", players[i].c_name, players[i].i_hp[HP_CUR]);
							}
						}
						writeLog(mybuf2);
						if (players[i].i_hp[HP_CUR] < -9) {		// are they DEAD
							sprintf(mybuf, "%s is DEAD !!!", players[i].c_name);
							doSystemAlert(mybuf);
						} else if (players[i].i_hp[HP_CUR] <= players[i].i_unconciousAt) {		// are they UNCONCIOUS
							sprintf(mybuf, "%s is UNCONCIOUS !!!", players[i].c_name);
							doSystemAlert(mybuf);
						}
				}
			}
		}
	} else {
			// check to see that you didnt miss an action
		for (i=0; i<(MAX_MEMBERS); i++) {
			if (players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
				if (((players[i].i_hp[HP_CUR] <= players[i].i_disabledAt && players[i].flags.f_isStabilised == 0) || players[i].i_bleeding != 0) && players[i].i_doneAttacks == 0) {
if (config.flags.f_debug != 0) { printf("eOR: Bl %d:%d\n", i, players[i].i_bleeding); }
						if (players[i].i_bleeding != 0) {
							players[i].i_hp[HP_CUR] -= players[i].i_bleeding;
							if (players[i].i_hp[HP_CUR] > players[i].i_hp[HP_MAX]) { players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX]; }
							sprintf(mybuf2, "Bleed: %s (HP:%d) Cur:%d", players[i].c_name, players[i].i_bleeding, players[i].i_hp[HP_CUR]);
						} else {
							if (getRND(100) > 10) {		// did they stabilise ?
								players[i].i_hp[HP_CUR]--;
								sprintf(mybuf2, "Bleed: %s (HP:1) Cur:%d", players[i].c_name, players[i].i_hp[HP_CUR]);
							} else {
								players[i].flags.f_isStabilised = 1;
								sprintf(mybuf2, "Bleed: %s (HP:0 Sb:1) Cur:%d", players[i].c_name, players[i].i_hp[HP_CUR]);
							}
						}
						writeLog(mybuf2);
				}
			}
		}
	}

		// do misc checks
	for (i=0; i<(MAX_MEMBERS); i++) {
		if (players[i].i_inactive > 0) {
			players[i].i_inactive--;
		}
		players[i].i_doneAttacks = 0;
		for (j=0; j<MAX_ATTACKS+2; j++) {
			players[i].w_weapons[j].i_doneAttacks = 0;
		}
		if (players[i].i_stun > 0) {
			players[i].i_stun--;
		}
	}

		// do we need to save the data
	if (config.i_roundsSave > 0) {
		if ((config.i_roundCount % config.i_roundsSave) == 0) {
			saveAllData();
		}
	}

	if (config.flags.f_rollInitEveryRnd != 0) {
		doInitiativeAction(1);		// reroll inits
	}

	// recalc the INIT seq
	doInitiativeAction(2);		// reload inits
	doInitiativeAction(3);		// accept inits

	reloadNames(0);			// update player/monster details
	reloadNames(1);			// update player/monster details
	reloadNames(9);			// update triggers/adjustments/auto-damage
	doOtherAction(20);		// update the status tab

	aLF_Logfile->load("logfile.txt");

	if (config.flags.f_logfile != 0) {
		writeLog("");
		sprintf(mybuf1, "        New Round: %d", config.i_roundCount);
		writeLog(mybuf1);
	}
}

/******************************************************************************/
void saveAllData() {
	FILE *fp;
	int i=0;

		  if (config.f_version > 1.10) {
				writeAllPlayerData(config.c_partyFilename);
				writeAllMonsterData(config.c_monstFilename);
				config.f_version = ADMPversion;
				writeSystemData("system.sys");
		  } else {
			  sprintf(mybuf2, "%ssystem.dat", systemDir);
			  if ((fp = fopen(mybuf2, "w")) != NULL) {
					 fwrite(&config.i_roundCount, sizeof(ADMPsystem), 1, fp);
					 fchmod(i, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
					 fclose(fp);
					 if (config.flags.f_debug != 0) printf("sAD - system file save OK (%s) !!!\n", mybuf2); 
			  } else {
					 if (config.flags.f_debug != 0) printf("sAD - system file save failed (%s) !!!\n", mybuf2); 
					 sprintf(mybuf1, "System File Save Failed (%s)", mybuf2);
					 doSystemAlert(mybuf1);
			  }

			  strncpy(config.c_partyFilename, aSys_playerFilename->value(), 43);
			  config.c_partyFilename[43] = '\0';
			  if (strncasecmp(&config.c_partyFilename[0], systemDir, strlen(systemDir)-1) != 0) {
			  		sprintf(mybuf2, "%s%s", systemDir, config.c_partyFilename);
			  } else {
			  		sprintf(mybuf2, "%s", config.c_partyFilename);
			  }
			  if ((fp = fopen(mybuf2, "w")) != NULL) {
					fwrite(&players[0].c_name[0], sizeof(ADMPplayer) * MAX_PLAYERS, 1, fp);
					fchmod(i, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
					fclose(fp);
					if (config.flags.f_debug != 0) printf("sAD - party file save OK (%s) !!!\n", mybuf2);
			  } else {
					if (config.flags.f_debug != 0) printf("sAD - party file save failed (%s) !!!\n", mybuf2);
					sprintf(mybuf1, "Party File Save Failed (%s)", mybuf2);
					doSystemAlert(mybuf1);
			  }
	
			  strncpy(config.c_monstFilename, aSys_monsterFilename->value(), 43);
			  config.c_monstFilename[43] = '\0';
			  if (strncasecmp(&config.c_monstFilename[0], systemDir, strlen(systemDir)-1) != 0) {
			  		sprintf(mybuf2, "%s%s", systemDir, config.c_monstFilename);
			  } else {
			  		sprintf(mybuf2, "%s", config.c_monstFilename);
			  }
			  if ((fp = fopen(mybuf2, "w")) != NULL) {
					fwrite(&players[MAX_PLAYERS].c_name[0], sizeof(ADMPplayer) * MAX_MONSTERS, 1, fp);
					fchmod(i, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
					fclose(fp);
					if (config.flags.f_debug != 0) printf("sAD - monster file save OK (%s) !!!\n", mybuf2);
			  } else {
					if (config.flags.f_debug != 0) printf("sAD - monster file save failed !!!\n");
					sprintf(mybuf1, "Monster File Save Failed (%s)", mybuf2);
					doSystemAlert(mybuf1);
			  }
	
// if (config.flags.f_debug != 0) { printf("sAD: %s %s\n", config.c_monstFilename, config.c_partyFilename); }
			  // save the notes as well
			  notes_buffer->savefile("notes.txt");
		  }

}

/******************************************************************************/
void loadAllData() {
	FILE *fp=NULL;

		init16();

		if (config.flags.f_debug != 0) printf("lAD: %s %s\n", config.c_partyFilename, config.c_monstFilename);

		if (ADMPversion > 1.11) {
			readSystemData("system.sys");
		} else {
			sprintf(mybuf2, "%ssystem.dat", systemDir);
			if ((fp = fopen(mybuf2, "r")) != NULL) {
				 fread(&config.i_roundCount, sizeof(ADMPsystem), 1, fp);
				 fclose(fp);
				 if (config.flags.f_debug != 0) printf("lAD - system file load OK (%s) !!!\n", mybuf2);
			} else {
				 if (config.flags.f_debug != 0) printf("lAD - system file load failed %s %d !!!\n", mybuf2, errno);
				 sprintf(mybuf1, "System Load Failed (%s)", mybuf2);
				 doSystemAlert(mybuf1);
				 doOtherAction(6);
				 doOtherAction(7);
				 doOtherAction(10);
			}
		}

#ifdef VISUALC
		config.f_version = ADMPversion;		// force windows to be the version
#endif

		if (config.f_version > 1.10) {
				i_monsterType = 1;
		  		readAllPlayerData(config.c_partyFilename);
		  		readAllMonsterData(config.c_monstFilename, &players[MAX_PLAYERS]);
				i_monsterType = 0; qkMonster = -1;
				readAllStdMonsterData("srdmonster");
				i_monsterType = -1;
				readAllStdMonsterData("stdmonster");
		} else {
			  if (strncasecmp(&config.c_partyFilename[0], systemDir, strlen(systemDir)-1) != 0) {
					sprintf(mybuf2, "%s%s", systemDir, config.c_partyFilename);
			  } else {
					strcpy(mybuf2, config.c_partyFilename);
			  }
			  if ((fp = fopen(mybuf2, "r")) != NULL) {
						 fread(&players[0].c_name[0], sizeof(ADMPplayer) * MAX_PLAYERS, 1, fp);
						 fclose(fp);
						 if (config.flags.f_debug != 0) printf("lAD - party file load OK (%s) !!!\n", mybuf2);
			  } else {
						 if (config.flags.f_debug != 0) printf("lAD - party file load failed %s %d !!!\n", mybuf2, errno);
						 sprintf(mybuf1, "Party Load Failed (%s)", mybuf2);
						 doSystemAlert(mybuf1);
			  }
	
			  if (strncasecmp(&config.c_monstFilename[0], systemDir, strlen(systemDir)-1) != 0) {
			  		sprintf(mybuf2, "%s%s", systemDir, config.c_monstFilename);
			  } else {
					strcpy(mybuf2, config.c_monstFilename);
			  }
			  if ((fp = fopen(mybuf2, "r")) != NULL) {
						 fread(&players[MAX_PLAYERS].c_name[0], sizeof(ADMPplayer) * MAX_MONSTERS, 1, fp);
						 fclose(fp);
						 if (config.flags.f_debug != 0) printf("lAD - monster file load OK (%s) !!!\n", mybuf2);
			  } else {
						 if (config.flags.f_debug != 0) printf("lAD - monster file load failed %s %d !!!\n", mybuf2, errno);
						 sprintf(mybuf1, "Monster Load Failed (%s)", mybuf2);
						 doSystemAlert(mybuf1);
			  }
	
			  sprintf(mybuf2, "%sqkmonst.dat", systemDir);
			  if ((fp = fopen(mybuf2, "r")) != NULL) {
						 fread(&qkmonsters[0].c_name[0], sizeof(ADMPplayer) * MAX_QKMONSTERS, 1, fp);
						 fclose(fp);
						 if (config.flags.f_debug != 0) printf("lAD - quick monster file load OK !!!\n");
			  } else {
						 if (config.flags.f_debug != 0) printf("lAD - quick monster file load failed !!!\n");
						 sprintf(mybuf1, "Standard Monster Load Failed (%s)", mybuf2);
						 doSystemAlert(mybuf1);
			  }
		}

		if (config.f_version < 1.11) {
			convertToVersion();
		}

		if (config.flags.f_debug != 0) printf("lAD:1\n");

			// load up the system screen
//		sprintf(mybuf, "%d", config.i_roundCount); aSys_Rounds->value(mybuf);
//		aSys_Bleeding->value(config.i_whenBleed);
//		aSys_XPAllocation->value(config.i_xpAllocate);
//		aSys_GroupInitiatives->value(config.i_doGroupInitiatives);
//		aSys_MultipleAttacks->value(config.i_attackGroup);
//		aSys_Debug->value(config.flags.f_debug);
//
//		aSys_monsterFilename->value(config.c_monstFilename);
//		aSys_playerFilename->value(config.c_partyFilename);
//		sprintf(mybuf, "%d", config.i_roundsSave); aSys_saveRounds->value(mybuf);

		reloadNames(0);
}

/******************************************************************************/
void copyMonsterNotes() {
char *cp;
FILE *of;

	sprintf(mybuf1, "%s.mnt", config.c_monstFilename);
	cp = strstr(pdbDataLine, "|"); cp++;
	cp = strstr(cp, "|"); cp++;
//printf("cMN: %d %s\n", i_monsterNoteID, cp);

	if ((of = fopen(mybuf1, "a+")) != NULL) {
		fprintf(of, "+%d|%d|%s", i_monsterNoteID, i_monsterNoteID, cp);
		fclose(of);
	}
}

/******************************************************************************/
void duplicatePlayer(int plyr, int cnt) {
	int j=0;

	if (cnt == 0) { return; }

	for (int i=MAX_PLAYERS;  i < MAX_MONSTERS; i++) {
		if (players[i].c_name[0] == '\0' && players[i].i_hp[HP_MAX] == 0) {
if (config.flags.f_debug != 0) printf("dP: %d %d\n", plyr, i);
			memcpy(&players[i].c_name[0], &players[plyr].c_name[0], sizeof(ADMPplayer));
			if (players[i].c_name[2] == '-') {
				players[i].c_name[0] = identifiers[i];
			} else {
				sprintf(mybuf2, "%c - %s", identifiers[i], players[i].c_name);
				strncpy(players[i].c_name, mybuf2, 29);
			}
			players[i].flags.f_isStabilised = 0; // TODO players[i].flags.f_autoStablise = 0;
			players[i].flags.f_disabled = 0; 
			players[i].i_initiative[1] = getRND(20);
			players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX];
			for (j=0; j<MAX_D20ROLLS; j++) {
				players[i].i_d20Rolls[j] = getRND(20);
			}
				// copy std monster notes to the new monster
			if (plyr == (MAX_MEMBERS+1)) {
				i_monsterNoteID = i;
					// delete the old note just incase
				sprintf(mybuf1, "%s.mnt", config.c_monstFilename);
				sprintf(mybuf2, "%d", i);
				pdbDeleteRecord(mybuf1, mybuf2, NULL, NULL);
					// find it and copy it over
//printf("dP: SMnote %d = %d (%d)\n", i_srdMonsterID, i, players[plyr].flags.f_srdMonster);
				sprintf(mybuf2, "%d", i_srdMonsterID);
				if (players[i].flags.f_srdMonster == -1) {
					sprintf(mybuf2, "%d", players[i].i_stdMonsterID);
					pdbKeySearch("stdmonster.snt", copyMonsterNotes, NULL, 1, mybuf2, NULL, NULL);
				} else {
					pdbKeySearch("srdmonster.snt", copyMonsterNotes, NULL, 1, mybuf2, NULL, NULL);
				}
				//players[i].i_stdMonsterID = i_srdMonsterID; 
			}
			cnt--;
			if (cnt == 0)
			  break;
		}
	}
}

/******************************************************************************/
void doEditSpecialActions(int action) {
	int i, j;

if (config.flags.f_debug != 0) { printf("dESA: %d %d\n", action, config.i_editID); }

	if (config.i_editID < 0 || config.i_editID > MAX_MEMBERS) {
		return;
	}

	i = config.i_editID;

	switch (action) {
		case 0: memset(&players[i].c_name[0], '\0', sizeof(ADMPplayer));		// delete
				  players[i].flags.f_disabled = 1;
				  cancelEdit();
				  sprintf(mybuf1, "%s.mnt", config.c_monstFilename);
				  sprintf(mybuf2, "%d", i);
				  pdbDeleteRecord(mybuf1, mybuf2, NULL, NULL);
				  doInitiativeAction(2);		// reload & refresh the init lists
				  doInitiativeAction(3);
				  break;

		case 1: savePlayer(i);
				  duplicatePlayer(i, 1);
				  break;

		case 2: savePlayer(i);
				  duplicatePlayer(i, 2);
				  break;

		case 3: savePlayer(i);
				  duplicatePlayer(i, 4);
				  break;

		case 4: // save as a standard monster
				  actionQkMonsters(-1);
				  break;

		case 5: savePlayer(i);			// resequence ids
				  for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
if (config.flags.f_debug != 0) { printf("dESA: 5 %d\n", i); }
				  			for (j=MAX_PLAYERS; j<MAX_MEMBERS && j < i; j++) {
								if (players[j].c_name[0] == '\0' && players[j].i_hp[HP_MAX] == 0) {
if (config.flags.f_debug != 0) { printf("dESA: 5 %d %d\n", i, j); }
									memcpy(&players[j].c_name[0], &players[i].c_name[0], sizeof(ADMPplayer));
									players[j].c_name[0] = identifiers[j];
									memset(&players[i].c_name[0], '\0', sizeof(ADMPplayer));
				  					players[i].flags.f_disabled = 1;
									break;
								}
							}
						}
				  }
				  a_OTDisableList->clear();
				  for (i=0; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
							a_OTDisableList->add(players[i].c_name);
							config.i_idList[ID_DISA][j++] = i;
						}
				  }
				  a_OTDisableList->redraw();
				  break;

		case -1: 	// resequence ids
				  for (i=0; i<MAX_PLAYERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
if (config.flags.f_debug != 0) { printf("dESA: 5 %d\n", i); }
				  			for (j=0; j<MAX_PLAYERS && j < i; j++) {
								if (players[j].c_name[0] == '\0' && players[j].i_hp[HP_MAX] == 0) {
if (config.flags.f_debug != 0) { printf("dESA: 5 %d %d\n", i, j); }
									memcpy(&players[j].c_name[0], &players[i].c_name[0], sizeof(ADMPplayer));
									players[j].c_name[0] = identifiers[j];
									memset(&players[i].c_name[0], '\0', sizeof(ADMPplayer));
				  					players[i].flags.f_disabled = 1;
									break;
								}
							}
						}
				  }
				  for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
if (config.flags.f_debug != 0) { printf("dESA: 5 %d\n", i); }
				  			for (j=MAX_PLAYERS; j<MAX_MEMBERS && j < i; j++) {
								if (players[j].c_name[0] == '\0' && players[j].i_hp[HP_MAX] == 0) {
if (config.flags.f_debug != 0) { printf("dESA: 5 %d %d\n", i, j); }
									memcpy(&players[j].c_name[0], &players[i].c_name[0], sizeof(ADMPplayer));
									players[j].c_name[0] = identifiers[j];
									memset(&players[i].c_name[0], '\0', sizeof(ADMPplayer));
				  					players[i].flags.f_disabled = 1;
									break;
								}
							}
						}
				  }
				  a_OTDisableList->clear();
				  for (i=0; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
							a_OTDisableList->add(players[i].c_name);
							config.i_idList[ID_DISA][j++] = i;
						}
				  }
				  a_OTDisableList->redraw();
				  break;
	}

	reloadNames(0);
}

/********************************************************************************/
Fl_Window *alertWindow=NULL;
Fl_Button *alertButton=NULL;
Fl_Output *alertMessage=NULL;

static void cb_acceptAlertButton(Fl_Button*, void*) {
  alertWindow->hide();
}

/********************************************************************************/
void doSystemAlert(char *msg) {
	if (alertWindow == NULL) {
		alertWindow = new Fl_Window(440, 70, "ADMP: Alert Message");
		alertWindow->labelsize(ADMP_LABEL_SIZE);
		{ alertMessage = new Fl_Output(1, 1, 434, 40);
		  alertMessage->type(4);
		  alertMessage->labelsize(ADMP_LABEL_SIZE);
		  alertMessage->textsize(ADMP_TEXT_SIZE);
		}
		{ alertButton = new Fl_Button(198, 43, 55, 25, "OK");
		  alertButton->labelsize(ADMP_LABEL_SIZE);
		  alertButton->callback((Fl_Callback*)cb_acceptAlertButton);
		}
		alertWindow->set_modal();
		alertWindow->end();
	}
	alertMessage->value(msg);
	alertWindow->show();
	alertButton->take_focus();
}

/********************************************************************************/
Fl_Window *AYSWindow=NULL;
Fl_Output *AYSMessage=NULL;

/********************************************************************************/
int doAreYouSure(char *msg) {
int AYSreturn = -1;			// Assume No

	if (AYSWindow == NULL) {
		AYSWindow = new Fl_Window(350, 70, "ADMP: Are You Sure ??");
		AYSWindow->labelsize(ADMP_LABEL_SIZE);
		{ AYSMessage = new Fl_Output(1, 1, 324, 40);
		  AYSMessage->type(4);
		  AYSMessage->labelsize(ADMP_LABEL_SIZE);
		  AYSMessage->textsize(ADMP_TEXT_SIZE+1);
		}
		{ AYSButton[0] = new Fl_Button(120, 43, 55, 25, "Yes");
		  AYSButton[0]->labelsize(ADMP_LABEL_SIZE);
		}
		{ AYSButton[1] = new Fl_Button(198, 43, 55, 25, "No");
		  AYSButton[1]->labelsize(ADMP_LABEL_SIZE);
		}
		{ AYSButton[2] = new Fl_Button(260, 43, 85, 25, "Maybe");
		  AYSButton[2]->labelsize(ADMP_LABEL_SIZE);
		}
		AYSButton[2]->deactivate();
		AYSWindow->end();
	}
	AYSWindow->set_modal();
	if (msg == NULL) {		// check to see if it's just to configure it
		return -1;
	}

	AYSMessage->value(msg);
	AYSWindow->show();
	for (;;) {
		Fl_Widget *o = Fl::readqueue();
		if (!o) {
			Fl::wait();
		} else if ( o == AYSButton[0]) { AYSreturn = 1; break;
		} else if ( o == AYSButton[1]) { AYSreturn = -1; break;
		} else if ( o == AYSButton[2]) { AYSreturn = -2; break;
		}
	}
	AYSWindow->hide();
	return AYSreturn;
}

/********************************************************************************/
int parseDice(char *dice) { //, int *mult, int *die, int *plus) {
	int roll[3] = {1,0,0};		/* xDy+z */
	int i=0,j=0,d=-1, result=0;
	char mybuf[25], *cp;

	strncpy(mybuf, dice, 24); mybuf[24] = '\0';
if (config.flags.f_debug) { printf("pD: %s\n", dice); }

	if ((cp = strstr(mybuf, " + ")) != NULL) {
		*cp = '\0';
		cp++; cp++; cp++;
		result += parseDice(cp);
	} else if ((cp = strstr(mybuf, " - ")) != NULL) {
		*cp = '\0';
		cp++; cp++; cp++;
		result += parseDice(cp);
	}

	while (mybuf[i] != '\0') {
		if (mybuf[i] == 'D' || mybuf[i] == 'd') {
			mybuf[i] = '\0'; roll[0] = atoi(mybuf); j = i+1;
			d=i;
		} else if (mybuf[i] == '+' || mybuf[i] == '-') {
			roll[2] = atoi(&mybuf[i]); mybuf[i] = '\0'; roll[1] = atoi(&mybuf[j]); 
		}
//if (config.flags.f_debug) { printf("pD: %d %s = %d/%d/%d\n", i, dice, roll[0], roll[1], roll[2]); }
		i++;
	}
		// check for a XdY string
	if (roll[1] == 0) {
		roll[1] = atoi(&mybuf[j]);
	}

	if (roll[0] == 0) {
		roll[0] = 1;
	}

	if (d == -1) {
		result += atoi(dice);
//if (config.flags.f_debug) { printf("pD: %s = %d/%d/%d = %d\n", dice, roll[0], roll[1], roll[2], j); }
	} else {
//if (config.flags.f_debug) { printf("pD: %s = %d/%d/%d = ", dice, roll[0], roll[1], roll[2]); }
		result += roll[2];
		while (roll[0] > 0) {
			result += getRND(roll[1]);
			roll[0] --;
		}
		if (result < 1) {
			result = 1;
		}
if (config.flags.f_debug != 0) { printf("pD: %s = %d\n", dice, result); }
	}

	return result;
}

/********************************************************************************/
int parseCritical(char *crit, int dice, int *from, int *to, int *mult) {
	int roll[3] = {20,20,3};		/* x-y,*z */
	int i=0,j=0;
	char mybuf[15];

	strncpy(mybuf, crit, 14); mybuf[14] = '\0';

	while (mybuf[i] != '\0') {
		if (mybuf[i] == '-') {
			mybuf[i] = '\0'; roll[0] = atoi(&mybuf[j]); j = i+1;
		} else if (mybuf[i] == '/') {
			mybuf[i] = '\0'; roll[1] = atoi(&mybuf[j]); j = i+1;
		} else if (mybuf[i] == '*' || mybuf[i] == 'x') {
			mybuf[i] = '\0'; roll[2] = atoi(&mybuf[i+1]); j = i+1;
		}
		i++;
	}

if (config.flags.f_debug != 0) { printf("pC: %s = %d/%d/%d (%d)\n", crit, roll[0], roll[1], roll[2], dice); }
//sprintf(mybuf2, "pC: %s = %d/%d/%d (%d)\n", crit, roll[0], roll[1], roll[2], dice); writeLog(mybuf2);

	*from = roll[0];
	*to = roll[1];
	*mult = roll[2];

	if (dice >= roll[0]) {
		return roll[2];
	}

	*mult = 3;

	return 3;		// return the standard *3 critical multiplier

}

/********************************************************************************/
void loadMonsterFile() {
	int i=0,j=0;
	ADMPplayer p[MAX_MONSTERS];
	char *iname, *cp;

	if (config.f_version < 1.11) {
		if ((iname = fl_file_chooser("Monster Load", "*.dat", config.c_monstFilename, 0)) == NULL) {
			return;
		}
	} else {
		if ((iname = fl_file_chooser("Monster Load", "*.mon", config.c_monstFilename, 0)) == NULL) {
			return;
		}
	}

	if (strlen(iname) > 42) { doSystemAlert("Monster Load file name is too long ... it may not SAVE !!!!"); }

	for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
		if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
			j++;
		}
	}

if (config.flags.f_debug != 0) { printf("lMF: %s %d\n", iname, j); }
	memset(&p[0].c_name[0], '\0', sizeof(ADMPplayer)*MAX_MONSTERS);
	if (config.f_version < 1.11) {
		if ((openf = fopen(iname, "r")) != NULL) {
			fread(&p[0].c_name[0], sizeof(ADMPplayer) * MAX_MONSTERS, 1, openf);
			fclose(openf);
		} else {
			return;
		}
	} else {	
			 // remove the file extension
		if ((cp = strrchr(iname, '.')) != NULL) { *cp = '\0'; }
		readAllMonsterData(iname, &p[0]);
	}

	if (strncasecmp(iname, systemDir, strlen(systemDir)-1) != 0) {
		if ((cp = strrchr(iname, directorySep[0])) != NULL) {
			cp++;
			strncpy(config.c_monstFilename, cp, 43);
		} else {
			strncpy(config.c_monstFilename, iname, 43);
		}
	} else {
		strncpy(config.c_monstFilename, &iname[strlen(systemDir)], 43);
	}
	config.c_monstFilename[43] = '\0';
	aSys_monsterFilename->value(config.c_monstFilename);

if (config.flags.f_debug != 0) { printf("lMF: %s\n", config.c_monstFilename); }
	if (j > 0) {
		if (doAreYouSure("Shall I merge them in ?") == 1) {
			for (i=0; i<MAX_MONSTERS; i++) {
				if (p[i].c_name[0] != '\0' && p[i].i_hp[HP_MAX] > 0) {
					for (j=MAX_PLAYERS; j<MAX_MEMBERS; j++) {
						if (players[j].c_name[0] == '\0' && players[j].i_hp[HP_MAX] == 0) {
							memcpy(&players[j].c_name[0], &p[i].c_name[0], sizeof(ADMPplayer));
							if (players[j].c_name[2] == '-') {
								players[j].c_name[0] = identifiers[j];
							} else {
								sprintf(mybuf2, "%c - %s", identifiers[j], players[j].c_name);
								strncpy(players[j].c_name, mybuf2, 29); players[j].c_name[29] = '\0';
							}
							break;
						}
					}
				}
			}
		} else {
			memset(&players[MAX_PLAYERS].c_name[0], '\0', sizeof(ADMPplayer)*MAX_MONSTERS);
			memcpy(&players[MAX_PLAYERS].c_name[0], &p[0].c_name[0], sizeof(ADMPplayer) * MAX_MONSTERS);
		}
	} else {
		memset(&players[MAX_PLAYERS].c_name[0], '\0', sizeof(ADMPplayer)*MAX_MONSTERS);
		memcpy(&players[MAX_PLAYERS].c_name[0], &p[0].c_name[0], sizeof(ADMPplayer) * MAX_MONSTERS);
	}

	reloadNames(0);
	doOtherAction(20);
}

/********************************************************************************/
void loadPlayerFile() {
	int j=0;
	FILE *fp=NULL;
	char *iname, *cp;

	if (config.f_version < 1.11) {
		if ((iname = fl_file_chooser("Player Load", "*.dat", config.c_partyFilename, 0)) == NULL) {
			return;
		}
	} else {
		if ((iname = fl_file_chooser("Player Load", "*.pty", config.c_partyFilename, 0)) == NULL) {
			return;
		}
	}

	if (strlen(iname) > 42) { doSystemAlert("Player Load file name is too long ... it may not SAVE !!!!"); }

if (config.flags.f_debug != 0) { printf("lPF: %s %d\n", iname, j); }

		// cleanup the filename
	if (strncasecmp(iname, systemDir, strlen(systemDir)-1) != 0) {
		if ((cp = strrchr(iname, directorySep[0])) != NULL) {
			cp++;
			strncpy(config.c_partyFilename, cp, 43);
		} else {
			strncpy(config.c_partyFilename, iname, 43);
		}
	} else {
		strncpy(config.c_partyFilename, &iname[strlen(systemDir)], 43);
	}
	config.c_partyFilename[43] = '\0';

	if (config.f_version < 1.11) {
		if ((fp = fopen(iname, "r")) != NULL) {
			fread(&players[0].c_name[0], sizeof(ADMPplayer) * MAX_PLAYERS, 1, fp);
			fclose(fp);
		} else {
			sprintf(mybuf1, "Unable to load Party File (%s)", iname);
			doSystemAlert(mybuf1);
		}
	} else {	
			 // remove the file extension
		if ((cp = strrchr(config.c_partyFilename, '.')) != NULL) { *cp = '\0'; }
		readAllPlayerData(config.c_partyFilename);
	}
	aSys_playerFilename->value(config.c_partyFilename);

if (config.flags.f_debug != 0) { printf("lPF: %s\n", config.c_partyFilename); }

	reloadNames(0);
	doOtherAction(20);
}

/********************************************************************************/
void saveMonsterFile() {
	int i=0;
	FILE *fp=NULL;
	char *iname, *cp;

	if (config.f_version < 1.11) {
		if ((iname = fl_file_chooser("Monster Save", "*.dat", config.c_monstFilename, 0)) == NULL) {
			return;
		}
		if ((fp = fopen(iname, "w")) != NULL) {
			fwrite(&players[MAX_PLAYERS].c_name[0], sizeof(ADMPplayer) * MAX_MONSTERS, 1, fp);
			fchmod(i, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
			fclose(fp);
		} else {
			sprintf(mybuf1, "Unable to save Monster File (%s)", iname);
			doSystemAlert(mybuf1);
		}
	} else {
		if ((iname = fl_file_chooser("Monster Save", "*.mon", config.c_monstFilename, 0)) == NULL) {
			return;
		}
		writeAllMonsterData(config.c_monstFilename);
	}

	if (strncasecmp(iname, systemDir, strlen(systemDir)-1) != 0) {
		if ((cp = strrchr(iname, directorySep[0])) != NULL) {
			cp++;
			strncpy(config.c_monstFilename, cp, 43);
		} else {
			strncpy(config.c_monstFilename, iname, 43);
		}
	} else {
		strncpy(config.c_monstFilename, &iname[strlen(systemDir)], 43);
	}
	config.c_monstFilename[43] = '\0';
	aSys_monsterFilename->value(config.c_monstFilename);
}

/********************************************************************************/
void doInitiativeAction(int action) {
	int i=0, x=25, y=235, j=0, a=0, b=0, k=0;

if (config.flags.f_debug != 0) { printf("dIA: %d\n", action); }

	switch (action) {
		case 1:		// reload
		case 2:		// refresh
			for (i=0; i < MAX_PLAYERS; i++) {
				if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0 && getHPs(i) > -10) {
					if (w_Initiatives[j] == NULL) {
						w_Initiatives[j] = new Fl_Input(x, y, 25, 20);
						w_Initiatives[j]->type(2);
						w_Initiatives[j]->labelsize(ADMP_LABEL_SIZE);
						w_Initiatives[j]->textsize(ADMP_TEXT_SIZE);
						w_Initiatives[j]->align(FL_ALIGN_RIGHT);
						a_IPlayers->add(w_Initiatives[j]);
					}
					y += 25;
					w_Initiatives[j]->label(players[i].c_name);
					w_Initiatives[j]->activate();
					w_Initiatives[j]->tooltip("Add their roll to this value if you're doing New initiatives");
					if (action == 2) {
						sprintf(mybuf1, "%d", players[i].i_initiative[1]);
						w_Initiatives[j]->value(mybuf1);
					} else {
						if (config.flags.f_rollInitRolls != 0 || config.flags.f_rollAllRolls != 0) {
							players[i].i_initiative[1] = getRND(20) + getAdjustment(i, MOD_INIT) + players[i].i_initiative[0];
							sprintf(mybuf1, "%d", players[i].i_initiative[1]);
						} else {
							sprintf(mybuf1, "+%d", players[i].i_initiative[0] + getAdjustment(i, MOD_INIT));
						}
						w_Initiatives[j]->value(mybuf1);
					}
					config.i_idList[ID_TEMP][j] = i;
					j++;
				}
			}
			for (; j < MAX_PLAYERS; j++) {
				if (w_Initiatives[j] != NULL) {
					w_Initiatives[j]->label("");
					w_Initiatives[j]->tooltip("");
					w_Initiatives[j]->deactivate();
				}
				config.i_idList[ID_TEMP][j] = -1;
			}
			x=215; y=235; j = MAX_PLAYERS;
			for (i=MAX_PLAYERS; i < MAX_MONSTERS; i++) {
				if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0 && getHPs(i) > -10) {
						// if they are unconcious and the system flag set, dont show 'em
					if (config.flags.f_removeMonstersInit != 0 && getHPs(i) <= players[i].i_unconciousAt) {
if (config.flags.f_debug != 0) { printf("dIA: 2a %d %d %d\n", i, getHPs(i), players[i].i_unconciousAt); }
						;
					} else {
if (config.flags.f_debug != 0) { printf("dIA: 2 %d\n", i); }
						if (w_Initiatives[j] == NULL) {
							w_Initiatives[j] = new Fl_Input(x, y, 25, 20);
							w_Initiatives[j]->type(2);
							w_Initiatives[j]->labelsize(ADMP_LABEL_SIZE);
							w_Initiatives[j]->textsize(ADMP_TEXT_SIZE);
							w_Initiatives[j]->align(FL_ALIGN_RIGHT);
							a_IMonsters->add(w_Initiatives[j]);
						}
						y += 25;
						if (y > 655) { y = 235; x += 150; }
						w_Initiatives[j]->label(players[i].c_name);
						if (action != 2) {
							players[i].i_initiative[1] = getRND(20) + players[i].i_initiative[0];
						}
						sprintf(mybuf1, "%d", players[i].i_initiative[1]);
						w_Initiatives[j]->value(mybuf1);
						w_Initiatives[j]->activate();
						config.i_idList[ID_TEMP][j] = i;
						j++;
					}
				}
			}
			for (; j < MAX_MEMBERS; j++) {
				if (w_Initiatives[j] != NULL) {
					w_Initiatives[j]->label("");
					w_Initiatives[j]->deactivate();
				}
				config.i_idList[ID_TEMP][j] = -1;
			}
			break;

		case 3:		// accept or do
			a=0;
			for (i=0; i<MAX_MEMBERS; i++) {
				j = config.i_idList[ID_TEMP][i];
				if (config.i_idList[ID_TEMP][i] != -1) {
if (config.flags.f_debug != 0) { printf("dIA: 3 %d:%d:%d\n", a, i, j); }
					players[config.i_idList[ID_TEMP][i]].i_initiative[1] = atoi(w_Initiatives[i]->value());
					config.i_idList[ID_INIT][a++] = j;
				}
			}
			for (; a<MAX_MEMBERS; a++) {
				config.i_idList[ID_INIT][a] = -1;
			}

			for (b=0; b<MAX_MEMBERS; b++) {
				for (i=0; i<MAX_MEMBERS; i++) {
					if (config.i_idList[ID_INIT][i+1] != -1) {
						k = players[config.i_idList[ID_INIT][i+1]].i_initiative[1];
						j = players[config.i_idList[ID_INIT][i]].i_initiative[1];
						if (k > j) {
							a = config.i_idList[ID_INIT][i];
							config.i_idList[ID_INIT][i] = config.i_idList[ID_INIT][i+1];
							config.i_idList[ID_INIT][i+1] = a;
						}
					}
				}
			}
			reloadNames(1);
//			for (i=0; i<MAX_MEMBERS; i++) { printf("dIA: 3: %d = %d\n", i, config.i_idList[ID_INIT][i]); }
			break;
	}

	a_IPlayers->redraw();
	a_IMonsters->redraw();
}

/********************************************************************************/
void acceptCombatAction(int action) {
	int i, j, k, l;

if (config.flags.f_debug != 0) { printf("aCA: %d:%d:%d\n", action, config.i_actionSelected, config.i_actionRecvID); }
	switch (action) {
		case 0:	// accept this melee attack
				  if (config.i_actionRecvID == -1) {
						doSystemAlert("Please select a DEFENDER !!!!");
						return;
				  }
				  i = getHPs(config.i_actionRecvID);
				  l = atoi(a_CTMeleeDamage->value());
				  if (l != 0) { 
						l += atoi(a_CTMeleeDmgAdjustments->value()); 
						players[config.i_actionRecvID].flags.f_isStabilised = players[config.i_actionRecvID].flags.f_autoStablise;		// if they take damage, make them 'un-stabilised' if possible
				  }
						// apply any damage reduction but only magic weapon type
if (config.flags.f_debug != 0) { printf("aCA: DR: %d %d %d %d\n", players[config.i_actionRecvID].i_damageReduction[1], players[config.i_actionRecvID].i_damageReduction[0], players[config.i_actionInitID].w_weapons[config.i_actionWeaponSelected].i_magicalBonus, players[config.i_actionRecvID].i_damageReduction[2]); }
					if (players[config.i_actionRecvID].i_damageReduction[1] != 0 && players[config.i_actionRecvID].i_damageReduction[0] == 1 && players[config.i_actionInitID].w_weapons[config.i_actionWeaponSelected].i_magicalBonus < players[config.i_actionRecvID].i_damageReduction[2]) {
if (config.flags.f_debug != 0) { printf("aCA: DR\n"); }
						if (l > players[config.i_actionRecvID].i_damageReduction[1]) {
							l -= players[config.i_actionRecvID].i_damageReduction[1];
						} else {
							l = 0;
						}
					}
				  		// is it subdual damage
				  if (a_CTSpecialAttacks->checked(2) == 0) {
				  		players[config.i_actionRecvID].i_hp[HP_CUR] -= l;
				  } else {
				  		players[config.i_actionRecvID].i_hp[HP_NL] += l;
				  }
				  players[config.i_actionRecvID].i_inactive += atoi(a_CTMeleeInactive->value());
				  players[config.i_actionRecvID].i_bleeding += atoi(a_CTMeleeBleeding->value());
				  players[config.i_actionRecvID].i_stun += atoi(a_CTMeleeStun->value());
				  if (players[config.i_actionRecvID].i_hp[HP_CUR] <= players[config.i_actionRecvID].i_unconciousAt && i > players[config.i_actionRecvID].i_unconciousAt) {
						sprintf(mybuf2, "%s is now unconcious !!", players[config.i_actionRecvID].c_name);
						if (players[config.i_actionRecvID].i_hp[HP_CUR] < -9) {
							sprintf(mybuf2, "%s is now splatted (truely DEAD!) !!", players[config.i_actionRecvID].c_name);
						} else if (players[config.i_actionRecvID].i_hp[HP_CUR] < -4) {
							sprintf(mybuf2, "%s is now whupped bad !!", players[config.i_actionRecvID].c_name);
						}
						writeLog(mybuf2);
						doSystemAlert(mybuf2);
				  } else if (players[config.i_actionRecvID].i_hp[HP_CUR] <= players[config.i_actionRecvID].i_disabledAt && i > players[config.i_actionRecvID].i_disabledAt) {
						sprintf(mybuf2, "%s is now disabled !!", players[config.i_actionRecvID].c_name);
						writeLog(mybuf2);
						doSystemAlert(mybuf2);
				  }
				  		// if its not an Op. Action, decrement the #attacks
				  if (a_CTSpecialAttacks->checked(1) == 0) {
							// set the defender and weapon used
						i = players[config.i_actionInitID].i_doneAttacks;
						players[config.i_actionInitID].i_whomAttacked[i][0] = config.i_actionRecvID;
						players[config.i_actionInitID].i_whomAttacked[i][1] = config.i_actionWeaponSelected;
if (config.flags.f_debug != 0) { printf("--- aCA: 0 %d:%d:%d\n", i, config.i_actionRecvID, config.i_actionWeaponSelected); }

				  		players[config.i_actionInitID].i_doneAttacks++;
				  		players[config.i_actionInitID].w_weapons[config.i_actionWeaponSelected].i_doneAttacks++;
				  }
				  sprintf(mybuf2, "  %d", (players[config.i_actionInitID].i_noAttacks - players[config.i_actionInitID].i_doneAttacks)); a_CTAttacksLeft->value(mybuf2);
				  if (config.flags.f_logfile != 0) {
						sprintf(mybuf2, "Melee: %s -> %s (HP:%d Bd:%d St:%d In:%d) Cur:%d", players[config.i_actionInitID].c_name, players[config.i_actionRecvID].c_name, l, atoi(a_CTMeleeBleeding->value()), atoi(a_CTMeleeStun->value()), atoi(a_CTMeleeInactive->value()), players[config.i_actionRecvID].i_hp[HP_CUR]);
						writeLog(mybuf2);
				  }
				  sprintf(mybuf2, "@B223@.%s takes HP:%d Bd:%d St:%d In:%d", players[config.i_actionRecvID].c_name, l, atoi(a_CTMeleeBleeding->value()), atoi(a_CTMeleeStun->value()), atoi(a_CTMeleeInactive->value()));
				  i = (players[config.i_actionInitID].i_doneAttacks);
				  if (i < 0) { i = 0; }
				  if (i > 4) { i = 4; }
				  a_CTMessage->text(4+i, mybuf2);
				  setCombatDisplay(2);
				  a_CTallAttacksDone->activate();
				  a_CTMessage->text(1, ""); a_CTMessage->text(2, ""); a_CTMessage->text(3, "");
				  a_CTMessage->text(4, ""); // a_CTMessage->text(5, ""); a_CTMessage->text(6, ""); a_CTMessage->text(7, "");
				  break;

		case 1:	// all melee attacks done
				  if (players[config.i_actionInitID].i_doneAttacks == 0) {
				  		players[config.i_actionInitID].i_doneAttacks++;
				  }
				  reloadNames(1);
				  setCombatDisplay(0);
				  break;

		case 2:	// accept this spell attack
				  i = a_CTSpellEffects->nitems();
				  for (; i>=1; i--) {
						if (a_CTSpellEffects->checked(i) == 1) {
							j = config.i_idList[ID_DEFD][i-1];
if (config.flags.f_debug != 0) { printf("aCA: %d:%d:%d\n", action,i,j); }
				  			k = getHPs(j);
							l = parseDice((char *)a_CTSpellDamage->value());
							if (l != 0) { l += atoi(a_CTSpellDmgMods->value()); }

							if (a_CTSpellSaved->checked(i) == 1 || a_CTSpellModifiers->checked(3) != 0) {		// they didnt save ....
						  		players[j].i_hp[HP_CUR] -= l;
						  		players[j].i_inactive += atoi(a_CTSpellInactive->value());
						  		players[j].i_bleeding += atoi(a_CTSpellBleeding->value());
						  		players[j].i_stun += atoi(a_CTSpellStunned->value());
								players[j].flags.f_isStabilised = players[j].flags.f_autoStablise;	// if they take damage, make them 'un-stabilised'
				  				if (config.flags.f_logfile != 0) {
										sprintf(mybuf2, "Spell: %s -> %s (HP:%d Bd:%d St:%d In:%d) Cur:%d", players[config.i_actionInitID].c_name, players[j].c_name, l, atoi(a_CTSpellBleeding->value()), atoi(a_CTSpellStunned->value()), atoi(a_CTSpellInactive->value()), players[j].i_hp[HP_CUR]);
										writeLog(mybuf2);
				  				}
							} else {
								if (a_CTSpellModifiers->checked(2) == 1) {	// NO Damage
									;
								} else if (a_CTSpellModifiers->checked(1) == 1) {	// 50% Damage
							  		players[j].i_hp[HP_CUR] -= l/2;
							  		players[j].i_inactive += atoi(a_CTSpellInactive->value()) / 2;
							  		players[j].i_bleeding += atoi(a_CTSpellBleeding->value()) / 2;
							  		players[j].i_stun += atoi(a_CTSpellStunned->value()) / 2;
									players[j].flags.f_isStabilised = players[j].flags.f_autoStablise;	// if they take damage, make them 'un-stabilised'
					  				if (config.flags.f_logfile != 0) {
										sprintf(mybuf2, "Spell: %s -> %s (HP:%d Bd:%d St:%d In:%d) Cur:%d", players[config.i_actionInitID].c_name, players[j].c_name, l/2, atoi(a_CTSpellBleeding->value())/2, atoi(a_CTSpellStunned->value())/2, atoi(a_CTSpellInactive->value())/2, players[j].i_hp[HP_CUR]);
										writeLog(mybuf2);
					  				}
								}
							}
				  			if (players[j].i_hp[HP_CUR] <= players[j].i_unconciousAt && k > players[j].i_unconciousAt) {
								sprintf(mybuf2, "%s is now unconcious !!", players[j].c_name);
								if (players[config.i_actionRecvID].i_hp[HP_CUR] < -9) {
									sprintf(mybuf2, "%s is now splatted (truely DEAD!) !!", players[config.i_actionRecvID].c_name);
								} else if (players[config.i_actionRecvID].i_hp[HP_CUR] < -4) {
									sprintf(mybuf2, "%s is now whupped bad !!", players[config.i_actionRecvID].c_name);
								}
								writeLog(mybuf2);
								doSystemAlert(mybuf2);
						  } else if (players[j].i_hp[HP_CUR] <= players[j].i_disabledAt && k > players[j].i_disabledAt) {
								sprintf(mybuf2, "%s is now disabled !!", players[j].c_name);
								writeLog(mybuf2);
								doSystemAlert(mybuf2);
						  }
						}
				  }
				  if (a_CTSpellModifiers->checked(5) == 1) {	// Remove from spell slot count
						i = (int )a_CTSpellLevel->value();
						if (i == 0) { players[config.i_actionInitID].spellsAvailable.i_curLevel0--; }
						if (i == 1) { players[config.i_actionInitID].spellsAvailable.i_curLevel1--; }
						if (i == 2) { players[config.i_actionInitID].spellsAvailable.i_curLevel2--; }
						if (i == 3) { players[config.i_actionInitID].spellsAvailable.i_curLevel3--; }
						if (i == 4) { players[config.i_actionInitID].spellsAvailable.i_curLevel4--; }
						if (i == 5) { players[config.i_actionInitID].spellsAvailable.i_curLevel5--; }
						if (i == 6) { players[config.i_actionInitID].spellsAvailable.i_curLevel6--; }
						if (i == 7) { players[config.i_actionInitID].spellsAvailable.i_curLevel7--; }
						if (i == 8) { players[config.i_actionInitID].spellsAvailable.i_curLevel8--; }
						if (i == 9) { players[config.i_actionInitID].spellsAvailable.i_curLevel9--; }
				  }
				  players[config.i_actionInitID].i_doneAttacks++;
				  setCombatDisplay(2);
				  a_CTSpellAllDone->activate();
				  break;

		case 3:	// all spell attacks done
				  reloadNames(1);
				  setCombatDisplay(0);
				  break;

		case 4:	// other
				  players[config.i_actionInitID].i_doneAttacks++;
				  switch (config.i_actionSelected) {
						case 8:	// healing
								  if (a_CTOtherActions->nchecked() == 0) {
										doSystemAlert("Please select a person who is effected by this action !!!!");
										return;
								  }
								  config.i_actionRecvID = -1;
								  for (i=a_CTOtherActions->nitems(); i>=0; i--) {
									  if (a_CTOtherActions->checked(i) == 1) {
										  config.i_actionRecvID = config.i_idList[ID_DEFD][i-1];
										  if (a_CTOtherFlags->checked(1) != 0) {
										  	players[config.i_actionRecvID].flags.f_isStabilised = 1;
										  }
										  l = parseDice((char *)a_CTOtherHealHP->value());
										  players[config.i_actionRecvID].i_hp[HP_CUR] += l;
										  players[config.i_actionRecvID].i_inactive -= atoi(a_CTOtherHealInactive->value());
										  players[config.i_actionRecvID].i_bleeding -= atoi(a_CTOtherHealBleeding->value());
										  players[config.i_actionRecvID].i_stun -= atoi(a_CTOtherHealStun->value());
										  if (players[config.i_actionRecvID].i_hp[HP_CUR] > getMaxHPs(config.i_actionRecvID)) { players[config.i_actionRecvID].i_hp[HP_CUR] = getMaxHPs(config.i_actionRecvID); }
										  if (players[config.i_actionRecvID].i_inactive < 0) { players[config.i_actionRecvID].i_inactive = 0; }
										  if (players[config.i_actionRecvID].i_bleeding < 0) { players[config.i_actionRecvID].i_bleeding = 0; }
										  if (players[config.i_actionRecvID].i_stun < 0) { players[config.i_actionRecvID].i_stun = 0; }
						  				  if (config.flags.f_logfile != 0) {
												sprintf(mybuf2, "Heal: %s -> %s (HP:%d Bd:%d St:%d In:%d Sb:%d) Cur:%d", players[config.i_actionInitID].c_name, players[config.i_actionRecvID].c_name, l, atoi(a_CTOtherHealBleeding->value()), atoi(a_CTOtherHealStun->value()), atoi(a_CTOtherHealInactive->value()), a_CTOtherFlags->checked(1), players[config.i_actionRecvID].i_hp[HP_CUR]);
												writeLog(mybuf2);
						  				  }
					  				  }
				  				  }
								  if (a_CTOtherFlags->checked(2) != 0 && config.i_actionRecvID != -1) { 		// remove from spell slot
										i = (int )a_CTOtherSpellLevel->value();
										if (i == 0) { players[config.i_actionInitID].spellsAvailable.i_curLevel0--; }
										if (i == 1) { players[config.i_actionInitID].spellsAvailable.i_curLevel1--; }
										if (i == 2) { players[config.i_actionInitID].spellsAvailable.i_curLevel2--; }
										if (i == 3) { players[config.i_actionInitID].spellsAvailable.i_curLevel3--; }
										if (i == 4) { players[config.i_actionInitID].spellsAvailable.i_curLevel4--; }
										if (i == 5) { players[config.i_actionInitID].spellsAvailable.i_curLevel5--; }
										if (i == 6) { players[config.i_actionInitID].spellsAvailable.i_curLevel6--; }
										if (i == 7) { players[config.i_actionInitID].spellsAvailable.i_curLevel7--; }
										if (i == 8) { players[config.i_actionInitID].spellsAvailable.i_curLevel8--; }
										if (i == 9) { players[config.i_actionInitID].spellsAvailable.i_curLevel9--; }
								  }
								  break;

						case 9:	// turning
				  				  if (config.flags.f_logfile != 0) {
										sprintf(mybuf2, "Turning Attempt: %s", players[config.i_actionInitID].c_name);
										writeLog(mybuf2);
				  				  }
								  break;

						case 10:	// skill check -- remove the last d20 roll
				  				  if (config.flags.f_logfile != 0) {
										sprintf(mybuf2, "Skill Check: %s", players[config.i_actionInitID].c_name);
										writeLog(mybuf2);
				  				  }
								  for (i=7; i>-1; i--) {
										if (players[config.i_actionRecvID].i_d20Rolls[i] > 0) {
											players[config.i_actionRecvID].i_d20Rolls[i] = 0;
											break;
										}
								  }
								  break;
				  }
				  reloadNames(1);
				  setCombatDisplay(0);
				  break;
	}
}

/********************************************************************************/
int getMaxHPs(int p) {
	return (players[p].i_hp[HP_MAX] + players[p].i_hp[HP_TMP] + getAdjustment(p, MOD_HP));
}

/********************************************************************************/
int getHPs(int p) {
	return (players[p].i_hp[HP_CUR] + players[p].i_hp[HP_TMP] - players[p].i_hp[HP_NL] + getAdjustment(p, MOD_HP));
}

/********************************************************************************/
int getHPpercentage(int p) {
	int i; 

	i = getHPs(p);
if (config.flags.f_debug != 0) { printf("gHPP: %d %d:%d\n", p, i, players[p].i_hp[HP_MAX]); }
	if (i < 0) {
		return (10 * i);
	} else if (i == 0) {
		return 0;
	} else {
		return int((i * 100) / players[p].i_hp[HP_MAX]);
	}
}

/********************************************************************************/
int calcAction(int action) {
	int i, j, k, a;

	switch (action) {
		case 0:	// melee
				  a = config.i_actionWeaponSelected;
				  if (a == -1) {
						doSystemAlert("Please select a Weapon to use !!!");
				  }
				  a_CTMeleeDamage->value(""); a_CTMeleeInactive->value(""); a_CTMeleeBleeding->value(""); a_CTMeleeStun->value("");
				  a_CTMessage->text(4, "");
				  i = (atoi(a_CTMeleeBAB->value()) + atoi(a_CTMeleeBABAdjustments->value()) - (atoi(a_CTMeleeAC->value()) + atoi(a_CTMeleeACAdjustments->value()))) + atoi(a_CTMeleeRoll->value());
				  j = atoi(a_CTMeleeRoll->value());
				  if (config.flags.f_oneMissTwentyHits != 0) {
						if (j == 1) { i = -1; }		// force a miss
						if (j == 20) { i = 1; }		// force a hit
if (config.flags.f_debug != 0) { printf("cA: 1M %d:%d\n", j, i); }
				  }
if (config.flags.f_debug != 0) { printf("cA: PC %d\n", i); }
				  if (i >= 0) {	// did they HIT them
						parseCritical(players[config.i_actionInitID].w_weapons[a].c_critical, atoi(a_CTMeleeRoll->value()), &i, &j, &k);
						if (atoi(a_CTMeleeRoll->value()) >= i && atoi(a_CTMeleeRoll->value()) <= j && players[config.i_actionRecvID].flags.f_ignoreCriticals == 0) {
							if (config.i_actionInitID >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0) {
				  				i = atoi(a_CTMeleeBAB->value()) + atoi(a_CTMeleeBABAdjustments->value()) - atoi(a_CTMeleeAC->value()) - atoi(a_CTMeleeACAdjustments->value()) + getRND(20);
if (config.flags.f_debug != 0) { printf("cA: CT %d\n", i); }
								if (i >= 0) {
									j = 0;
									for (; k > 0; k--) {
										j += parseDice(players[config.i_actionInitID].w_weapons[a].c_damage);
									}
									a_CTMessage->text(4, "@B88@.!!! A CRITICAL Success ... you smacked 'em bad !!!");
								} else {
									j = parseDice(players[config.i_actionInitID].w_weapons[a].c_damage);
									a_CTMessage->text(4, "!!! Almost a CRITICAL Success ... bad luck !!!");
								}
								sprintf(mybuf1, "%d", j); a_CTMeleeDamage->value(mybuf1);
							} else {
								a_CTMessage->text(4, "@B88@.!!! A possible CRITICAL Success ... roll again !!!");
							}
						} else {
							if (config.i_actionInitID >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0) {
								sprintf(mybuf1, "%d", parseDice(players[config.i_actionInitID].w_weapons[a].c_damage)); a_CTMeleeDamage->value(mybuf1);
							}
							a_CTMessage->text(4, "@B63@.!!! Success ... you've hit them !!!");
						}
				  } else {
						a_CTMessage->text(4, "!!! You've missed them ... bad luck !!!");
				  }
				  break;

		case 1:	// spell
				  i = a_CTSpellEffects->nitems();
				  a = atoi(a_CTSpellDC->value());
				  for (; i >=1; i--) {
						if (a_CTSpellEffects->checked(i) == 1) {
							j = config.i_idList[ID_DEFD][i-1];
							if (j >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0) {
								k = getRND(20);
								switch (config.i_actionSelected) {
									case 2:	// damage spell
											  break;
									case 3:	// WILL spell
											  k += players[j].i_savingThrows[SAVE_WISDOM] + getAdjustment(j, MOD_SAVE) + getAdjustment(j, MOD_SAVE_WILL);
											  break;
									case 4:	// REF spell
											  k += players[j].i_savingThrows[SAVE_REFLEX] + getAdjustment(j, MOD_SAVE) + getAdjustment(j, MOD_SAVE_REFLEX);
											  break;
									case 5:	// FORT spell
											  k += players[j].i_savingThrows[SAVE_FORTITUDE] + getAdjustment(j, MOD_SAVE) + getAdjustment(j, MOD_SAVE_FORTITUDE);
											  break;
									case 6:	// touch attk spell
											  k = players[j].i_armorClass[AC_TOUCH] + getAdjustment(j, MOD_AC) - 1;
				  							  if (config.flags.f_oneMissTwentyHits != 0) {
													if (a == 1) { k = -100; }		// force a fail
													if (a == 20) { k = 100; }		// force a hit
				  							  }
											  break;
									case 7:	// ranged touch attk spell
											  k = players[j].i_armorClass[AC_TOUCH] + getAdjustment(j, MOD_AC) - 1;
				  							  if (config.flags.f_oneMissTwentyHits != 0) {
													if (a == 1) { k = -100; }		// force a fail
													if (a == 20) { k = 100; }		// force a hit
				  							  }
											  break;
								}
								if (a_CTSpellModifiers->checked(3) != 0) {		// No Save so force a fail
									k = -100;
								}
if (config.flags.f_debug != 0) { printf("cA: %d:%d:%d\n", i, j, k); }		// Handle Spell Resistance
								if (a_CTSpellModifiers->checked(4) == 1 && players[j].i_spellResistance > 0) {
									if (j >= MAX_PLAYERS || config.flags.f_rollAllRolls != 0) {
										if (players[j].i_spellResistance > (getRND(20) + a_CTSpellLevel->value())) {
											k = 100;
											a_CTSpellEffects->checked(i, 0);		// Uncheck it
										}
									} else {
										a_CTMessage->text(2, "@B88@.!!! Please roll Spell Resistances !!!");
									}
if (config.flags.f_debug != 0) { printf("cA: SR: %d:%d:%d\n", i, j, k); }
								}
								if (k >= (atoi(a_CTSpellDC->value()) + atoi(a_CTSpellMods->value()))) {
									a_CTSpellSaved->checked(i, 0);
										// if they have evasion & it's a REF spell they take no damage
									if (players[j].flags.f_evasion != 0 && config.i_actionSelected == 4 && a_CTSpellModifiers->checked(1) == 1) {
										a_CTSpellEffects->checked(i, 0);		// Uncheck it
									}
								} else {			// nope, didnt save
									a_CTSpellSaved->checked(i, 1);
								}
							} else {
								a_CTSpellSaved->checked(i, 1);
								if (players[j].i_spellResistance > 0) {
									a_CTMessage->text(2, "@B88@.!!! Please roll Spell Resistances !!!");
								}
							}
						}
				  }
				  break;
		case 2:	// roll another d20
				  sprintf(mybuf2, "%d", getRND(20));
				  a_CTMeleeRoll->value(mybuf2);
				  break;
	}
	return 0;
}

/********************************************************************************/
void actionQkMonsters(int action) {
int i, j, k;
char b1[30], b2[30], *cp;
FILE *nf;

	i = a_EdtQkMonList->value()-1;
	if (i < 0 && (action != 3 && action != -1)) {
		doSystemAlert("Please select a monster first !!!");
		return;
	}

if (config.flags.f_debug != 0) { printf("aQM: %d:%d:%d\n", i, config.i_idList[ID_QMON][i], config.i_editID); }
	switch (action) {
		case -1:		// standard monster save/update
				  savePlayer(config.i_editID);
				  if (i > -1) {
				  		if (doAreYouSure("This will over-write the selected Standard Monster ?") == 1) {
							j = config.i_idList[ID_QMON][i];
						  	memcpy(&qkmonsters[j].c_name[0], &players[config.i_editID].c_name[0],  sizeof(ADMPplayer));
						} else {
							return;
						}
				  } else {
					  for (j = 0; j < MAX_QKMONSTERS; j++) {
							if (strncmp(qkmonsters[j].c_name, "-- empty slot", 13) == 0 || qkmonsters[j].c_name[0] == '\0') {
if (config.flags.f_debug != 0) { printf("aQM: 0 %d:%d:%d:%d:%s\n", j, config.i_idList[ID_QMON][j], (int )a_EdtQkMonAddCtr->value(), config.i_editID, qkmonsters[j].c_name); }
					  			memcpy(&qkmonsters[j].c_name[0], &players[config.i_editID].c_name[0],  sizeof(ADMPplayer));
								qkmonsters[j].flags.f_srdMonster = -1;
								a_EdtQkMonAddCtr->value(1);
								break;
							}
					  }
				  }
				  if (qkmonsters[j].c_name[1] == ' ' && qkmonsters[j].c_name[2] == '-') {
						memmove(&qkmonsters[j].c_name[0], &qkmonsters[j].c_name[4], 25);
				  }
				  reloadNames(0);
				  if (config.f_version > 1.10) {
						writeAllStdMonsterData("stdmonster");
				  }
				  // TODO - how to write out a note
					sprintf(mybuf2, "%d", qkmonsters[j].i_stdMonsterID);
					pdbDeleteRecord("stdmonster.snt", mybuf2, NULL, NULL);
					if ((nf = fopen("stdmonster.snt", "a+")) != NULL) {
						strncpy(mybuf1, edit_notes_buffer->text(),2047);
						while ((cp = strstr(mybuf1, "\n")) != NULL) {
							*cp = '|';
						}
						fprintf(nf, "+%d|%d|%s|\n", qkmonsters[j].i_stdMonsterID, qkmonsters[j].i_stdMonsterID, mybuf1);
						fclose(nf);
					}
				  break;

		case 0:	// add a std monster to the list of active monsters
				  if (config.flags.f_debug != 0) { printf("aQM: 0 %d:%d:%d\n", i, config.i_idList[ID_QMON][i], (int )a_EdtQkMonAddCtr->value()); }
				   i_srdMonsterID = config.i_idList[ID_QMON][i];
					memcpy(&players[MAX_MEMBERS+1].c_name[0], &qkmonsters[config.i_idList[ID_QMON][i]].c_name[0], sizeof(ADMPplayer));
					duplicatePlayer(MAX_MEMBERS+1, (int )a_EdtQkMonAddCtr->value());
					a_EdtQkMonAddCtr->value(1);
					reloadNames(0);
				  break;

		case 1:	// edit a std monster
				  a_EdtQkMonList->deactivate();
				  a_EdtQkMonstSearch->deactivate();
				  aEdit_AddQkMonsters->deactivate();
				  aEdit_EditQkMonsters->deactivate();
//				  aEdit_SaveQkMonsters->deactivate();
				  a_EdtQkMonCRSearch->deactivate();
				  aEdit_QkMonstersWildSearch->deactivate();
				  a_EdtQkMonAddCtr->deactivate();
				  aEdt_Accept->activate();
				  aEdt_Cancel->activate();
				  aEdt_QkMonsterAdd->deactivate();
				  memcpy(&players[MAX_MEMBERS+1].c_name[0], &qkmonsters[config.i_idList[ID_QMON][i]].c_name[0], sizeof(ADMPplayer));
if (config.flags.f_debug != 0) { printf("aQM: 1 %s (%d)\n", players[MAX_MEMBERS+1].c_name, config.i_idList[ID_QMON][i]); }
				  editPlayer(MAX_MEMBERS+1);
				  break;

		case 2:		// save them all but not sure if we need this bit
				  memcpy(&qkmonsters[config.i_idList[ID_QMON][i]].c_name[0], &players[MAX_MEMBERS+1].c_name[0], sizeof(ADMPplayer));
if (config.flags.f_debug != 0) { printf("aQM: 2 %s\n", players[MAX_MEMBERS+1].c_name); }
if (config.flags.f_debug != 0) { printf("aQM: 2 %s\n", qkmonsters[config.i_idList[ID_QMON][i]].c_name); }
		  		  sprintf(mybuf2, "%sqkmonst.dat", systemDir);
				  if ((openf = fopen(mybuf2, "w")) != NULL) {
						fwrite(&qkmonsters[0].c_name[0], sizeof(ADMPplayer) * MAX_QKMONSTERS, 1, openf);
						fchmod(i, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
				  		fclose(openf);
				  }
				  break;

		case 3:		// advanced search
				  j=0;
				  strncpy(b1, a_EdtQkMonstSearch->value(), 20); b1[19] = '\0';  mytolower(b1);
// if (config.flags.f_debug != 0) { printf("sQM: 3 %s : %s\n", b1, a_EdtQkMonstSearch->value()); }
				  a_EdtQkMonList->clear();
				  for (int i=0; i<MAX_QKMONSTERS; i++) {
					strncpy(b2, qkmonsters[i].c_name, 30); b2[29] = '\0'; mytolower(b2);
// if (config.flags.f_debug != 0) { printf("sQM: 3 %s : %s\n", b1, b2); }
			  		if (strstr(b2, b1) != NULL) {
						k=0;
						if (a_EdtQkMonCRSearch->value() > 0) {
							if ((int )a_EdtQkMonCRSearch->value() == (int )qkmonsters[i].f_challengeRating) {
								k = 1;
							}
						} else {
							k = 1;
						}
						if (k == 1) {
				  			sprintf(mybuf2, "%s (CR: %.1f)", qkmonsters[i].c_name, qkmonsters[i].f_challengeRating);
				  			a_EdtQkMonList->add(mybuf2);
				  			config.i_idList[ID_QMON][j++] = i;
				  			if (j > MAX_MEMBERS) {
				  				return;
				  			}
			  			}
			  		}
				  }
				  break;
	}
}

/********************************************************************************/
void searchQkMonsters(const char *name) {
	int i, j=0, k, l=0;

	a_EdtQkMonList->clear();
	qkMonsterAction(0);

if (config.flags.f_debug != 0) { printf("sQM: 0 %s:%d\n", name, a_EdtQkMonCREnvironment->value()); }
	k = strlen(name);
	for (i=0; i<MAX_QKMONSTERS; i++) {
		if (strncasecmp(qkmonsters[i].c_name, name, k) == 0) {
if (config.flags.f_debug != 0) { printf("sQM: 0 %s %.2f %d %d\n", qkmonsters[i].c_name, qkmonsters[i].f_challengeRating, qkmonsters[i].c_environment, a_EdtQkMonCREnvironment->value()); }
			j=0;
			if (a_EdtQkMonCRSearch->value() > 0) {
				if ((int )a_EdtQkMonCRSearch->value() == (int )qkmonsters[i].f_challengeRating) {
					if (a_EdtQkMonCREnvironment->value() > 0) {
						if (qkmonsters[i].c_environment == a_EdtQkMonCREnvironment->value()) {
							j = 1;
						}
					} else {
						j = 1;
					}
				}
			} else {
				if (a_EdtQkMonCREnvironment->value() > 0) {
					if (qkmonsters[i].c_environment == a_EdtQkMonCREnvironment->value()) {
						j = 1;
					}
				} else {
					j = 1;
				}
			}
			if (j == 1) {
				sprintf(mybuf2, "%s (CR: %.1f)", qkmonsters[i].c_name, qkmonsters[i].f_challengeRating);
				a_EdtQkMonList->add(mybuf2);
				config.i_idList[ID_QMON][l++] = i;
				if (l > MAX_MEMBERS) {
					return;
				}
			}
		}
	}
}

/********************************************************************************/
void doOtherAction(int action) {
	int i=0, j=0;
	
if (config.flags.f_debug != 0) { printf("dOA: %d\n", action); }

	switch (action) {
		case 0:	// reset players
				  for (i=0; i<MAX_PLAYERS; i++) {
						players[i].flags.f_isStabilised = players[i].i_hp[HP_TMP] = players[i].i_hp[HP_NL] = 0;
						players[i].i_initiative[1] = 0;
						players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX];
						players[i].i_bleeding = players[i].i_doneAttacks = players[i].i_stun = players[i].i_inactive = 0;
							// players[i].flags.f_disabled = removed from the line above
						for (j=0; j<MAX_ATTACKS+2; j++) {
							players[i].w_weapons[j].i_doneAttacks = 0;
							players[i].i_whomAttacked[j][0] = -1;
						}
				  }
				  break;
		case 1:	// clean players
				  break;
		case 2:	// remove all players
				  if (doAreYouSure("Really remove all players ?") == 1) {
				  		memset(&players[0].c_name[0], '\0', sizeof(ADMPplayer)*MAX_PLAYERS);
				  }
				  break;
		case 3:	// reset monsters
				  for (i=MAX_PLAYERS; i<MAX_MONSTERS; i++) {
						players[i].flags.f_isStabilised = players[i].i_hp[HP_TMP] = players[i].i_hp[HP_NL] = 0;
						players[i].i_initiative[1] = getRND(20);
						players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX];
						players[i].i_bleeding = players[i].i_doneAttacks = players[i].flags.f_disabled = players[i].i_stun = players[i].i_inactive = 0;
						for (j=0; j<MAX_ATTACKS+2; j++) {
							players[i].w_weapons[j].i_doneAttacks = 0;
							players[i].i_whomAttacked[j][0] = -1;
						}
				  }
				  break;
		case 4:	// clean monsters
				  break;
		case 5:	// remove all monsters
				  memset(&players[MAX_PLAYERS].c_name[0], '\0', sizeof(ADMPplayer)*MAX_MONSTERS);
				  break;
		case 6:	// clear triggers
				  for (i=0; i<MAX_TRIGGERS; i++) {
						sprintf(config.action_msgs[i].c_message, "-- empty slot %d", i);
						config.action_msgs[i].i_noRounds = config.action_msgs[i].i_display = 0;
				  }
					a_OTTriggerList->clear();
					for (i=0; i<MAX_TRIGGERS; i++) {
						a_OTTriggerList->add(config.action_msgs[i].c_message);
					}
				  break;
		case 7:	// clear adjustments
				  for (i=0; i<MAX_MODIFIERS; i++) {
						sprintf(config.action_mods[i].c_message, "-- empty slot %d", i);
						config.action_mods[i].i_noRounds = 0;
						config.action_mods[i].i_whom[0] = config.action_mods[i].i_whom[1] = config.action_mods[i].i_idFollows = -255;
						config.action_mods[i].i_data[0][0] = config.action_mods[i].i_data[0][2] = 0; config.action_mods[i].i_data[0][1] = -1;
						config.action_mods[i].i_data[1][0] = config.action_mods[i].i_data[1][2] = 0; config.action_mods[i].i_data[0][1] = -1;
						config.action_mods[i].i_data[2][0] = config.action_mods[i].i_data[2][2] = 0; config.action_mods[i].i_data[0][1] = -1;
						config.action_mods[i].i_data[3][0] = config.action_mods[i].i_data[3][2] = 0; config.action_mods[i].i_data[0][1] = -1;
						config.action_mods[i].i_data[4][0] = config.action_mods[i].i_data[4][2] = 0; config.action_mods[i].i_data[4][1] = -1;
				  }
					a_OTAdjustList->clear();
					for (i=0; i<MAX_MODIFIERS; i++) {
						a_OTAdjustList->add(config.action_mods[i].c_message);
					}
				  break;
		case 8:	// reset for a new combat ... reset the players and clear any triggers/adjustments etc
				  doOtherAction(0);
				  doOtherAction(3);
				  doOtherAction(6);
				  doOtherAction(7);
				  doOtherAction(10);
				  doOtherAction(11);
				  doInitiativeAction(1);
				  config.i_roundCount = 1; aSys_Rounds->value("1");
				  break;
		case 9:	// monster HP based on CR
				  for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
							players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX] = parseDice(players[i].c_hitDice);
						}
				  }
				  break;
		case 10: // clear auto-damage lists
				  a_OTADList->clear();
				  for (i=0; i<MAX_AUTODAMAGE; i++) {
						config.auto_damage[i].i_initiatorID = config.auto_damage[i].i_whom[0] = -1;
						config.auto_damage[i].i_noRounds = config.auto_damage[i].i_type = 0;
						config.auto_damage[i].c_damage[0] = '\0';
						sprintf(config.auto_damage[i].c_description, "-- empty slot %d", i);
						config.auto_damage[i].flags.f_flag1 = config.auto_damage[i].flags.f_flag2 = 0;
						config.auto_damage[i].flags.f_flag3 = config.auto_damage[i].flags.f_flag4 = 0;
						config.auto_damage[i].flags.f_flag5 = config.auto_damage[i].flags.f_flag6 = 0;
						config.auto_damage[i].flags.f_flag7 = config.auto_damage[i].flags.f_flag8 = 0;
						config.auto_damage[i].i_doneRounds = 0;
				  		a_OTADList->add(config.auto_damage[i].c_description);
				  }
				  break;
		case 11: // new day stuff ... healing & spell slots
				  for (i=0; i<MAX_PLAYERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
							players[i].i_hp[HP_CUR] += players[i].i_levels[0] + players[i].i_levels[1] + players[i].i_levels[2] + players[i].i_levels[3];
							if (players[i].i_hp[HP_CUR] > players[i].i_hp[HP_MAX]) {
								players[i].i_hp[HP_CUR] = players[i].i_hp[HP_MAX];
							}
							if (players[i].spellsAvailable.i_maxLevel0 < 0) { players[i].spellsAvailable.i_maxLevel0 = 0; }
							if (players[i].spellsAvailable.i_maxLevel1 < 0) { players[i].spellsAvailable.i_maxLevel1 = 0; }
							if (players[i].spellsAvailable.i_maxLevel2 < 0) { players[i].spellsAvailable.i_maxLevel2 = 0; }
							if (players[i].spellsAvailable.i_maxLevel3 < 0) { players[i].spellsAvailable.i_maxLevel3 = 0; }
							if (players[i].spellsAvailable.i_maxLevel4 < 0) { players[i].spellsAvailable.i_maxLevel4 = 0; }
							if (players[i].spellsAvailable.i_maxLevel5 < 0) { players[i].spellsAvailable.i_maxLevel5 = 0; }
							if (players[i].spellsAvailable.i_maxLevel6 < 0) { players[i].spellsAvailable.i_maxLevel6 = 0; }
							if (players[i].spellsAvailable.i_maxLevel7 < 0) { players[i].spellsAvailable.i_maxLevel7 = 0; }
							if (players[i].spellsAvailable.i_maxLevel8 < 0) { players[i].spellsAvailable.i_maxLevel8 = 0; }
							if (players[i].spellsAvailable.i_maxLevel9 < 0) { players[i].spellsAvailable.i_maxLevel9 = 0; }
							players[i].spellsAvailable.i_curLevel0 = players[i].spellsAvailable.i_maxLevel0;
							players[i].spellsAvailable.i_curLevel1 = players[i].spellsAvailable.i_maxLevel1;
							players[i].spellsAvailable.i_curLevel2 = players[i].spellsAvailable.i_maxLevel2;
							players[i].spellsAvailable.i_curLevel3 = players[i].spellsAvailable.i_maxLevel3;
							players[i].spellsAvailable.i_curLevel4 = players[i].spellsAvailable.i_maxLevel4;
							players[i].spellsAvailable.i_curLevel5 = players[i].spellsAvailable.i_maxLevel5;
							players[i].spellsAvailable.i_curLevel6 = players[i].spellsAvailable.i_maxLevel6;
							players[i].spellsAvailable.i_curLevel7 = players[i].spellsAvailable.i_maxLevel7;
							players[i].spellsAvailable.i_curLevel8 = players[i].spellsAvailable.i_maxLevel8;
							players[i].spellsAvailable.i_curLevel9 = players[i].spellsAvailable.i_maxLevel9;
							players[i].i_bleeding = players[i].i_doneAttacks = players[i].i_stun = players[i].i_inactive = 0;
							for (j=0; j<MAX_ATTACKS+2; j++) {
								players[i].w_weapons[j].i_doneAttacks = 0;
								players[i].i_whomAttacked[j][0] = -1;
							}
						}
				  }
				  doOtherAction(6);
				  doOtherAction(7);
				  doOtherAction(10);
				  doInitiativeAction(1);
				  config.i_roundCount = 1; aSys_Rounds->value("1");
				  writeLog("**** Players reset for a new day");
				  break;
		case 20:
				  a_OStatusList->clear();
				  sprintf(mybuf1, "Round : %d", config.i_roundCount); a_OStatusList->add(mybuf1);
				  a_OStatusList->add("");
				  for (i=0; i<MAX_PLAYERS; i++) {
						if (players[i].i_hp[HP_CUR] < 1) { j = players[i].i_hp[HP_CUR] * 10; }
						else { j = (players[i].i_hp[HP_CUR] * 100) / players[i].i_hp[HP_MAX]; }
						j = getHPpercentage(i);
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
							sprintf(mybuf1, "%-25.25s HP:%-3d/%-3d/%-3d %3d%%  AC:%-2d/%-2d/%-2d  Bld:%-2d Stn:%-2d Ina:%-2d Sbl:%-2d Lvl:%d (%d)", players[i].c_name, /* players[i].i_hp[HP_CUR] */ getHPs(i), players[i].i_hp[HP_MAX], players[i].i_hp[HP_NL], j, players[i].i_armorClass[AC_NORMAL], players[i].i_armorClass[AC_TOUCH], players[i].i_armorClass[AC_FLATFOOTED], players[i].i_bleeding, players[i].i_stun, players[i].i_inactive, players[i].flags.f_isStabilised, (players[i].i_levels[0]+players[i].i_levels[1]+players[i].i_levels[2]+players[i].i_levels[3]), players[i].i_xp[MAX]);
							if ((i % 2) == 0) {
								sprintf(mybuf2, "@B223@.%s", mybuf1);
								a_OStatusList->add(mybuf2);
							} else {
								a_OStatusList->add(mybuf1);
							}
						}
				  }
				  a_OStatusList->add("");
				  for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
							if (players[i].i_hp[HP_CUR] < 1) { j = players[i].i_hp[HP_CUR] * 10; }
							else { j = (players[i].i_hp[HP_CUR] * 100) / players[i].i_hp[HP_MAX]; }
							sprintf(mybuf1, "%-25.25s HP:%-3d/%-3d/%-3d %3d%%  AC:%-2d/%-2d/%-2d  Bld:%-2d Stn:%-2d Ina:%-2d Sbl:%-2d CR:%.1f", players[i].c_name, players[i].i_hp[HP_CUR], players[i].i_hp[HP_MAX], players[i].i_hp[HP_NL], j, players[i].i_armorClass[AC_NORMAL], players[i].i_armorClass[AC_TOUCH], players[i].i_armorClass[AC_FLATFOOTED], players[i].i_bleeding, players[i].i_stun, players[i].i_inactive, players[i].flags.f_isStabilised, players[i].f_challengeRating);
							if ((i % 2) == 0) {
								sprintf(mybuf2, "@B95@.%s", mybuf1);
								a_OStatusList->add(mybuf2);
							} else {
								a_OStatusList->add(mybuf1);
							}
						}
				  }
				  break;
	}
	reloadNames(0);
}

/********************************************************************************/
void calcCombatMods(int action) {
	int i=0,j=0,bab=0,ac=0;

if (config.flags.f_debug != 0) { printf("cCM: %d %d\n", action, a_CTCombatMods->value()); }

	switch (action) {
		case 0:	// melee
				for (i=0; ; i++) {
					if (combatmods[i].c_message[0] == '\0') { break; }
					if (a_CTCombatMods->checked(i+1) == 1) {
						switch (config.i_actionSelected) {
							case 0:	// melee
									  if (combatmods[i].i_modifications[0][1] == 0) {
									  		bab += combatmods[i].i_modifications[0][0];
									  } else if (combatmods[i].i_modifications[0][1] == 1) {
									  		ac += combatmods[i].i_modifications[0][0];
									  }
									  break;
			
							case 1:	// missile
									  if (combatmods[i].i_modifications[1][1] == 0) {
									  		bab += combatmods[i].i_modifications[1][0];
									  } else if (combatmods[i].i_modifications[1][1] == 1) {
									  		ac += combatmods[i].i_modifications[1][0];
									  }
									  break;
						}
					}
				}
				sprintf(mybuf2, "%d", bab + getAdjustment(config.i_actionInitID, MOD_BAB)); a_CTMeleeBABAdjustments->value(mybuf2);
				sprintf(mybuf2, "%d", ac + getAdjustment(config.i_actionRecvID, MOD_AC)); a_CTMeleeACAdjustments->value(mybuf2);
				if (a_CTCombatMods->value() == 8) {		// Surprised/flat-footed
if (config.flags.f_debug != 0) { printf("cCM: %d %d\n", a_CTCombatMods->checked(8), a_CTCombatMods->value()); }
					if (a_CTCombatMods->checked(8) == 1) {
						sprintf(mybuf2, "%d", players[config.i_actionRecvID].i_armorClass[AC_FLATFOOTED]); a_CTMeleeAC->value(mybuf2);
					} else {
						sprintf(mybuf2, "%d", players[config.i_actionRecvID].i_armorClass[AC_NORMAL]); a_CTMeleeAC->value(mybuf2);
					}
				}
				if (a_CTCombatMods->checked(4) == 1 || a_CTCombatMods->checked(7) == 1 || a_CTCombatMods->checked(8) == 1 || a_CTCombatMods->checked(9) == 1 || a_CTCombatMods->checked(10) == 1) {
					a_CTMessage->text(2, "!!! You may need to adjust their A/C by their DEX !!!");
				}
				break;
		case 1:	// spells
				j = a_CTSpellModifiers->value();
if (config.flags.f_debug != 0) { printf("cCM: %d %d %d A\n", action, j, a_CTSpellModifiers->checked(j)); }
				if (j == 1) {		// Save = 50%
					a_CTSpellModifiers->checked(2, 0);
					a_CTSpellModifiers->checked(3, 0);
				} else if (j == 2) {	// Save = nothing
					a_CTSpellModifiers->checked(1, 0);
					a_CTSpellModifiers->checked(3, 0);
				} else if (j == 3) {	// NO Save
					a_CTSpellModifiers->checked(1, 0);
					a_CTSpellModifiers->checked(2, 0);
				}
if (config.flags.f_debug != 0) { printf("cCM: %d %d %d D\n", action, j, a_CTSpellModifiers->checked(j)); }
				if (a_CTSpellModifiers->checked(1) == 0 && a_CTSpellModifiers->checked(2) == 0 && a_CTSpellModifiers->checked(3) == 0) {
					a_CTSpellModifiers->checked(1, 1);		// you must have something checked
				}
				break;
	}
}

/********************************************************************************/
void doSpecialAttacks() {

if (config.flags.f_debug != 0) { printf("dSA: %d:%d:%d:%d:%d\n", a_CTSpecialAttacks->checked(1), a_CTSpecialAttacks->checked(2), a_CTSpecialAttacks->checked(3), a_CTSpecialAttacks->checked(4), a_CTSpecialAttacks->value()); }

	switch (a_CTSpecialAttacks->value()) {
		case 1:		// Op Act
				  break;
		case 2:		// Subdual Damage
				  break;
		case 3:		// Touch Attack
					if (a_CTSpecialAttacks->checked(4) == 1) { a_CTSpecialAttacks->checked(4, 0); } // only one of these
					sprintf(mybuf2, "%d", players[config.i_actionInitID].i_bab[BAB_MELEE][0]); a_CTMeleeBAB->value(mybuf2);
					if (a_CTSpecialAttacks->checked(3) == 1) { 	// set the A/C for a TA
						sprintf(mybuf2, "%d", players[config.i_actionRecvID].i_armorClass[AC_TOUCH]); a_CTMeleeAC->value(mybuf2);
					} else {
						sprintf(mybuf2, "%d", players[config.i_actionRecvID].i_armorClass[AC_NORMAL]); a_CTMeleeAC->value(mybuf2);
					}
				  break;
		case 4:		// Ranged touch attack
					if (a_CTSpecialAttacks->checked(3) == 1) { a_CTSpecialAttacks->checked(3, 0); } // only one of these
					sprintf(mybuf2, "%d", players[config.i_actionInitID].i_bab[BAB_RANGED][0]); a_CTMeleeBAB->value(mybuf2);
					if (a_CTSpecialAttacks->checked(4) == 1) { 	// set the A/C for a TA
						sprintf(mybuf2, "%d", players[config.i_actionRecvID].i_armorClass[AC_TOUCH]); a_CTMeleeAC->value(mybuf2);
					} else {
						sprintf(mybuf2, "%d", players[config.i_actionRecvID].i_armorClass[AC_NORMAL]); a_CTMeleeAC->value(mybuf2);
					}
				  break;
	}
	if (a_CTSpecialAttacks->checked(3) == 0 && a_CTSpecialAttacks->checked(4) == 0) { // set weapon BAB
		sprintf(mybuf2, "%d", getWeaponAttackBonus(config.i_actionInitID, config.i_actionWeaponSelected)); a_CTMeleeBAB->value(mybuf2);
	}
}

/********************************************************************************/
void doTriggerAction(int action) {
	int i = a_OTTriggerList->value() - 1;

	switch (action) {
		case 0:		// edit/add
					a_OTTriggerList->deactivate();
					a_OTTriggerRounds->activate();
					a_OTTriggerMessage->activate();
					a_OTTriggerEveryRound->activate();
					a_OTTriggerEvery->activate();
					a_OTTriggerAccept->activate();
					a_OTTriggerCancel->activate();
					a_OTTriggerMessage->value(config.action_msgs[i].c_message);
					sprintf(mybuf2, "%d", config.action_msgs[i].i_noRounds); a_OTTriggerRounds->value(mybuf2);
					a_OTTriggerEveryRound->value(config.action_msgs[i].i_display);
					a_OTTriggerEvery->value(config.action_msgs[i].c_every);
				  break;

		case 1:		// accept
					a_OTTriggerList->activate(); a_OTTriggerList->check_none();
					a_OTTriggerRounds->deactivate();
					a_OTTriggerMessage->deactivate();
					a_OTTriggerEveryRound->deactivate();
					a_OTTriggerEvery->deactivate();
					a_OTTriggerAccept->deactivate();
					a_OTTriggerCancel->deactivate();
					strncpy(config.action_msgs[i].c_message, a_OTTriggerMessage->value(), 59); config.action_msgs[i].c_message[59] = '\0';
					config.action_msgs[i].i_noRounds = atoi(a_OTTriggerRounds->value());
					config.action_msgs[i].i_display = a_OTTriggerEveryRound->value();
					strncpy(config.action_msgs[i].c_every, a_OTTriggerEvery->value(), 9); config.action_msgs[i].c_every[9] = '\0';
					a_OTTriggerList->clear();
					for (i=0; i<MAX_TRIGGERS; i++) {
						a_OTTriggerList->add(config.action_msgs[i].c_message);
					}
				  break;
		case 2:		// cancel
					a_OTTriggerList->activate();
					a_OTTriggerRounds->deactivate();
					a_OTTriggerMessage->deactivate();
					a_OTTriggerEveryRound->deactivate();
					a_OTTriggerEvery->deactivate();
					a_OTTriggerAccept->deactivate();
					a_OTTriggerCancel->deactivate();
				  break;

		case 9:		// refresh the list
					a_OTTriggerList->clear();
					for (i=0; i<MAX_TRIGGERS; i++) {
						a_OTTriggerList->add(config.action_msgs[i].c_message);
					}
				  break;
	}
}

/********************************************************************************/
void doAdjustmentAction(int action) {
	int i = a_OTAdjustList->value() - 1, j, k=0, z=0;

if (config.flags.f_debug != 0) { printf("dAA: %d:%d:%d\n", action, i, a_OTAdjustSystemList->value()); }
	switch (action) {
		case -1: 	// system adjustment selected
				  i = -1;
				  action = 0;

		case 0:		// edit/add
				   prev_part_selected = -1;
					a_OTAdjustList->deactivate();
					a_OTAdjustSystemList->deactivate(); 
					a_OTAdjustEffects->activate(); a_OTAdjustEffects->check_none();
					a_OTAdjustAccept->activate();
					a_OTAdjustCancel->activate();
					a_OTAdjustRounds->activate();
					a_OTAdjustMessage->activate();
					a_OTAdjustValue->activate();
					a_OTAdjustType->activate(); a_OTAdjustType->check_none();
					a_OTAdjustPartSelect->activate(); a_OTAdjustPartSelect->check_none();
					a_OTAdjustValue->activate(); a_OTAdjustValue->value("");
					a_OTAdjustType->activate(); a_OTAdjustType->check_none();
					a_OTAdjustEffectsType->activate(); a_OTAdjustEffectsType->check_none();
					a_OTAdjustEffects->clear();
					a_OTAdjustIDEnds->clear();
					a_OTAdjustEffects->add("All Players");
					a_OTAdjustEffects->add("All Monsters");
					k=0;
					config.i_idList[ID_ADJ1][k++] = -1;
					config.i_idList[ID_ADJ1][k++] = -2;
					for (j=0; j<MAX_MEMBERS; j++) {
						if (players[j].c_name[0] != '\0' && players[j].i_hp[HP_MAX] > 0 && players[j].flags.f_disabled == 0) {
							a_OTAdjustEffects->add(players[j].c_name);
							a_OTAdjustIDEnds->add(players[j].c_name);
if (config.flags.f_debug != 0) { printf("dAA: 0 A %d:%d\n", k, j); }
							config.i_idList[ID_ADJ1][k++] = j;
						}
					}
					config.i_idList[ID_ADJ1][k] = -255;
				  		// is it a predefined one
					if (i == -1 && a_OTAdjustSystemList->value() > 0) {
if (config.flags.f_debug != 0) { printf("dAA: 0 B\n"); }
						a_OTAdjustSystemSave->activate();
				   	i = a_OTAdjustSystemList->value() - 1;
						a_OTAdjustMessage->value(config.system_mods[i].c_message);
						sprintf(mybuf2, "%d", config.system_mods[i].i_noRounds); a_OTAdjustRounds->value(mybuf2);
						for (z=0; z<5; z++) {
							for (int x=0; x<3; x++) {
								adjustValues[z][x] = config.system_mods[i].i_data[z][x];
							}
if (config.flags.f_debug != 0) { printf("dAA: 0 C %d = %d:%d:%d\n", i, adjustValues[z][0], adjustValues[z][1], adjustValues[z][2]); }
						}
					} else {
						if (config.action_mods[i].i_whom[0] != -255) {
							for (j=0; j<MAX_MEMBERS && config.action_mods[i].i_whom[j] != -255; j++) {
								for (k=0; k<MAX_MEMBERS && config.i_idList[ID_ADJ1][k] != -255; k++) {
									if (config.action_mods[i].i_whom[j] == config.i_idList[ID_ADJ1][k]) {
if (config.flags.f_debug != 0) { printf("dAA: 2 D %d:%d:%d:%d\n", j,k,config.action_mods[i].i_whom[j],config.i_idList[ID_ADJ1][k]); }
										a_OTAdjustEffects->checked(k+1, 1);
									}
								}
							}
						}
						a_OTAdjustMessage->value(config.action_mods[i].c_message);
						sprintf(mybuf2, "%d", config.action_mods[i].i_noRounds); a_OTAdjustRounds->value(mybuf2);
						for (z=0; z<5; z++) {
							for (int x=0; x<3; x++) {
								adjustValues[z][x] = config.action_mods[i].i_data[z][x];
							}
if (config.flags.f_debug != 0) { printf("dAA: 0 c %d = %d:%d:%d\n", i, adjustValues[z][0], adjustValues[z][1], adjustValues[z][2]); }
						}
					}
					a_OTAdjustIDEnds->check_none();
					a_OTAdjustIDEnds->activate();
					if (config.action_mods[i].i_idFollows > -1) {
						for (j=0; j<MAX_MEMBERS && config.i_idList[ID_ADJ1][j] != -255; j++) {
							if (config.i_idList[ID_ADJ1][j] == config.action_mods[i].i_idFollows) {
								a_OTAdjustIDEnds->checked(j-1, 1);
if (config.flags.f_debug != 0) { printf("dAA: 0 F %d:%d:%d\n", j,config.action_mods[i].i_idFollows,config.i_idList[ID_ADJ1][j]); }
								break;
							}
						}
					}
				  break;
				  
		case 1:		// accept
				   if (prev_part_selected != -1) {		// save the current part details
						if (a_OTAdjustEffectsType->checked(1) == 1) { i = 0; }
						if (a_OTAdjustEffectsType->checked(2) == 1) { i = 1; }
						if (a_OTAdjustEffectsType->checked(3) == 1) { i = 2; }
						if (a_OTAdjustEffectsType->checked(4) == 1) { i = 3; }
						adjustValues[prev_part_selected][0] = atoi(a_OTAdjustValue->value());
						if (a_OTAdjustType->nchecked() > 0) {
							adjustValues[prev_part_selected][1] = whichChecked(a_OTAdjustType);
						} else {
							adjustValues[prev_part_selected][1] = -1;
						}
						if (a_OTAdjustEffectsType->nchecked() > 0) {
							adjustValues[prev_part_selected][2] = whichChecked(a_OTAdjustEffectsType);
						} else {
							adjustValues[prev_part_selected][2] = -1;
						}
if (config.flags.f_debug != 0) { printf("dAA: 1 A %d = %d:%d:%d\n", prev_part_selected, adjustValues[i][0], adjustValues[i][1], adjustValues[i][2]); }
					}
						// if we're adding a predefined one, find the first empty slot
					i = whichChecked(a_OTAdjustList) - 1;
if (config.flags.f_debug != 0) { printf("dAA: 1 a %d\n", i); }

					a_OTAdjustList->activate(); a_OTAdjustList->check_none();
					a_OTAdjustSystemList->activate(); a_OTAdjustSystemList->check_none();
					a_OTAdjustEffects->deactivate();
					a_OTAdjustAccept->deactivate();
					a_OTAdjustCancel->deactivate();
					a_OTAdjustRounds->deactivate();
					a_OTAdjustMessage->deactivate();
					a_OTAdjustValue->deactivate();
					a_OTAdjustType->deactivate();
					a_OTAdjustSystemSave->deactivate();
					a_OTAdjustPartSelect->deactivate(); a_OTAdjustPartSelect->check_none();
					a_OTAdjustEffectsType->deactivate(); a_OTAdjustEffectsType->check_none();
					if (i == -1) {
						for (i=0; i<MAX_MODIFIERS; i++) {
							if (strncasecmp(config.action_mods[i].c_message, "-- empty slot", 13) == 0) { break; }
						}
					}
						// save whom it effects or follows
					config.action_mods[i].i_idFollows = -255;
					k=0; config.action_mods[i].i_whom[0] = -255;
					for (j=a_OTAdjustEffects->nitems(); j > 0; j--) {
						if (a_OTAdjustEffects->checked(j) == 1) {
if (config.flags.f_debug != 0) { printf("dAA: 1 B %d:%d:%d\n", j,k,config.i_idList[ID_ADJ1][j-1]); }
							config.action_mods[i].i_whom[k++] = config.i_idList[ID_ADJ1][j-1];
						}
						if (a_OTAdjustIDEnds->checked(j) == 1) {
if (config.flags.f_debug != 0) { printf("dAA: 1 C %d:%d\n", j,config.i_idList[ID_ADJ1][j+1]); }
							config.action_mods[i].i_idFollows = config.i_idList[ID_ADJ1][j+1];
						}
					}
					config.action_mods[i].i_whom[k] = -255;
					if (config.action_mods[i].i_whom[0] == -255) {
						doSystemAlert("You've not selected whom it will effect !!!!");
					}
					strncpy(config.action_mods[i].c_message, a_OTAdjustMessage->value(), 39); config.action_mods[i].c_message[39] = '\0';
					config.action_mods[i].i_noRounds = atoi(a_OTAdjustRounds->value());

					for (z=0; z<5; z++) {
						for (int x=0; x<3; x++) {
							config.action_mods[i].i_data[z][x] = adjustValues[z][x];
						}
if (config.flags.f_debug != 0) { printf("dAA: 1 D %d = %d:%d:%d\n", i, adjustValues[z][0], adjustValues[z][1], adjustValues[z][2]); }
					}
					a_OTAdjustList->clear();
					for (i=0; i<MAX_MODIFIERS; i++) {
						a_OTAdjustList->add(config.action_mods[i].c_message);
					}
if (config.flags.f_debug != 0) { printf("dAA: 1 E %d:%d:%d:%d\n", config.action_mods[i].i_data[0][0], config.action_mods[i].i_data[1][0], config.action_mods[i].i_data[2][0], config.action_mods[i].i_data[3][0]); }
					a_OTAdjustIDEnds->check_none();
					a_OTAdjustIDEnds->deactivate();
				  break;

		case 2:		// cancel
					a_OTAdjustList->activate(); a_OTAdjustList->check_none();
					a_OTAdjustSystemList->activate(); a_OTAdjustSystemList->check_none();
					a_OTAdjustEffects->deactivate();
					a_OTAdjustAccept->deactivate();
					a_OTAdjustCancel->deactivate();
					a_OTAdjustRounds->deactivate();
					a_OTAdjustMessage->deactivate();
					a_OTAdjustValue->deactivate();
					a_OTAdjustType->deactivate();
					a_OTAdjustSystemSave->deactivate();
					a_OTAdjustIDEnds->check_none();
					a_OTAdjustIDEnds->deactivate();
					a_OTAdjustPartSelect->deactivate(); a_OTAdjustPartSelect->check_none();
					a_OTAdjustValue->deactivate(); a_OTAdjustValue->value("");
					a_OTAdjustType->deactivate(); a_OTAdjustType->check_none();
					a_OTAdjustEffectsType->deactivate(); a_OTAdjustEffectsType->check_none();
				  break;

		case 3:		// adjustment part selected
				  if (prev_part_selected != -1) {
						adjustValues[prev_part_selected][0] = atoi(a_OTAdjustValue->value());
						if (a_OTAdjustType->nchecked() > 0) {
							adjustValues[prev_part_selected][1] = whichChecked(a_OTAdjustType);
						} else {
							adjustValues[prev_part_selected][1] = -1;
						}
						if (a_OTAdjustEffectsType->nchecked() > 0) {
							adjustValues[prev_part_selected][2] = whichChecked(a_OTAdjustEffectsType);
						} else {
							adjustValues[prev_part_selected][2] = -1;
						}
				  }
if (config.flags.f_debug != 0) { printf("dAA: 3 %d:%d", prev_part_selected, whichChecked(a_OTAdjustPartSelect)-1); }
				  i = prev_part_selected = a_OTAdjustPartSelect->value() - 1;
if (config.flags.f_debug != 0) { printf(" %d:%d:%d\n", adjustValues[i][0], adjustValues[i][1], adjustValues[i][2]); }
				  a_OTAdjustPartSelect->check_none(); a_OTAdjustPartSelect->checked(i+1, 1);
				  sprintf(mybuf1, "%d", adjustValues[i][0]); a_OTAdjustValue->value(mybuf1);
				  a_OTAdjustType->check_none();
				  if (adjustValues[i][1] != -1) { a_OTAdjustType->checked(adjustValues[i][1], 1); }
				  a_OTAdjustEffectsType->check_none();
				  if (adjustValues[i][2] != -1) { a_OTAdjustEffectsType->checked(adjustValues[i][2], 1); }
				  break;

		case 7:		// save system
				   if (prev_part_selected != -1) {		// save the current part details
						if (a_OTAdjustEffectsType->checked(1) == 1) { i = 0; }
						if (a_OTAdjustEffectsType->checked(2) == 1) { i = 1; }
						if (a_OTAdjustEffectsType->checked(3) == 1) { i = 2; }
						if (a_OTAdjustEffectsType->checked(4) == 1) { i = 3; }
						adjustValues[prev_part_selected][0] = atoi(a_OTAdjustValue->value());
						if (a_OTAdjustType->nchecked() > 0) {
							adjustValues[prev_part_selected][1] = whichChecked(a_OTAdjustType);
						} else {
							adjustValues[prev_part_selected][1] = -1;
						}
						if (a_OTAdjustEffectsType->nchecked() > 0) {
							adjustValues[prev_part_selected][2] = whichChecked(a_OTAdjustEffectsType);
						} else {
							adjustValues[prev_part_selected][2] = -1;
						}
if (config.flags.f_debug != 0) { printf("dAA: 7 A %d = %d:%d:%d\n", prev_part_selected, adjustValues[i][0], adjustValues[i][1], adjustValues[i][2]); }
					}
				  //
				   i = a_OTAdjustSystemList->value() - 1;
					strncpy(config.system_mods[i].c_message, a_OTAdjustMessage->value(), 39); config.system_mods[i].c_message[39] = '\0';
					config.system_mods[i].i_noRounds = atoi(a_OTAdjustRounds->value());
					for (z=0; z<5; z++) {
						for (int x=0; x<3; x++) {
							config.system_mods[i].i_data[z][x] = adjustValues[z][x];
						}
if (config.flags.f_debug != 0) { printf("dAA: 7 D %d = %d:%d:%d\n", i, adjustValues[z][0], adjustValues[z][1], adjustValues[z][2]); }
					}
					a_OTAdjustSystemList->clear();
					for (i=0; i<MAX_MODIFIERS; i++) {
						a_OTAdjustSystemList->add(config.system_mods[i].c_message);
					}
					doAdjustmentAction(2);
				  break;

		case 8:		// selected an INIT to follow
				  i = a_OTAdjustIDEnds->nchecked();
if (config.flags.f_debug != 0) { printf("dAA: 2 %d:%d\n", i, a_OTAdjustIDEnds->value()); }
				  if (a_OTAdjustIDEnds->value() > 0 && i > 0) {
						if (i == 1) {
							doSystemAlert("Please add ONE to the # Rounds to ensure that it stops on their Initiative");
						}
						i = a_OTAdjustIDEnds->value();
						a_OTAdjustIDEnds->check_none();
						a_OTAdjustIDEnds->checked(i, 1);
				  }
				  break;

		case 9:		// refresh the list
					a_OTAdjustList->clear();
					for (i=0; i<MAX_MODIFIERS; i++) {
						a_OTAdjustList->add(config.action_mods[i].c_message);
					}
					a_OTAdjustSystemList->clear();
					for (i=0; i<MAX_MODIFIERS; i++) {
						a_OTAdjustSystemList->add(config.system_mods[i].c_message);
					}
				  break;
	}
}

/********************************************************************************/
int getAdjustment(int plyr, int type) { 
	int i, j=0, k, adj[MAX_ADJEFFECTTYPES+1];

	for (i=0; i<MAX_ADJEFFECTTYPES+1; i++) { adj[i] = -255; }

	for (i=0; i<MAX_MODIFIERS; i++) {
		if (config.action_mods[i].i_noRounds > 0) {
if (config.flags.f_debug != 0) { printf("gA: 0 %s %d %d %d\n", config.action_mods[i].c_message, type, plyr, i); }
			for (k=0; k<MAX_MEMBERS && config.action_mods[i].i_whom[k] != -255; k++) {
if (config.flags.f_debug != 0) { printf("gA: 1 %d %d\n", config.action_mods[i].i_whom[k], k); }
			}
			for (k=0; k<MAX_MEMBERS && config.action_mods[i].i_whom[k] != -255; k++) {
				if ((plyr < MAX_PLAYERS && config.action_mods[i].i_whom[k] == -1) ||		// does it effect this player
					 (plyr >= MAX_PLAYERS && config.action_mods[i].i_whom[k] == -2) ||
					 (plyr == config.action_mods[i].i_whom[k])) {
						  for (int j=0; j<5; j++) {
						  		if (adjustments[config.action_mods[i].i_data[j][1]-1].i_type == type) { 
									if (config.action_mods[i].i_data[j][0] > adj[config.action_mods[i].i_data[j][2]]) {
										adj[config.action_mods[i].i_data[j][2]] = config.action_mods[i].i_data[j][0];
if (config.flags.f_debug != 0) { printf("gA: 2 %s %d %d:%d:%d\n", config.action_mods[i].c_message, type, config.action_mods[i].i_data[j][0], config.action_mods[i].i_data[0][1], config.action_mods[i].i_data[j][2]); }
									}
								}
						  }
				}
			}
		}
	}
	j = 0;
	for (i=0; i<MAX_ADJEFFECTTYPES+1; i++) { 
		if (adj[i] != -255) { 
			j += adj[i]; 
		} 
	}
if (config.flags.f_debug != 0) { printf("gA: %d:%d:%d\n", plyr, type, j); }
	return(j);
}

/********************************************************************************/
int doEndOfRoundMessages(int action) {
	int AYSreturn;

	if (a_EORWindow == NULL) {
//  		{ Fl_Double_Window* o = a_EORWindow = new Fl_Double_Window(380, 395, "ADMP - End Of Round Messages");
  		{ a_EORWindow = new Fl_Double_Window(380, 395, "ADMP - End Of Round Messages");
		    { Fl_Browser* o = a_EORMessageList = new Fl_Browser(15, 10, 350, 335);
		      o->labelsize(ADMP_LABEL_SIZE);
		      o->textsize(ADMP_TEXT_SIZE);
		    }
		    { Fl_Button* o = a_EORMessageYes = new Fl_Button(60, 355, 105, 25, "Yes - Continue");
		      o->labelsize(ADMP_LABEL_SIZE);
		    }
		    { Fl_Button* o = a_EORMessageNo = new Fl_Button(190, 355, 105, 25, "No - Do Actions");
		      o->labelsize(ADMP_LABEL_SIZE);
		    }
		    a_EORWindow->set_modal();
		    a_EORWindow->end();
		    a_EORWindow->resizable(a_EORWindow);
 		 }
		 if (action == 0) { a_EORWindow->hide(); return 0; }
	}
	a_EORWindow->show();
	for (;;) {
		Fl_Widget *o = Fl::readqueue();
		if (!o) {
			Fl::wait();
		} else if ( o == a_EORMessageYes) {
			AYSreturn = 1;
			break;
		} else if ( o == a_EORMessageNo) {
			AYSreturn = -1;
			break;
		}
	}
	a_EORWindow->hide();
	return AYSreturn;
}

/********************************************************************************/
void doCalcXP() {
	int i, j, xp=0, np=0;

	a_OXPCalcList->clear();
	i = atoi(a_OXPPartyLevel->value());
	for (j=0; j<MAX_PLAYERS; j++) {
		if (players[j].c_name[0] != '\0' && players[j].i_hp[HP_MAX] > 0 && players[j].flags.f_disabled == 0) {
			np++;
		}
	}
	if (np == 0) { np = 1; }
	for (j=MAX_PLAYERS; j<MAX_MEMBERS; j++) {
		if (players[j].c_name[0] != '\0' && players[j].i_hp[HP_MAX] > 0 && players[j].flags.f_npc == 0) {
			if (a_OXPCalcIncAll->value() == 1 || players[j].flags.f_disabled == 0) {
				sprintf(mybuf2, "%s (CR: %.1f) = %d", players[j].c_name, players[j].f_challengeRating, xp_cr[i][(int )players[j].f_challengeRating]);
				a_OXPCalcList->add(mybuf2);
				xp += xp_cr[i][(int )players[j].f_challengeRating];
			}
		}
	}
	a_OXPCalcList->add("");
	sprintf(mybuf2, "   XP Total: %d (Per Player: %d)", xp, (xp / np)); a_OXPCalcList->add(mybuf2);

	sprintf(mybuf2, "%d", (xp / np)); a_OXPValue->value(mybuf2);
}

/********************************************************************************/
void doAddPlayerXP() {

	sprintf(mybuf2, "XP: adding %d to all players", atoi(a_OXPValue->value()));
	writeLog(mybuf2);
	for (int i=0; i<MAX_PLAYERS; i++) {
		if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
			players[i].i_xp[MAX] += atoi(a_OXPValue->value());
		}
	}
	a_OXPValue->value("0");
}

/********************************************************************************/
void writeLog(char *text) {
	char fn[512];
	
	if (config.flags.f_logfile == 0) { return; }

	if (logfile == NULL) {
		sprintf(fn, "%slogfile.txt", systemDir);
		if ((logfile = fopen(fn, "a")) == NULL) {
			doSystemAlert("Unable to open the logfile.txt .... !!!");
			return;
		}
	}
	if (fprintf(logfile, "%s\n", text) < 0) {
		doSystemAlert("Unable to log the logfile.txt .... !!!");
	}
	fflush(logfile);
}

/********************************************************************************/
void mytolower(char *string) {
	int i=0;

	while (string[i] != '\0') {
		string[i] = tolower(string[i]);
		i++;
	}
}

/********************************************************************************/
void doWeaponSearchAction(int action) {
	int i = a_WSearchList->value() - 1;

	i = config.i_idList[ID_WSCH][i];

if (config.flags.f_debug != 0) { printf("dWSA: %d:%d:%d\n", action, i, i_WeaponUsed); }
	switch (action) {
		case 0:
				  a_WSearchDetails->clear();
				  a_WSearchDetails->add(stdweapons[i].c_description);
				  break;
		case 1: 
				  if (i_prevWeaponEditted > -1) {
					  aEdit_WeaponAvailable->value(1);
					  aEdit_WeaponName->value(stdweapons[i].c_description);
					  aEdit_WeaponAttkBonus1->value("0");
					  aEdit_WeaponAttkBonus2->value("0");
					  aEdit_WeaponAttkBonus3->value("0");
					  aEdit_WeaponAttkBonus4->value("0");
					  aEdit_WeaponDamage->value(stdweapons[i].c_damage);
					  aEdit_WeaponCritical->value(stdweapons[i].c_critical);
					  aEdit_WeaponMagicBonus->value("0");
					  sprintf(mybuf2, "%d", stdweapons[i].i_range); aEdit_WeaponRange->value(mybuf2);
					  sprintf(mybuf2, "%d", stdweapons[i].i_reach); aEdit_WeaponReach->value(mybuf2);
					  aEdit_WeaponNoAttacks->value("1");
					  aEdit_WeaponSpecialMessage->value("");
					  aEdit_WeaponType->value(stdweapons[i].c_attackType);
				  } else {
					  if (i_prevWeaponEditted == -10) {
						  aQkAdd_WeaponName1->value(stdweapons[i].c_description);
						  aQkAdd_WeaponAttkBonus11->value("0");
						  aQkAdd_WeaponAttkBonus12->value("0");
						  aQkAdd_WeaponAttkBonus13->value("0");
						  aQkAdd_WeaponAttkBonus14->value("0");
						  aQkAdd_WeaponDamage1->value(stdweapons[i].c_damage);
						  aQkAdd_WeaponCritical1->value(stdweapons[i].c_critical);
						  aQkAdd_WeaponMagicBonus1->value("0");
						  sprintf(mybuf2, "%d", stdweapons[i].i_range); aQkAdd_WeaponRange1->value(mybuf2);
						  sprintf(mybuf2, "%d", stdweapons[i].i_reach); aQkAdd_WeaponReach1->value(mybuf2);
						  aQkAdd_WeaponNoAttacks1->value("1");
					  } else {
						  aQkAdd_WeaponName2->value(stdweapons[i].c_description);
						  aQkAdd_WeaponAttkBonus21->value("0");
						  aQkAdd_WeaponAttkBonus22->value("0");
						  aQkAdd_WeaponAttkBonus23->value("0");
						  aQkAdd_WeaponAttkBonus24->value("0");
						  aQkAdd_WeaponDamage2->value(stdweapons[i].c_damage);
						  aQkAdd_WeaponCritical2->value(stdweapons[i].c_critical);
						  aQkAdd_WeaponMagicBonus2->value("0");
						  sprintf(mybuf2, "%d", stdweapons[i].i_range); aQkAdd_WeaponRange2->value(mybuf2);
						  sprintf(mybuf2, "%d", stdweapons[i].i_reach); aQkAdd_WeaponReach2->value(mybuf2);
						  aQkAdd_WeaponNoAttacks2->value("1");
					  }
				  }
				  a_WeaponWindow->hide();
				  break;
		case 2:
				  a_WeaponWindow->hide();
				  break;
	}
}

/********************************************************************************/
int whichChecked(Fl_Check_Browser *b) {
	for (int i=b->nitems(); i>0; i--) {
		if (b->checked(i) == 1) {
if (config.flags.f_debug != 0) { printf("wC: %d\n", i); }
			return i;
		}
	}
	return 0;
}

/********************************************************************************/
void doAutoDamageAction(int act) {
	int i,j,k;

if (config.flags.f_debug != 0) { printf("dADA: %d\n", act); }
	switch (act) {
		case 0:
				  a_OTADList->clear();
				  for (i=0; i<MAX_AUTODAMAGE; i++) {
				  		a_OTADList->add(config.auto_damage[i].c_description);
				  }
					a_OTADList->activate();
					a_OTADAccept->deactivate();
					a_OTADCancel->deactivate();
					a_OTADInitiateList->deactivate();
					a_OTADReceiveList->deactivate();
					a_OTADDescription->deactivate();
					a_OTADDamage->deactivate();
					a_OTADRounds->deactivate();
					a_OTADRoundsDone->deactivate();
					a_OTADType->deactivate();
					a_OTADFlags->deactivate();
				  break;
		case 1:
				  i = a_OTADList->value() - 1;
				  if (i < 0)  {
						return;
				  }
if (config.flags.f_debug != 0) { printf("dADA: 1 %d:%d\n", i, config.auto_damage[i].i_initiatorID); }
				  k=0;
				  a_OTADInitiateList->clear();
					a_OTADReceiveList->clear();
				  for (j=0; j<MAX_MEMBERS; j++) {
						if (players[j].c_name[0] != '\0' && players[j].i_hp[HP_MAX] > 0) {
				  			config.i_idList[ID_ADAM][k++] = j;
							if (config.auto_damage[i].i_initiatorID == j) {
								a_OTADInitiateList->add(players[j].c_name, 1);
							} else {
								a_OTADInitiateList->add(players[j].c_name);
							}
							a_OTADReceiveList->add(players[j].c_name);
						}
				  }
				  	config.i_idList[ID_ADAM][k] = -1;
					for (k=0; config.auto_damage[i].i_whom[k] != -1 && k < MAX_MEMBERS; k++) {
						for (j=0; j < MAX_MEMBERS && config.i_idList[ID_ADAM][j] != -1; j++) {
							if (config.auto_damage[i].i_whom[k] == config.i_idList[ID_ADAM][j]) {
								a_OTADReceiveList->checked(j+1, 1);
							}
						}
					}
					a_OTADList->deactivate();
					a_OTADAccept->activate();
					a_OTADCancel->activate();
					a_OTADInitiateList->activate();
					a_OTADReceiveList->activate();
					a_OTADDescription->activate();
					a_OTADDamage->activate();
					a_OTADRounds->activate();
					a_OTADRoundsDone->activate();
					a_OTADType->deactivate();
					a_OTADFlags->activate();
					a_OTADFlags->check_none();

				  a_OTADDescription->value(config.auto_damage[i].c_description);
				  a_OTADDamage->value(config.auto_damage[i].c_damage);
				  sprintf(mybuf2, "%d", config.auto_damage[i].i_noRounds); a_OTADRounds->value(mybuf2);
				  sprintf(mybuf2, "%d", config.auto_damage[i].i_doneRounds); a_OTADRoundsDone->value(mybuf2);
				  a_OTADType->value(config.auto_damage[i].i_type);
				  a_OTADFlags->checked(1, config.auto_damage[i].flags.f_flag1);
				  a_OTADFlags->checked(2, config.auto_damage[i].flags.f_flag2);
				  a_OTADFlags->checked(3, config.auto_damage[i].flags.f_flag3);
				  		// adding a new one, so mark it as being done else it'll double add
				  if (config.auto_damage[i].i_noRounds == 0 && strncasecmp(config.auto_damage[i].c_description, "-- empty", 8) == 0) {
				  		a_OTADFlags->checked(2, 1);
				  }
				  break;

		case 2:	// accept
				  i = a_OTADList->value() - 1;
				  strncpy(config.auto_damage[i].c_description, a_OTADDescription->value(), 32); config.auto_damage[i].c_description[31] = '\0';
				  strncpy(config.auto_damage[i].c_damage, a_OTADDamage->value(), 32); config.auto_damage[i].c_damage[31] = '\0';
				  for (j=a_OTADInitiateList->nitems(); j>0; j--) {
						if (a_OTADInitiateList->checked(j) == 1) {
				  			config.auto_damage[i].i_initiatorID = config.i_idList[ID_ADAM][j-1];
						}
				  }
				  config.auto_damage[i].i_noRounds = atoi(a_OTADRounds->value());
				  config.auto_damage[i].i_doneRounds = atoi(a_OTADRoundsDone->value());
				  config.auto_damage[i].i_type = a_OTADType->value();
				  config.auto_damage[i].flags.f_flag1 = a_OTADFlags->checked(1);
				  config.auto_damage[i].flags.f_flag2 = a_OTADFlags->checked(2);
				  config.auto_damage[i].flags.f_flag3 = a_OTADFlags->checked(3);
				  k=0;
				  for (j=a_OTADReceiveList->nitems(); j>0; j--) {
						if (a_OTADReceiveList->checked(j) == 1) {
				  			config.auto_damage[i].i_whom[k++] = config.i_idList[ID_ADAM][j-1];
						}
				  }
				  config.auto_damage[i].i_whom[k] = -1;
printf("aADA: 3: %d:%d\n", config.auto_damage[i].i_initiatorID, a_OTADInitiateList->value());
				  doAutoDamageAction(0);
				  break;

		case 3:	// cancel
				  doAutoDamageAction(0);
				  break;

		case 4:	// flags/options selected
				  mybuf2[0] = '\0';
				  if (a_OTADFlags->checked(1) == 1) { strcat(mybuf2, "Dont forget to add ONE to the number of Rounds"); }
				  if (mybuf2[0] != '\0') {
						doSystemAlert(mybuf2);
				  }
				  break;

		case 9:	// refresh the list
				  a_OTADList->clear();
				  for (i=0; i<MAX_AUTODAMAGE; i++) {
				  		a_OTADList->add(config.auto_damage[i].c_description);
				  }
				  break;
	}
}

/********************************************************************************/
int calcAutoDamage(int adID, int recvID) {
	int i, j, k, l;
	char *dam=NULL;

		// find all the Auto-damages not done & do all the damage
//	for (i=0; i<MAX_AUTODAMAGE; i++) {
		i = adID;	// only this autoDamage config
if (config.flags.f_debug != 0) { printf("cAD: 0 %d:%d:%d:%d\n", i, recvID, config.auto_damage[i].i_noRounds, config.auto_damage[i].flags.f_flag2); }
		if (config.auto_damage[i].i_noRounds > 0 && config.auto_damage[i].flags.f_flag2 == 0) {
//			config.auto_damage[i].flags.f_flag2 = 1;		// mark as being done
			strcpy(mybuf2, config.auto_damage[i].c_damage);
			dam = strtok(mybuf2, " ");
			if (dam == NULL) {	// only ONE damage 'value' specified
				dam = &mybuf2[0];
			} else {
				k = 1;
				while (k < config.auto_damage[i].i_doneRounds && dam != NULL) {
					dam = strtok(NULL, " ");
					k++;
				}
			}
			if (dam == NULL) {	// only ONE damage 'value' specified
				dam = &mybuf2[0];
			}
if (config.flags.f_debug != 0) { printf("cAD: 0 %s\n", dam); }
			k = parseDice(dam);
			for (j=0; j<MAX_MEMBERS && config.auto_damage[i].i_whom[j] != -1; j++) {
				if (config.auto_damage[i].i_whom[j] == recvID || recvID == -1) {
					if (config.auto_damage[i].flags.f_flag3 != 0) {		// reroll flag set ?
						k = parseDice(dam);
					}
					l = config.auto_damage[i].i_whom[j];
					players[l].i_hp[HP_CUR] -= k;
					if (players[l].i_hp[HP_CUR] > players[l].i_hp[HP_MAX]) {
						players[l].i_hp[HP_CUR] = players[l].i_hp[HP_MAX];
					}
					sprintf(mybuf2, "AutoDamage: %s -> %s (HP:%d) Cur:%d", players[config.auto_damage[i].i_initiatorID].c_name, players[l].c_name, k, players[l].i_hp[HP_CUR]);
					writeLog(mybuf2);
if (config.flags.f_debug != 0) { printf("cAD: 1 %d:%d:%d\n", i, j, config.auto_damage[i].i_whom[j]); }
				}
			}
		}
//	}
	return 0;
}

/********************************************************************************/
void selectEditWeapon(int w) {
	int i = config.i_editID;
	char buf[10];

	if (i_prevWeaponEditted != -1) {		// save the previous weapon details
		aEdit_WeaponAvailable->value(players[i].w_weapons[i_prevWeaponEditted].c_available);
		strncpy(players[i].w_weapons[i_prevWeaponEditted].c_description, aEdit_WeaponName->value(), 29);
		players[i].w_weapons[i_prevWeaponEditted].i_attackBonus[0] = atoi(aEdit_WeaponAttkBonus1->value());
		players[i].w_weapons[i_prevWeaponEditted].i_attackBonus[1] = atoi(aEdit_WeaponAttkBonus2->value());
		players[i].w_weapons[i_prevWeaponEditted].i_attackBonus[2] = atoi(aEdit_WeaponAttkBonus3->value());
		players[i].w_weapons[i_prevWeaponEditted].i_attackBonus[3] = atoi(aEdit_WeaponAttkBonus4->value());
		strncpy(players[i].w_weapons[i_prevWeaponEditted].c_damage, aEdit_WeaponDamage->value(), 19);
		strncpy(players[i].w_weapons[i_prevWeaponEditted].c_critical, aEdit_WeaponCritical->value(), 19);
		players[i].w_weapons[i_prevWeaponEditted].i_magicalBonus = atoi(aEdit_WeaponMagicBonus->value());
		players[i].w_weapons[i_prevWeaponEditted].i_range = atoi(aEdit_WeaponRange->value());
		players[i].w_weapons[i_prevWeaponEditted].i_reach = atoi(aEdit_WeaponReach->value());
		players[i].w_weapons[i_prevWeaponEditted].i_noAttacks = atoi(aEdit_WeaponNoAttacks->value());
		strncpy(players[i].w_weapons[i_prevWeaponEditted].c_message, aEdit_WeaponSpecialMessage->value(), 29);
		players[i].w_weapons[i_prevWeaponEditted].c_description[29] = '\0';
		players[i].w_weapons[i_prevWeaponEditted].c_damage[19] = '\0';
		players[i].w_weapons[i_prevWeaponEditted].c_critical[19] = '\0';
		players[i].w_weapons[i_prevWeaponEditted].c_message[29] = '\0';
		if (players[i].w_weapons[i_prevWeaponEditted].c_description[0] != '\0') {
			aEdit_WeaponSelector->text(i_prevWeaponEditted+1, players[i].w_weapons[i_prevWeaponEditted].c_description);
		}
		players[i].w_weapons[i_prevWeaponEditted].c_attackType = aEdit_WeaponType->value();
	}
	aEdit_WeaponAvailable->activate(); aEdit_WeaponAvailable->value(players[i].w_weapons[w].c_available);
	aEdit_WeaponName->activate(); aEdit_WeaponName->value(players[i].w_weapons[w].c_description);
	aEdit_WeaponSearch->activate();
	aEdit_WeaponAttkBonus1->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_attackBonus[0]); aEdit_WeaponAttkBonus1->value(buf);
	aEdit_WeaponAttkBonus2->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_attackBonus[1]); aEdit_WeaponAttkBonus2->value(buf);
	aEdit_WeaponAttkBonus3->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_attackBonus[2]); aEdit_WeaponAttkBonus3->value(buf);
	aEdit_WeaponAttkBonus4->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_attackBonus[3]); aEdit_WeaponAttkBonus4->value(buf);
	aEdit_WeaponDamage->activate(); aEdit_WeaponDamage->value(players[i].w_weapons[w].c_damage);
	aEdit_WeaponCritical->activate(); aEdit_WeaponCritical->value(players[i].w_weapons[w].c_critical);
	aEdit_WeaponMagicBonus->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_magicalBonus); aEdit_WeaponMagicBonus->value(buf);
	aEdit_WeaponRange->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_range); aEdit_WeaponRange->value(buf);
	aEdit_WeaponReach->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_reach); aEdit_WeaponReach->value(buf);
	aEdit_WeaponNoAttacks->activate(); sprintf(buf, "%d", players[i].w_weapons[w].i_noAttacks); aEdit_WeaponNoAttacks->value(buf);
	aEdit_WeaponSpecialMessage->activate(); aEdit_WeaponSpecialMessage->value(players[i].w_weapons[w].c_message);
	aEdit_WeaponType->activate(); aEdit_WeaponType->value(players[i].w_weapons[w].c_attackType);
	i_prevWeaponEditted = w;
}
	
/********************************************************************************/
int getWeaponAttackBonus(int p, int w) {
	int i, j;

	i = players[p].w_weapons[w].i_doneAttacks; if (i > MAX_WEAPONATTACKS) { i = MAX_WEAPONATTACKS-1; }
	j = players[p].w_weapons[w].i_attackBonus[i];
if (config.flags.f_debug != 0) { printf("gWAB: 0: %d %d %d %d\n", p, w, i, j); }
	if (j > 0 || i == 0) { return(j); }
//	for (i=(MAX_WEAPONATTACKS-1); i >= 0; i--) {
//		j = players[p].w_weapons[w].i_attackBonus[i];
//		if (j > 0) { return(j); }
//	}
//		we assume that since there's not a AB for that attack that it's the first bonus
	return (players[p].w_weapons[w].i_attackBonus[0]);
}

/********************************************************************************/
void setOtherActionDescription(int d) {
	switch (d) {
		case 0: a_OOtherActionDescription->value("This will reset them back to full health");
				  break;
		case 1: a_OOtherActionDescription->value("This currently does nothing");
				  break;
		case 2: a_OOtherActionDescription->value("This will remove all the players currently defined");
				  break;
		case 3: a_OOtherActionDescription->value("This will reset them back to full health");
				  break;
		case 4: a_OOtherActionDescription->value("This currently does nothing");
				  break;
		case 5: a_OOtherActionDescription->value("This will remove all the monsters currently defined");
				  break;
		case 6: a_OOtherActionDescription->value("This will remove all the Triggers currently defined");
				  break;
		case 7: a_OOtherActionDescription->value("This will remove all the Adjustments currently defined");
				  break;
		case 8: a_OOtherActionDescription->value("This will reset the players/monsters to full health and\nclear any triggers/adjustments/auto-damages\ncurrently defined");
				  break;
		case 9: a_OOtherActionDescription->value("This will calculate the monster's Hit Points based\non their Hit Dice");
				  break;
		case 10: a_OOtherActionDescription->value("This will remove all the Auto-Damages currently defined");
				  break;
		case 11: a_OOtherActionDescription->value("This will heal all enabled players by LEVEL HPs and reset their\nspell slot counts");
				  break;
		case 12: a_OOtherActionDescription->value("");
				  break;
		case 13: a_OOtherActionDescription->value("");
				  break;
	}
	//a_OOtherActionDescription->value(a_OOActionList->text(d+1));
}

/********************************************************************************/
void setSystemFlag(int type) {

	switch (type) {
		case 0: aSys_flags->clear();
				  aSys_flags->add("Debug", config.flags.f_debug);
				  aSys_flags->add("Roll All Rolls", config.flags.f_rollAllRolls);
				  aSys_flags->add("Roll All INIT Rolls", config.flags.f_rollInitRolls);
				  aSys_flags->add("Write Log File", config.flags.f_logfile);
				  aSys_flags->add("Roll INIT's Every Round", config.flags.f_rollInitEveryRnd);
				  aSys_flags->add("Remove Uncon Monsters from Init Seq", config.flags.f_removeMonstersInit);
				  aSys_flags->add("One always Misses, 20 Always Hits", config.flags.f_oneMissTwentyHits);
				  break;

		case 1:
				  config.flags.f_debug = aSys_flags->checked(1);
				  config.flags.f_rollAllRolls = aSys_flags->checked(2);
				  config.flags.f_rollInitRolls = aSys_flags->checked(3);
				  config.flags.f_logfile = aSys_flags->checked(4);
				  config.flags.f_rollInitEveryRnd = aSys_flags->checked(5);
				  config.flags.f_removeMonstersInit = aSys_flags->checked(6);
				  config.flags.f_oneMissTwentyHits = aSys_flags->checked(7);
				  if (config.flags.f_rollInitEveryRnd != 0) {
						config.flags.f_rollInitRolls = -1;
						aSys_flags->checked(3, 1);
				  }
				  break;
	}
// printf("sSF: %d\n", config.flags.f_debug);
}

// List of possible vowels; duplicates increase the likelihood of selection:
char *vowels[] = {
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"a", "e", "i", "o", "u",
"ae", "ai", "ao", "au", "aa",
"ea", "eo", "eu", "ee",
"ia", "io", "iu", "ii",
"oa", "oe", "oi", "ou", "oo",
"eau", "'", "y"};

typedef struct consons {
	char *conson;
	int flag;
} consons;

consons consonants[] = {
{"b", 3},  {"c", 3},  {"d", 3},  {"f", 3},  {"g", 3},  {"h", 3}, 
{"j", 3},  {"k", 3},  {"l", 3},  {"m", 3},  {"n", 3},  {"p", 3}, 
{"qu", 2},  {"r", 3}, {"s", 3},  {"t", 3},  {"v", 3},  {"w", 3},
{"x", 3},  {"y", 3},  {"z", 3}, 
// Blends, sorted by second character:
{"sc", 3},		// 22
{"ch", 3},  {"gh", 3},  {"ph", 3}, {"sh", 3},  {"th", 3}, {"wh", 2},
{"ck", 1},  {"nk", 1},  {"rk", 1}, {"sk", 3},  {"wk", 0},
{"cl", 2},  {"fl", 2},  {"gl", 2}, {"kl", 2},  {"ll", 2}, {"pl", 2}, {"sl", 2},
{"br", 2},  {"cr", 2},  {"dr", 2},  {"fr", 2},  {"gr", 2},  {"kr", 2}, 
{"pr", 2},  {"sr", 2},  {"tr", 2}, {"ss", 1}, {"st", 3},  {"str", 2},	// 53
// Repeat some entries to make them more common.
{"b", 3},  {"c", 3},  {"d", 3},  {"f", 3},  {"g", 3},  {"h", 3}, 
{"j", 3},  {"k", 3},  {"l", 3},  {"m", 3},  {"n", 3},  {"p", 3}, 
{"r", 3}, {"s", 3},  {"t", 3},  {"v", 3},  {"w", 3}, 
{"b", 3},  {"c", 3},  {"d", 3},  {"f", 3},  {"g", 3},  {"h", 3}, 
{"j", 3},  {"k", 3},  {"l", 3},  {"m", 3},  {"n", 3},  {"p", 3}, 
{"r", 3}, {"s", 3},  {"t", 3},  {"v", 3},  {"w", 3},
{"br", 2},  {"dr", 2},  {"fr", 2},  {"gr", 2},  {"kr", 2}
};

// Return a random value between minvalue and maxvalue, inclusive, with equal probability.
int rolldie(int minvalue, int maxvalue) {
int result;

	while (1) {
//		 result = random() % maxvalue;
		 result = getRND(maxvalue-minvalue) + minvalue - 1;
		 if ((result >= minvalue) && (result <= maxvalue)) { 
			return result;
		 }
	}
}

char genname[90];         // this accumulates the generated name.

// Create a random name.  It must have at least between minsyl and maxsyl number of syllables (inclusive).
char * RandomName(int minsyl, int maxsyl) {
char data[90];
int leng = rolldie(minsyl, maxsyl); // Compute number of syllables in the name
int isvowel = rolldie(0, 2); // randomly start with vowel or consonant
int i, j;

genname[0] = '\0';
for (i = 1; i <= leng; i++) { // syllable #. Start is 1 (not 0)
	if (isvowel) {
		strcpy(data, vowels[rolldie(0, 80)]);
			// Don't allow ' at beginning or end of name; change it to "e".
		if ( (data[0] == '\'') && ((i == 1) || (i == leng))) { data[0] = 'e';};
			strcat(genname, data);
		} else {
			do {
				j = rolldie(0, 91);
				strcpy(data, consonants[j].conson);
				j = consonants[j].flag;
					// Check if this consonant can occur in this syllable.
				if ( i == 1) { // first syllable.
					if (data[0] != '\0' && j == 2) { break;}
				} else if (i == leng) { // last syllable.
					if (data[0] != '\0' && j == 1) { break;}
				} else { break;}; // Middle syllable, any consonant allowed.
			} while (1);
			strncat(genname, &data[0], 1);
		}
		isvowel = 1 - isvowel; // Alternate between vowels and consonants.
	}
// Initial caps:
	genname[0] = toupper(genname[0]);
	return genname;
}

/********************************************************************************/
void generateRndNames() {
//	time_t ltime;
//	struct tm *ntime;
//	int seed1;
	char n1[50], n2[50], n3[50];

//	time(&ltime);
//	ntime = gmtime(&ltime);
//	seed1 = (ntime->tm_sec*400)+(ntime->tm_min*70)+ntime->tm_hour;
//	srandom(seed1);

	aGN_NameList1->clear(); aGN_NameList2->clear(); aGN_NameList3->clear();
	aGN_NameList1->add(""); aGN_NameList2->add(""); aGN_NameList3->add("");

	for (int i=0; i<6; i++) {
		strcpy(n1, RandomName(3,4));
		strcpy(n2, RandomName(3,5));
		strcpy(n3, RandomName(3,7));
		aGN_NameList1->add(n1);
		aGN_NameList2->add(n2);
		aGN_NameList3->add(n3);
		if ((i%5) == 0 && i != 0) { aGN_NameList1->add(""); aGN_NameList2->add(""); aGN_NameList3->add(""); }
	}
}

/********************************************************************************/
void npc_action(int action) {
FILE *ifile;
char buf[2048], *cp, *cp1;
int sz, i, j, ns=-1;
ADMPplayer p;

	switch (action) {
		case 0:		// generate NPC
				  a_NPCSave->activate();
				  a_NPCText->clear();
				  if (a_NPCRace->value() == 0) {
						a_NPCRace->value(getRND(6) + 1);
				  }
				  if (a_NPCClass->value() == 0) {
						a_NPCClass->value(getRND(11) + 1);
				  }
#ifndef VISUALC
				  sprintf(mybuf1, "./npc -o acibklfpS -l %d -r %s -c %s > npc.txt 2>&1", (int )a_NPCLevel->value(), races[a_NPCRace->value()].text, classes[a_NPCClass->value()].text);
#else
				  sprintf(mybuf1, "npc -o acibklfpS -l %d -r %s -c %s > npc.txt 2>&1", (int )a_NPCLevel->value(), races[a_NPCRace->value()].text, classes[a_NPCClass->value()].text);
#endif
				  system(mybuf1);
				  a_NPCText->clear();
				  if ((ifile = fopen("npc.txt", "r")) != NULL) {
						while (fgets(buf, 2047, ifile) != NULL) {
							if (strlen(buf) > 10) {
								if (strncmp(buf, ", ", 2) == 0 || strncmp(buf, "Skills", 6) == 0 || strncmp(buf, "Wizard", 6) == 0 ||
									strncmp(buf, "Sorcerer", 8) == 0 || strncmp(buf, "Bard", 4) == 0) {
									cp = strtok(buf, ";");
				  					a_NPCText->add(cp);
									while ((cp = strtok(NULL, ";")) != NULL) {
										if ((cp1 = strstr(cp, "--")) != NULL) {
											cp1 -= 5;
											*cp1 = '\0'; cp1++;
				  							a_NPCText->add(cp);
				  							a_NPCText->add(cp1);
										} else {
				  							a_NPCText->add(cp);
										}
									}
								} else {
				  					a_NPCText->add(buf);
								}
							}
						}
				  }
				  a_NPCText->add("");
				  break;
//	copy the details into a new monster slot
		case 1:
				  memset(p.c_name, '\0', sizeof(ADMPplayer));
				  sz = a_NPCText->size();
				  strcpy(buf, a_NPCText->text(1));
				  p.f_challengeRating = 1.0;
				  if ((cp = strstr(buf, " CR ")) != NULL) { *cp = '\0'; cp+=4; p.f_challengeRating = atof(cp); }
				  sprintf(p.c_name, "Z - %s", &buf[2]);
				  for (i=2; i<=sz; i++) {
				  		strcpy(buf, a_NPCText->text(i));
						if (strstr(buf, "HD") != NULL) { strcpy(p.c_hitDice, &buf[4]); }
						else if (strstr(buf, "hp") != NULL) { p.i_hp[HP_MAX] = p.i_hp[HP_CUR] = atoi(&buf[4]); }
						else if (strstr(buf, "Init") != NULL) { p.i_initiative[0] = atoi(&buf[7]); }
						else if (strstr(buf, "AC") != NULL) { p.i_armorClass[AC_NORMAL] = atoi(&buf[4]); }
						else if (strstr(buf, "Spd") != NULL) { p.i_speed = atoi(&buf[5]); }
						else if (strstr(buf, "Languages") != NULL) { ns = i; }
						else if (strstr(buf, "SV") != NULL) { 
							if ((cp = strrchr(buf, ',')) != NULL) {
								if ((cp1 = strstr(cp, "+")) != NULL) { p.i_savingThrows[SAVE_WISDOM] = atoi(cp);  }
								else if ((cp1 = strstr(cp, "-")) != NULL) { p.i_savingThrows[SAVE_WISDOM] = atoi(cp);  }
								cp = '\0';
							}
							if ((cp = strrchr(buf, ',')) != NULL) {
								if ((cp1 = strstr(cp, "+")) != NULL) { p.i_savingThrows[SAVE_REFLEX] = atoi(cp);  }
								else if ((cp1 = strstr(cp, "-")) != NULL) { p.i_savingThrows[SAVE_REFLEX] = atoi(cp);  }
								cp = '\0';
							}
							if ((cp = strstr(buf, "+")) != NULL) {
								cp1 = strstr(cp, ","); *cp1 = '\0'; cp1++;
							} else if ((cp = strstr(buf, "-")) != NULL) {
								cp1 = strstr(cp, ","); *cp1 = '\0'; cp1++;
							}
							p.i_savingThrows[SAVE_FORTITUDE] = atoi(cp);
						} else if (strstr(buf, "Skills") != NULL) {
							do {
								j = 0;
								if ((cp = strstr(buf, "+")) != NULL) { if ((cp1 = strstr(cp, " ")) != NULL) { *cp1 = '\0'; j = atoi(cp); }}
								else if ((cp = strstr(buf, "-")) != NULL) { if ((cp1 = strstr(cp, " ")) != NULL) { *cp1 = '\0'; j = atoi(cp); }}
								if (strstr(buf, "Hide") != NULL) { p.i_skills[SKILL_HIDE] = j;
								} else if (strstr(buf, "Listen") != NULL) { p.i_skills[SKILL_LISTEN] = j;
								} else if (strstr(buf, "Spot") != NULL) { p.i_skills[SKILL_SPOT] = j;
								} else if (strstr(buf, "Move") != NULL) { p.i_skills[SKILL_MOVESILENTLY] = j;
								} else if (strstr(buf, "Search") != NULL) { p.i_skills[SKILL_SEARCH] = j;
								} else if (strstr(buf, "Open") != NULL) { p.i_skills[SKILL_OPENLOCKS] = j;
								} else if (strstr(buf, "Concent") != NULL) { p.i_skills[SKILL_CONCENTRATION] = j;
								} else if (strstr(buf, "Sleight") != NULL) { p.i_skills[SKILL_SOH] = j;
								}
								i++; strcpy(buf, a_NPCText->text(i));
							} while (buf[0] == ' ' && i<= sz);
						} else if (strstr(buf, "Spells") != NULL) {
							do {
								i++; strcpy(buf, a_NPCText->text(i));
							} while (buf[0] == ' ' && i<= sz);
						} else if (strstr(buf, "Attack") != NULL) {
							p.i_noAttacks = 1;
							p.w_weapons[0].i_noAttacks = 1;
							p.w_weapons[6].i_noAttacks = 1;
							if ((cp = strstr(buf, "melee")) != NULL) {
								*cp = '\0'; cp++;
								if ((cp1 = strstr(cp, "(")) != NULL) { *cp1 = '\0'; }
								j = 0;
								strcpy(p.w_weapons[6].c_description, "Unknown Ranged");
								strcpy(p.w_weapons[6].c_damage, "1d6");
								strcpy(p.w_weapons[6].c_critical, "*3");
								p.w_weapons[6].i_range = 60;
								if ((cp = strstr(cp, "+")) != NULL) { if ((cp1 = strstr(cp, " ")) != NULL) { *cp1 = '\0'; } }
								else if ((cp = strstr(cp, "-")) != NULL) { if ((cp1 = strstr(cp, " ")) != NULL) { *cp1 = '\0'; } }

								if ((cp1 = strstr(cp, "/")) != NULL) {
									while ((cp1 = strstr(cp, "/")) != NULL && j < 4) {
										*cp1 = '\0';
										p.w_weapons[6].i_attackBonus[j++] = atoi(cp);
										cp = cp1; cp++;
									}
									p.w_weapons[6].i_attackBonus[j++] = atoi(cp);
									p.w_weapons[6].i_noAttacks = j;
									p.w_weapons[0].i_noAttacks = j;
									p.i_noAttacks = j;
								} else {
									p.w_weapons[6].i_attackBonus[0] = atoi(cp);
								}
							}

							if ((cp1 = strstr(buf, "(")) != NULL) { *cp1 = '\0'; }
							strcpy(p.w_weapons[0].c_description, "Unknown Melee");
							strcpy(p.w_weapons[0].c_damage, "1d8");
							strcpy(p.w_weapons[0].c_critical, "19-20/*2");
							j = 0;
							if ((cp = strstr(cp, "+")) != NULL) { if ((cp1 = strstr(cp, " ")) != NULL) { *cp1 = '\0'; } }
							else if ((cp = strstr(cp, "-")) != NULL) { if ((cp1 = strstr(cp, " ")) != NULL) { *cp1 = '\0'; } }

							if ((cp1 = strstr(cp, "/")) != NULL) {
								while ((cp1 = strstr(cp, "/")) != NULL && j < 4) {
									*cp1 = '\0';
									p.w_weapons[0].i_attackBonus[j++] = atoi(cp);
									cp = cp1; cp++;
								}
								p.w_weapons[0].i_attackBonus[j++] = atoi(cp);
							} else {
								p.w_weapons[0].i_attackBonus[0] = atoi(cp);
							}
						}
				  }
				  for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
						if (players[i].c_name[0] == '\0' && players[i].i_hp[HP_MAX] == 0) {
							memcpy(players[i].c_name, p.c_name, sizeof(ADMPplayer));
							players[i].c_name[0] = identifiers[i];
							if (ns != -1) {
								sprintf(buf, "%s.note", players[i].c_name);
								if ((ifile = fopen(buf, "w+")) != NULL) {
									for (i=ns; i<=sz; i++) {
				  						fprintf(ifile, "%s\n", a_NPCText->text(i));
									}
									fclose(ifile);
								}
							}
							break;
						}
				  }
				  reloadNames(0);
				  doSystemAlert("Dont forget to change the weapon details in the Edit tab !!!");
				  break;
	}
}

/********************************************************************************/
void dump_player_data(int pid) {
	printf("strcpy(p[i].c_name, \"%s\");\n", &players[pid].c_name[4]);
	printf("p[i].i_race = %d;\n", players[pid].i_race);
	printf("p[i].i_alignment = %d;\n", players[pid].i_alignment);
	printf("p[i].i_classes[0] = %d;\n", players[pid].i_classes[0]);
	printf("p[i].i_classes[1] = %d;\n", players[pid].i_classes[1]);
	printf("p[i].i_classes[2] = %d;\n", players[pid].i_classes[2]);
	printf("p[i].i_classes[3] = %d;\n", players[pid].i_classes[3]);
	printf("p[i].i_levels[0] = %d;\n", players[pid].i_levels[0]);
	printf("p[i].i_levels[1] = %d;\n", players[pid].i_levels[1]);
	printf("p[i].i_levels[2] = %d;\n", players[pid].i_levels[2]);
	printf("p[i].i_levels[3] = %d;\n", players[pid].i_levels[3]);
	printf("p[i].i_hp[HP_MAX] = %d;\n", players[pid].i_hp[HP_MAX]);
	printf("p[i].i_hp[HP_CUR] = %d;\n", players[pid].i_hp[HP_CUR]);
	printf("p[i].i_hp[HP_NL] = %d;\n", players[pid].i_hp[HP_NL]);
	printf("p[i].i_hp[HP_TMP] = %d;\n", players[pid].i_hp[HP_TMP]);
	printf("p[i].i_bleeding = %d;\n", players[pid].i_bleeding);
	printf("p[i].i_disabledAt = %d;\n", players[pid].i_disabledAt);
	printf("p[i].i_unconciousAt = %d;\n", players[pid].i_unconciousAt);
	printf("p[i].i_savingThrows[SAVE_FORTITUDE] = %d;\n", players[pid].i_savingThrows[SAVE_FORTITUDE]);
	printf("p[i].i_savingThrows[SAVE_REFLEX] = %d;\n", players[pid].i_savingThrows[SAVE_REFLEX]);
	printf("p[i].i_savingThrows[SAVE_WISDOM] = %d;\n", players[pid].i_savingThrows[SAVE_WISDOM]);
	printf("p[i].i_armorClass[AC_NORMAL] = %d;\n", players[pid].i_armorClass[AC_NORMAL]);
	printf("p[i].i_armorClass[AC_TOUCH] = %d;\n", players[pid].i_armorClass[AC_TOUCH]);
	printf("p[i].i_armorClass[AC_FLATFOOTED] = %d;\n", players[pid].i_armorClass[AC_FLATFOOTED]);
	printf("p[i].i_armorClassSplit[AC_SECTION_DEX] = %d;\n", players[pid].i_armorClassSplit[AC_SECTION_DEX]);
	printf("p[i].i_armorClassSplit[AC_SECTION_SIZE] = %d;\n", players[pid].i_armorClassSplit[AC_SECTION_SIZE]);
	printf("p[i].i_armorClassSplit[AC_SECTION_DEFLECTION] = %d;\n", players[pid].i_armorClassSplit[AC_SECTION_DEFLECTION]);
	printf("p[i].i_armorClassSplit[AC_SECTION_NATURAL] = %d;\n", players[pid].i_armorClassSplit[AC_SECTION_NATURAL]);
	printf("p[i].i_armorClassSplit[AC_SECTION_ARMOR] = %d;\n", players[pid].i_armorClassSplit[AC_SECTION_ARMOR]);
	printf("p[i].i_armorClassSplit[AC_SECTION_SHIELD] = %d;\n", players[pid].i_armorClassSplit[AC_SECTION_SHIELD]);
	printf("p[i].i_bab[BAB_MELEE][0] = %d;\n", players[pid].i_bab[BAB_MELEE][0]);
	printf("p[i].i_bab[BAB_RANGED][0] = %d;\n", players[pid].i_bab[BAB_RANGED][0]);
	printf("p[i].i_bab[BAB_GRAPPLE][0] = %d;\n", players[pid].i_bab[BAB_GRAPPLE][0]);
	printf("p[i].i_bab[BAB_MELEE][1] = %d;\n", players[pid].i_bab[BAB_MELEE][1]);
	printf("p[i].i_bab[BAB_RANGED][1] = %d;\n", players[pid].i_bab[BAB_RANGED][1]);
	printf("p[i].i_bab[BAB_GRAPPLE][1] = %d;\n", players[pid].i_bab[BAB_GRAPPLE][1]);
	printf("p[i].i_bab[BAB_MELEE][2] = %d;\n", players[pid].i_bab[BAB_MELEE][2]);
	printf("p[i].i_bab[BAB_RANGED][2] = %d;\n", players[pid].i_bab[BAB_RANGED][2]);
	printf("p[i].i_bab[BAB_GRAPPLE][2] = %d;\n", players[pid].i_bab[BAB_GRAPPLE][2]);
	printf("p[i].i_initiative[0] = %d;\n", players[pid].i_initiative[0]);
	printf("p[i].i_initiative[1] = %d;\n", players[pid].i_initiative[1]);
	printf("p[i].i_speed = %d;\n", players[pid].i_speed);
	printf("p[i].i_inactive = %d;\n", players[pid].i_inactive);
	printf("p[i].i_stun = %d;\n", players[pid].i_stun);
	printf("p[i].i_noAttacks = %d;\n", players[pid].i_noAttacks);
	
	for (int i=0; i<MAX_WEAPONS; i++) {
		if (players[i].w_weapons[i].c_description[0] != '\0') {
			printf("p[i].w_weapons[%d].c_available = %d;\n", i, players[pid].w_weapons[i].c_available);
			printf("strcpy(p[i].w_weapons[%d].c_description, \"%s\");\n", i, players[pid].w_weapons[i].c_description);
			printf("p[i].w_weapons[%d].i_attackBonus[0] = %d;\n", i, players[pid].w_weapons[i].i_attackBonus[0]);
			printf("p[i].w_weapons[%d].i_attackBonus[1] = %d;\n", i, players[pid].w_weapons[i].i_attackBonus[1]);
			printf("p[i].w_weapons[%d].i_attackBonus[2] = %d;\n", i, players[pid].w_weapons[i].i_attackBonus[2]);
			printf("p[i].w_weapons[%d].i_attackBonus[3] = %d;\n", i, players[pid].w_weapons[i].i_attackBonus[3]);
			printf("strcpy(p[i].w_weapons[%d].c_damage, \"%s\");\n", i, players[pid].w_weapons[i].c_damage);
			printf("strcpy(p[i].w_weapons[%d].c_critical, \"%s\");\n", i, players[pid].w_weapons[i].c_critical);
			printf("p[i].w_weapons[%d].i_magicalBonus = %d;\n", i, players[pid].w_weapons[i].i_magicalBonus);
			printf("p[i].w_weapons[%d].i_range = %d;\n", i, players[pid].w_weapons[i].i_range);
			printf("p[i].w_weapons[%d].i_reach = %d;\n", i, players[pid].w_weapons[i].i_reach);
			printf("p[i].w_weapons[%d].i_noAttacks = %d;\n", i, players[pid].w_weapons[i].i_noAttacks);
			printf("strcpy(p[i].w_weapons[%d].c_message, \"%s\");\n", i, players[pid].w_weapons[i].c_message);
			printf("p[i].w_weapons[%d].c_attackType = %d;\n", i, players[pid].w_weapons[i].c_attackType);
		}
	}
	
	printf("p[i].f_challengeRating = %.2f;\n", players[pid].f_challengeRating);
	printf("p[i].i_d20Rolls[0] = %d;\n", players[pid].i_d20Rolls[0]);
	printf("p[i].i_d20Rolls[1] = %d;\n", players[pid].i_d20Rolls[1]);
	printf("p[i].i_d20Rolls[2] = %d;\n", players[pid].i_d20Rolls[2]);
	printf("p[i].i_d20Rolls[3] = %d;\n", players[pid].i_d20Rolls[3]);
	printf("p[i].i_d20Rolls[4] = %d;\n", players[pid].i_d20Rolls[4]);
	printf("p[i].i_d20Rolls[5] = %d;\n", players[pid].i_d20Rolls[5]);
	printf("p[i].i_d20Rolls[6] = %d;\n", players[pid].i_d20Rolls[6]);
	printf("p[i].i_spellDC[0] = %d;\n", players[pid].i_spellDC[0]);
	printf("p[i].i_spellDC[1] = %d;\n", players[pid].i_spellDC[1]);
	printf("p[i].i_spellDC[2] = %d;\n", players[pid].i_spellDC[2]);
	printf("p[i].i_spellDC[3] = %d;\n", players[pid].i_spellDC[3]);
	printf("p[i].i_spellDC[4] = %d;\n", players[pid].i_spellDC[4]);
	printf("p[i].i_spellDC[5] = %d;\n", players[pid].i_spellDC[5]);
	printf("p[i].i_spellDC[6] = %d;\n", players[pid].i_spellDC[6]);
	printf("p[i].i_spellFailure = %d;\n", players[pid].i_spellFailure);
	printf("p[i].i_abilityStats[0][0] = %d;\n", players[pid].i_abilityStats[0][0]);
	printf("p[i].i_abilityStats[1][0] = %d;\n", players[pid].i_abilityStats[1][0]);
	printf("p[i].i_abilityStats[0][1] = %d;\n", players[pid].i_abilityStats[0][1]);
	printf("p[i].i_abilityStats[1][1] = %d;\n", players[pid].i_abilityStats[1][1]);
	printf("p[i].i_abilityStats[0][2] = %d;\n", players[pid].i_abilityStats[0][2]);
	printf("p[i].i_abilityStats[1][2] = %d;\n", players[pid].i_abilityStats[1][2]);
	printf("p[i].i_abilityStats[0][3] = %d;\n", players[pid].i_abilityStats[0][3]);
	printf("p[i].i_abilityStats[1][3] = %d;\n", players[pid].i_abilityStats[1][3]);
	printf("p[i].i_abilityStats[0][4] = %d;\n", players[pid].i_abilityStats[0][4]);
	printf("p[i].i_abilityStats[1][4] = %d;\n", players[pid].i_abilityStats[1][4]);
	printf("p[i].i_abilityStats[0][5] = %d;\n", players[pid].i_abilityStats[0][5]);
	printf("p[i].i_abilityStats[1][5] = %d;\n", players[pid].i_abilityStats[1][5]);
	printf("p[i].i_skills[SKILL_SEARCH] = %d;\n", players[pid].i_skills[SKILL_SEARCH]);
	printf("p[i].i_skills[SKILL_SOH] = %d;\n", players[pid].i_skills[SKILL_SOH]);
	printf("p[i].i_skills[SKILL_SPOT] = %d;\n", players[pid].i_skills[SKILL_SPOT]);
	printf("p[i].i_skills[SKILL_OPENLOCKS] = %d;\n", players[pid].i_skills[SKILL_OPENLOCKS]);
	printf("p[i].i_skills[SKILL_MOVESILENTLY] = %d;\n", players[pid].i_skills[SKILL_MOVESILENTLY]);
	printf("p[i].i_skills[SKILL_LISTEN] = %d;\n", players[pid].i_skills[SKILL_LISTEN]);
	printf("p[i].i_skills[SKILL_HIDE] = %d;\n", players[pid].i_skills[SKILL_HIDE]);
	printf("p[i].i_skills[SKILL_CONCENTRATION] = %d;\n", players[pid].i_skills[SKILL_CONCENTRATION]);
	printf("p[i].i_spellResistance = %d;\n", players[pid].i_spellResistance);
	printf("p[i].i_damageReduction[0] = %d;\n", players[pid].i_damageReduction[0]);
	printf("p[i].i_damageReduction[1] = %d;\n", players[pid].i_damageReduction[1]);
	printf("p[i].i_damageReduction[2] = %d;\n", players[pid].i_damageReduction[2]);
	printf("p[i].i_size = %d;\n", players[pid].i_size);
	printf("p[i].i_xp[MAX] = %d;\n", players[pid].i_xp[MAX]);
	printf("p[i].i_xp[TMP] = %d;\n", players[pid].i_xp[TMP]);
	printf("p[i].i_inGroup = %d;\n", players[pid].i_inGroup);
	printf("strcpy(p[i].c_hitDice, \"%s\");\n", players[pid].c_hitDice);
	printf("p[i].flags.f_isStabilised = %d;\n", players[pid].flags.f_isStabilised);
	printf("p[i].flags.f_disabled = %d;\n", players[pid].flags.f_disabled);
	printf("p[i].flags.f_ignoreCriticals = %d;\n", players[pid].flags.f_ignoreCriticals);
	printf("p[i].flags.f_autoStablise = %d;\n", players[pid].flags.f_autoStablise);
	printf("p[i].spellsAvailable.i_maxLevel0 = %d;\n", players[pid].spellsAvailable.i_maxLevel0);
	printf("p[i].spellsAvailable.i_maxLevel1 = %d;\n", players[pid].spellsAvailable.i_maxLevel1);
	printf("p[i].spellsAvailable.i_maxLevel2 = %d;\n", players[pid].spellsAvailable.i_maxLevel2);
	printf("p[i].spellsAvailable.i_maxLevel3 = %d;\n", players[pid].spellsAvailable.i_maxLevel3);
	printf("p[i].spellsAvailable.i_maxLevel4 = %d;\n", players[pid].spellsAvailable.i_maxLevel4);
	printf("p[i].spellsAvailable.i_maxLevel5 = %d;\n", players[pid].spellsAvailable.i_maxLevel5);
	printf("p[i].spellsAvailable.i_maxLevel6 = %d;\n", players[pid].spellsAvailable.i_maxLevel6);
	printf("p[i].spellsAvailable.i_maxLevel7 = %d;\n", players[pid].spellsAvailable.i_maxLevel7);
	printf("p[i].spellsAvailable.i_maxLevel8 = %d;\n", players[pid].spellsAvailable.i_maxLevel8);
	printf("p[i].spellsAvailable.i_maxLevel9 = %d;\n", players[pid].spellsAvailable.i_maxLevel9);
	printf("p[i].spellsAvailable.i_curLevel0 = %d;\n", players[pid].spellsAvailable.i_maxLevel0);
	printf("p[i].spellsAvailable.i_curLevel1 = %d;\n", players[pid].spellsAvailable.i_maxLevel1);
	printf("p[i].spellsAvailable.i_curLevel2 = %d;\n", players[pid].spellsAvailable.i_maxLevel2);
	printf("p[i].spellsAvailable.i_curLevel3 = %d;\n", players[pid].spellsAvailable.i_maxLevel3);
	printf("p[i].spellsAvailable.i_curLevel4 = %d;\n", players[pid].spellsAvailable.i_maxLevel4);
	printf("p[i].spellsAvailable.i_curLevel5 = %d;\n", players[pid].spellsAvailable.i_maxLevel5);
	printf("p[i].spellsAvailable.i_curLevel6 = %d;\n", players[pid].spellsAvailable.i_maxLevel6);
	printf("p[i].spellsAvailable.i_curLevel7 = %d;\n", players[pid].spellsAvailable.i_maxLevel7);
	printf("p[i].spellsAvailable.i_curLevel8 = %d;\n", players[pid].spellsAvailable.i_maxLevel8);
	printf("p[i].spellsAvailable.i_curLevel9 = %d;\n", players[pid].spellsAvailable.i_maxLevel9);
	printf("i++;\n\n");
}

void dump_player_details() {
	int i;
	for (i=MAX_PLAYERS; i<25; i++) { dump_player_data(i); }
}

/********************************************************************************/
void calcELLevel() {
	int i=0,j=0;

	aELC_Results->clear();
	i = (int )aELC_ELLevel->value();
//	sprintf(mybuf2, "Less than %d = why bother !", i); aELC_Results->add(mybuf2);
	if (i < 4) {
		sprintf(mybuf2, "1 * CR %d,%d", i,i+1); aELC_Results->add(mybuf2);
	} else {
		sprintf(mybuf2, "1 * CR %d,%d,%d", i-1,i,i+1); aELC_Results->add(mybuf2);
	}
	for (j=2; j<6; j++) {
		if ((i-j) == 1) { sprintf(mybuf2, "%d * CR 1,2", j); aELC_Results->add(mybuf2); } else
		if ((i-j) == 0) { sprintf(mybuf2, "%d * CR 1", j); aELC_Results->add(mybuf2); } else
		if ((i-j) > 1) { sprintf(mybuf2, "%d * CR %d", j, (i-j)); aELC_Results->add(mybuf2); } 
	}
	aELC_Results->add("");
	if (i > 3) { sprintf(mybuf2, "1 * CR %d + 1 * CR %d", i-1, i-3); aELC_Results->add(mybuf2); } else
	if (i == 3) { sprintf(mybuf2, "1 * CR 2 + 1 * CR 1"); aELC_Results->add(mybuf2); } else
	if (i == 2) { sprintf(mybuf2, "1 * CR 1 + 1 * CR 0.5"); aELC_Results->add(mybuf2); } else
	if (i < 2) { sprintf(mybuf2, "1 * CR 0.5 + 1 * CR 0.3"); aELC_Results->add(mybuf2); } 
}

/********************************************************************************/
const char *htmlLoadLink(Fl_Widget *w, const char *c) {	
//	printf("hLL: 0 : %s\n", c);
	if (a_SRDText->filename() != NULL) {
		htmlLevel++;
		if (htmlLevel >= 4) { htmlLevel = 3; }
		strcpy(&prevHtmlPage[0][0], a_SRDText->filename());
if (config.flags.f_debug != 0) { printf("hLL: 0 : %d %s\n", htmlLevel, &prevHtmlPage[0][0]); }
		if (htmlLevel == 0) { prevHtmlPage[1][0] = '\0'; prevHtmlPage[2][0] = '\0'; }
	}
	return(c);
}

/********************************************************************************/
void doMassSkillCheck(int option) {
	int i,j,k,l,m;

if (config.flags.f_debug != 0) { printf("dMSC: %d\n", option); }

	switch (option) {
		case 0: aMSC_DoingList->clear();
				  j=0;
				  for (i=0; i < MAX_MEMBERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0 && players[i].flags.f_disabled == 0) {
							config.i_idList[ID_MSKILLCHK][j++] = i;
							aMSC_DoingList->add(players[i].c_name);
						}
				  }
				  config.i_idList[ID_MSKILLCHK][j] = -1;
				  aMSC_DCValue->value("0");
				  break;

		case 1:
				  break;

		case 2:
				  k = atoi(aMSC_DCValue->value());
				  if (k > 0) {
if (config.flags.f_debug != 0) { printf("dMSC: %d %d %d\n", option, aMSC_flags->value(), aMSC_SkillList->value()); }
						aMSC_ResultList->clear();
						for (i=0; i < aMSC_DoingList->nitems(); i++) {
							if (aMSC_DoingList->checked(i) != 0) {				// is it marked
								j = config.i_idList[ID_MSKILLCHK][i-1];		// get the player ID
								if (aMSC_flags->value() == 1) {					// roll a d20
									l = getRND(20);
								}
								else if (aMSC_flags->value() == 2) {			// use a prerolled d20
									for (m=6; m >= 0; m--) {
										if (players[j].i_d20Rolls[m] > 0) {
											l = players[j].i_d20Rolls[m];
											players[j].i_d20Rolls[m] = 0;
											break;
										}
									}
									if (m < 0) {										// were there any left to use
										l = getRND(20);
									}
								} 
								l += getAdjustment(j, MOD_SKILL) + players[j].i_skills[aMSC_SkillList->value()];
if (config.flags.f_debug != 0) { printf("dMSC: %d %s (%d:%d:%d)\n", option, players[j].c_name, l, k, players[j].i_skills[aMSC_SkillList->value()]); }

								if (l >= k) {		// did they fail or succeed
									if (m < 0) {
										sprintf(mybuf2, "@B63@.>> %s by %d (Rolled %d)", players[j].c_name, (l-k), l);
									} else {
										sprintf(mybuf2, "@B63@.%s by %d (Rolled %d)", players[j].c_name, (l-k), l);
									}
								} else {
									if (m < 0) {
										sprintf(mybuf2, "@B88@.>> %s by %d (Rolled %d)", players[j].c_name, (l-k)*-1, l);
									} else {
										sprintf(mybuf2, "@B88@.%s by %d (Rolled %d)", players[j].c_name, (l-k)*-1, l);
									}
								}
								aMSC_ResultList->add(mybuf2);
							}
						}
				  }
				  break;
	}
	a_TMassSkillCheck->redraw();
	return;
}

/********************************************************************************/
void dumpAdjustments() {
	int i,j,k;

	for (i=0; i<MAX_MODIFIERS; i++) {
		printf("strcpy(config.system_mods[%d].c_message, \"%s\");\n", i, config.system_mods[i].c_message);
		printf("config.system_mods[%d].i_noRounds = %d;\n", i, config.system_mods[i].i_noRounds);
		for (j=0; j<5; j++) {
			for (k=0; k<3; k++) {
				printf("config.system_mods[%d].i_data[%d][%d] = %d;\n", i, j,k, config.system_mods[i].i_data[j][k]);
			}
		}
	}
}

/********************************************************************************/
void loadStdModifiers() {
#include "stdmodifiers.h"
}

/********************************************************************************/
void rollRNDDice() {
	int dt[8] = {4,6,8,10,12,20,30,100}, sm[8] = {0,0,0,0,0,0,0,0},
		     rd[8] = {0,0,0,0,0,0,0,0};
	char buf[256];

	a_ODRList1->clear(); a_ODRList2->clear(); a_ODRList3->clear(); a_ODRList4->clear();
	a_ODRList5->clear(); a_ODRList6->clear(); a_ODRList7->clear(); a_ODRList8->clear();

	a_ODRList1->add("@b  d4"); a_ODRList2->add("@b  d6"); a_ODRList3->add("@b  d8"); a_ODRList4->add("@b   d10");
	a_ODRList5->add("@b   d12"); a_ODRList6->add("@b   d20"); a_ODRList7->add("@b   d30"); a_ODRList8->add("@b   d100");

	for (int k=(int )a_ODRCount->value(); k > 0; k--) {
		rd[0] = getRND(dt[0]); sm[0] += rd[0];
		rd[1] = getRND(dt[1]); sm[1] += rd[1];
		rd[2] = getRND(dt[2]); sm[2] += rd[2];
		rd[3] = getRND(dt[3]); sm[3] += rd[3];
		rd[4] = getRND(dt[4]); sm[4] += rd[4];
		rd[5] = getRND(dt[5]); sm[5] += rd[5];
		rd[6] = getRND(dt[6]); sm[6] += rd[6];
		rd[7] = getRND(dt[7]); sm[7] += rd[7];
		sprintf(buf, "%d (%d)", rd[0], sm[0]); a_ODRList1->add(buf);
		sprintf(buf, "%d (%d)", rd[1], sm[1]); a_ODRList2->add(buf);
		sprintf(buf, "%d (%d)", rd[2], sm[2]); a_ODRList3->add(buf);
		sprintf(buf, "%2d (%d)", rd[3], sm[3]); a_ODRList4->add(buf);
		sprintf(buf, "%2d (%d)", rd[4], sm[4]); a_ODRList5->add(buf);
		sprintf(buf, "%2d (%d)", rd[5], sm[5]); a_ODRList6->add(buf);
		sprintf(buf, "%2d (%d)", rd[6], sm[6]); a_ODRList7->add(buf);
		sprintf(buf, "%3d (%d)", rd[7], sm[7]); a_ODRList8->add(buf);
	}
}

/********************************************************************************/
void doSystemExit() {
	a_Exit->show();
	for (;;) {
		Fl_Widget *o = Fl::readqueue();
		if (!o) {
			Fl::wait();
		} else if ( o == a_EBExit) { exit(0);
		} else if ( o == a_EBCancel) { break;
		} else if ( o == a_EBSaveExit) { saveAllData(); exit(0);
		}
	}
	a_Exit->hide();
}

/********************************************************************************/
void qkMonsterAction(int action) {		// set the color of the selected monster
static int selected=-1;
int i;

	i = config.i_idList[ID_QMON][selected];

if (config.flags.f_debug != 0) { printf("qkMA: %d %d %d\n", action, i, selected); }

	switch (action) {
		case 0: selected = -1;
				  break;

		case 1: if (selected != -1) {
				  		sprintf(mybuf2, "%s (CR: %.1f)", &qkmonsters[i].c_name[0], qkmonsters[i].f_challengeRating);
				  		a_EdtQkMonList->text(selected+1, mybuf2);
				  }
				  selected = a_EdtQkMonList->value()-1;
				  i = config.i_idList[ID_QMON][selected];	
				  if (selected < 0 || selected > MAX_MEMBERS) { selected = -1; return; }
				  sprintf(mybuf2, "@B95@.%s (CR: %.1f)", &qkmonsters[i].c_name[0], qkmonsters[i].f_challengeRating);
				  a_EdtQkMonList->text(selected+1, mybuf2);
				  break;
	}
}

/********************************************************************************/
int pi_id, wi_id;

int writePlayerWeaponData() {
FILE *wfile;
ADMPweapon *wp;

	wfile = pdbFilePointer;
	if ((wp = (ADMPweapon *)pdbDataPointer) == NULL) { return -1; }

//	pdbPutInt(wfile, players[pid].i_id);
	pdbPutInt(wfile, pi_id);
	pdbPutInt(wfile, wi_id);
	pdbPutInt(wfile, wp->c_available);
	pdbPutString(wfile, &wp->c_description[0]);
	pdbPutInt(wfile, wp->i_attackBonus[0]);
	pdbPutInt(wfile, wp->i_attackBonus[1]);
	pdbPutInt(wfile, wp->i_attackBonus[2]);
	pdbPutInt(wfile, wp->i_attackBonus[3]);
	pdbPutString(wfile, wp->c_damage);
	pdbPutString(wfile, wp->c_critical);
	pdbPutInt(wfile, wp->i_magicalBonus);
	pdbPutInt(wfile, wp->i_range);
	pdbPutInt(wfile, wp->i_reach);
	pdbPutInt(wfile, wp->i_noAttacks);
	pdbPutString(wfile, wp->c_message);
	pdbPutInt(wfile, wp->c_attackType);

	return 0;
}

/********************************************************************************/
int writePlayerData() {
FILE *ofile;
ADMPplayer *pd;

	ofile = pdbFilePointer;
	if ((pd = (ADMPplayer *)pdbDataPointer) == NULL) { return -1; }

	pdbPutInt(ofile, pi_id);

	pdbPutString(ofile, pd->c_name);
	pdbPutInt(ofile, pd->i_race);
	pdbPutInt(ofile, pd->i_alignment);
	pdbPutInt(ofile, pd->i_classes[0]);
	pdbPutInt(ofile, pd->i_classes[1]);
	pdbPutInt(ofile, pd->i_classes[2]);
	pdbPutInt(ofile, pd->i_classes[3]);
	pdbPutInt(ofile, pd->i_levels[0]);
	pdbPutInt(ofile, pd->i_levels[1]);
	pdbPutInt(ofile, pd->i_levels[2]);
	pdbPutInt(ofile, pd->i_levels[3]);
	pdbPutInt(ofile, pd->i_hp[HP_MAX]);
	pdbPutInt(ofile, pd->i_hp[HP_CUR]);
	pdbPutInt(ofile, pd->i_hp[HP_NL]);
	pdbPutInt(ofile, pd->i_hp[HP_TMP]);
	pdbPutInt(ofile, pd->i_bleeding);
	pdbPutInt(ofile, pd->i_disabledAt);
	pdbPutInt(ofile, pd->i_unconciousAt);
	pdbPutInt(ofile, pd->i_savingThrows[SAVE_FORTITUDE]);
	pdbPutInt(ofile, pd->i_savingThrows[SAVE_REFLEX]);
	pdbPutInt(ofile, pd->i_savingThrows[SAVE_WISDOM]);
	pdbPutInt(ofile, pd->i_armorClass[AC_NORMAL]);
	pdbPutInt(ofile, pd->i_armorClass[AC_TOUCH]);
	pdbPutInt(ofile, pd->i_armorClass[AC_FLATFOOTED]);
	pdbPutInt(ofile, pd->i_armorClassSplit[AC_SECTION_DEX]);
	pdbPutInt(ofile, pd->i_armorClassSplit[AC_SECTION_SIZE]);
	pdbPutInt(ofile, pd->i_armorClassSplit[AC_SECTION_DEFLECTION]);
	pdbPutInt(ofile, pd->i_armorClassSplit[AC_SECTION_NATURAL]);
	pdbPutInt(ofile, pd->i_armorClassSplit[AC_SECTION_ARMOR]);
	pdbPutInt(ofile, pd->i_armorClassSplit[AC_SECTION_SHIELD]);
	pdbPutInt(ofile, pd->i_bab[BAB_MELEE][0]);
	pdbPutInt(ofile, pd->i_bab[BAB_RANGED][0]);
	pdbPutInt(ofile, pd->i_bab[BAB_GRAPPLE][0]);
	pdbPutInt(ofile, pd->i_bab[BAB_MELEE][1]);
	pdbPutInt(ofile, pd->i_bab[BAB_RANGED][1]);
	pdbPutInt(ofile, pd->i_bab[BAB_GRAPPLE][1]);
	pdbPutInt(ofile, pd->i_bab[BAB_MELEE][2]);
	pdbPutInt(ofile, pd->i_bab[BAB_RANGED][2]);
	pdbPutInt(ofile, pd->i_bab[BAB_GRAPPLE][2]);
	pdbPutInt(ofile, pd->i_initiative[0]);
	pdbPutInt(ofile, pd->i_initiative[1]);
	pdbPutInt(ofile, pd->i_speed);
	pdbPutInt(ofile, pd->i_inactive);
	pdbPutInt(ofile, pd->i_stun);
	pdbPutInt(ofile, pd->i_noAttacks);

	pdbPutFloat(ofile, pd->f_challengeRating);
	pdbPutInt(ofile, pd->i_d20Rolls[0]);
	pdbPutInt(ofile, pd->i_d20Rolls[1]);
	pdbPutInt(ofile, pd->i_d20Rolls[2]);
	pdbPutInt(ofile, pd->i_d20Rolls[3]);
	pdbPutInt(ofile, pd->i_d20Rolls[4]);
	pdbPutInt(ofile, pd->i_d20Rolls[5]);
	pdbPutInt(ofile, pd->i_d20Rolls[6]);
	pdbPutInt(ofile, pd->i_spellDC[0]);
	pdbPutInt(ofile, pd->i_spellDC[1]);
	pdbPutInt(ofile, pd->i_spellDC[2]);
	pdbPutInt(ofile, pd->i_spellDC[3]);
	pdbPutInt(ofile, pd->i_spellDC[4]);
	pdbPutInt(ofile, pd->i_spellDC[5]);
	pdbPutInt(ofile, pd->i_spellDC[6]);
	pdbPutInt(ofile, pd->i_spellFailure);
	pdbPutInt(ofile, pd->i_abilityStats[0][0]);
	pdbPutInt(ofile, pd->i_abilityStats[1][0]);
	pdbPutInt(ofile, pd->i_abilityStats[0][1]);
	pdbPutInt(ofile, pd->i_abilityStats[1][1]);
	pdbPutInt(ofile, pd->i_abilityStats[0][2]);
	pdbPutInt(ofile, pd->i_abilityStats[1][2]);
	pdbPutInt(ofile, pd->i_abilityStats[0][3]);
	pdbPutInt(ofile, pd->i_abilityStats[1][3]);
	pdbPutInt(ofile, pd->i_abilityStats[0][4]);
	pdbPutInt(ofile, pd->i_abilityStats[1][4]);
	pdbPutInt(ofile, pd->i_abilityStats[0][5]);
	pdbPutInt(ofile, pd->i_abilityStats[1][5]);
	pdbPutInt(ofile, pd->i_skills[SKILL_SEARCH]);
	pdbPutInt(ofile, pd->i_skills[SKILL_SOH]);
	pdbPutInt(ofile, pd->i_skills[SKILL_SPOT]);
	pdbPutInt(ofile, pd->i_skills[SKILL_OPENLOCKS]);
	pdbPutInt(ofile, pd->i_skills[SKILL_MOVESILENTLY]);
	pdbPutInt(ofile, pd->i_skills[SKILL_LISTEN]);
	pdbPutInt(ofile, pd->i_skills[SKILL_HIDE]);
	pdbPutInt(ofile, pd->i_skills[SKILL_CONCENTRATION]);
	pdbPutInt(ofile, pd->i_spellResistance);
	pdbPutInt(ofile, pd->i_damageReduction[0]);
	pdbPutInt(ofile, pd->i_damageReduction[1]);
	pdbPutInt(ofile, pd->i_damageReduction[2]);
	pdbPutInt(ofile, pd->i_size);
	pdbPutInt(ofile, pd->i_xp[MAX]);
	pdbPutInt(ofile, pd->i_xp[TMP]);
	pdbPutInt(ofile, pd->i_inGroup);
	pdbPutString(ofile, pd->c_hitDice);
	pdbPutInt(ofile, pd->flags.f_isStabilised);
	pdbPutInt(ofile, pd->flags.f_disabled);
	pdbPutInt(ofile, pd->flags.f_ignoreCriticals);
	pdbPutInt(ofile, pd->flags.f_autoStablise);
	pdbPutInt(ofile, pd->flags.f_evasion);

	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel0);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel1);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel2);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel3);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel4);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel5);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel6);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel7);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel8);
	pdbPutInt(ofile, pd->spellsAvailable.i_maxLevel9);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel0);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel1);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel2);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel3);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel4);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel5);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel6);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel7);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel8);
	pdbPutInt(ofile, pd->spellsAvailable.i_curLevel9);

	pdbPutInt(ofile, pd->i_playerID);
	pdbPutInt(ofile, pd->i_stdMonsterID);
	pdbPutInt(ofile, pd->flags.f_npc);
	pdbPutInt(ofile, pd->i_regeneration);

	return 0;
}

/********************************************************************************/
int writeAllPlayerData(char *fx) {
int i, w;
char of[256], wf[256];

	sprintf(of, "%s.pty", fx);
	sprintf(wf, "%s.pwp", fx);

	i = pdbClearFile(of); if (i != 0) { printf("Unlinking %s = %d:%d\n", of, i, errno); }
	i = pdbClearFile(wf); if (i != 0) { printf("Unlinking %s = %d:%d\n", wf, i, errno); }

	for (i=0; i<MAX_PLAYERS; i++) {
		if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
			pi_id = i;
			pdbAddRecord(of, writePlayerData, (void *)&players[i]); if (errno != 0) { printf("wAPD: %d = %d\n", i, errno); }
			for (w=0; w<MAX_WEAPONS; w++) {
				if (players[i].w_weapons[w].c_description[0] != '\0' && players[i].w_weapons[w].c_damage[0] != '\0') {
					wi_id = w;
					pdbAddRecord(wf, writePlayerWeaponData, (void *)&players[i].w_weapons[w]); if (errno != 0) { printf("wAPD: %d = %d\n", i, errno); }
				}
			}
		}
	}

	return 0;
}

/********************************************************************************/
int writeAllMonsterData(char *fx) {
int i, w;
char of[256], wf[256];

	sprintf(of, "%s.mon", fx);
	sprintf(wf, "%s.mwp", fx);

	pdbClearFile(of);
	pdbClearFile(wf);

	for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
		if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
			pi_id = i;
			pdbAddRecord(of, writePlayerData, (void *)&players[i]);
			for (w=0; w<MAX_WEAPONS; w++) {
				if (players[i].w_weapons[w].c_description[0] != '\0' && players[i].w_weapons[w].c_damage[0] != '\0') {
					wi_id = w;
					pdbAddRecord(wf, writePlayerWeaponData, (void *)&players[i].w_weapons[w]);
				}
			}
		}
	}

	return 0;
}

/********************************************************************************/
void convertToVersion() {
int v = (int )config.f_version * 100;
char *cp;

	if (v < 111) { v = 0; }

	switch (v) {
		case 112:
				  break;

		case 111:
				  break;

		case 0:		// move the datafiles to backup names
				  	sprintf(mybuf1, "%s_", config.c_partyFilename); rename(config.c_partyFilename, mybuf1);
				  	sprintf(mybuf1, "%s_", config.c_monstFilename); rename(config.c_monstFilename, mybuf1);
					rename("qkmonst.dat", "qkmonst.dat_");

				   if ((cp = strstr(config.c_partyFilename, ".")) != NULL) { *cp = '\0'; }
				   if ((cp = strstr(config.c_monstFilename, ".")) != NULL) { *cp = '\0'; }
					writeAllPlayerData(config.c_partyFilename);
					writeAllMonsterData(config.c_monstFilename);
					writeAllSRDMonsterData("srdmonster");

					config.f_version = ADMPversion;
					writeSystemData("system.sys");

					writeLog("");
					writeLog("*********** doing a version convert to 1.11");
					writeLog("");

				  	break;
	}
}

/********************************************************************************/
void readPlayerWeaponData() {
int i_dummy, i_wpos;
ADMPweapon *wp;

	wp = (ADMPweapon *)pdbDataPointer;
	pdbTokenize(pdbDataLine);

	i_dummy = pdbGetInt();
	i_wpos = pdbGetInt();
	wp->c_available = pdbGetInt();
	strcpy(&wp->c_description[0], pdbGetString());
	wp->i_attackBonus[0] = pdbGetInt();
	wp->i_attackBonus[1] = pdbGetInt();
	wp->i_attackBonus[2] = pdbGetInt();
	wp->i_attackBonus[3] = pdbGetInt();
	strcpy(wp->c_damage, pdbGetString());
	strcpy(wp->c_critical, pdbGetString());
	wp->i_magicalBonus = pdbGetInt();
	wp->i_range = pdbGetInt();
	wp->i_reach = pdbGetInt();
	wp->i_noAttacks = pdbGetInt();
	strcpy(wp->c_message, pdbGetString());
	wp->c_attackType = pdbGetInt();
// printf("%d %s\n", qkMonster, wp->c_description);
}

/********************************************************************************/
void readPlayerData() {
ADMPplayer *pd;
int i;

	pd = (ADMPplayer *)pdbDataPointer;

	pdbTokenize(pdbDataLine);


	i = pdbGetInt();
	strcpy(pd->c_name, pdbGetString());
//printf("rPD: %s\n", pd->c_name);
	pd->i_race = pdbGetInt();
	pd->i_alignment = pdbGetInt();
	pd->i_classes[0] = pdbGetInt();
	pd->i_classes[1] = pdbGetInt();
	pd->i_classes[2] = pdbGetInt();
	pd->i_classes[3] = pdbGetInt();
	pd->i_levels[0] = pdbGetInt();
	pd->i_levels[1] = pdbGetInt();
	pd->i_levels[2] = pdbGetInt();
	pd->i_levels[3] = pdbGetInt();
	pd->i_hp[HP_MAX] = pdbGetInt();
	pd->i_hp[HP_CUR] = pdbGetInt();
	pd->i_hp[HP_NL] = pdbGetInt();
	pd->i_hp[HP_TMP] = pdbGetInt();
	pd->i_bleeding = pdbGetInt();
	pd->i_disabledAt = pdbGetInt();
	pd->i_unconciousAt = pdbGetInt();
	pd->i_savingThrows[SAVE_FORTITUDE] = pdbGetInt();
	pd->i_savingThrows[SAVE_REFLEX] = pdbGetInt();
	pd->i_savingThrows[SAVE_WISDOM] = pdbGetInt();
	pd->i_armorClass[AC_NORMAL] = pdbGetInt();
	pd->i_armorClass[AC_TOUCH] = pdbGetInt();
	pd->i_armorClass[AC_FLATFOOTED] = pdbGetInt();
	pd->i_armorClassSplit[AC_SECTION_DEX] = pdbGetInt();
	pd->i_armorClassSplit[AC_SECTION_SIZE] = pdbGetInt();
	pd->i_armorClassSplit[AC_SECTION_DEFLECTION] = pdbGetInt();
	pd->i_armorClassSplit[AC_SECTION_NATURAL] = pdbGetInt();
	pd->i_armorClassSplit[AC_SECTION_ARMOR] = pdbGetInt();
	pd->i_armorClassSplit[AC_SECTION_SHIELD] = pdbGetInt();
	pd->i_bab[BAB_MELEE][0] = pdbGetInt();
	pd->i_bab[BAB_RANGED][0] = pdbGetInt();
	pd->i_bab[BAB_GRAPPLE][0] = pdbGetInt();
	pd->i_bab[BAB_MELEE][1] = pdbGetInt();
	pd->i_bab[BAB_RANGED][1] = pdbGetInt();
	pd->i_bab[BAB_GRAPPLE][1] = pdbGetInt();
	pd->i_bab[BAB_MELEE][2] = pdbGetInt();
	pd->i_bab[BAB_RANGED][2] = pdbGetInt();
	pd->i_bab[BAB_GRAPPLE][2] = pdbGetInt();
	pd->i_initiative[0] = pdbGetInt();
	pd->i_initiative[1] = pdbGetInt();
	pd->i_speed = pdbGetInt();
	pd->i_inactive = pdbGetInt();
	pd->i_stun = pdbGetInt();
	pd->i_noAttacks = pdbGetInt();
	pd->f_challengeRating = pdbGetFloat();
	pd->i_d20Rolls[0] = pdbGetInt();
	pd->i_d20Rolls[1] = pdbGetInt();
	pd->i_d20Rolls[2] = pdbGetInt();
	pd->i_d20Rolls[3] = pdbGetInt();
	pd->i_d20Rolls[4] = pdbGetInt();
	pd->i_d20Rolls[5] = pdbGetInt();
	pd->i_d20Rolls[6] = pdbGetInt();
	pd->i_spellDC[0] = pdbGetInt();
	pd->i_spellDC[1] = pdbGetInt();
	pd->i_spellDC[2] = pdbGetInt();
	pd->i_spellDC[3] = pdbGetInt();
	pd->i_spellDC[4] = pdbGetInt();
	pd->i_spellDC[5] = pdbGetInt();
	pd->i_spellDC[6] = pdbGetInt();
	pd->i_spellFailure = pdbGetInt();
	pd->i_abilityStats[0][0] = pdbGetInt();
	pd->i_abilityStats[1][0] = pdbGetInt();
	pd->i_abilityStats[0][1] = pdbGetInt();
	pd->i_abilityStats[1][1] = pdbGetInt();
	pd->i_abilityStats[0][2] = pdbGetInt();
	pd->i_abilityStats[1][2] = pdbGetInt();
	pd->i_abilityStats[0][3] = pdbGetInt();
	pd->i_abilityStats[1][3] = pdbGetInt();
	pd->i_abilityStats[0][4] = pdbGetInt();
	pd->i_abilityStats[1][4] = pdbGetInt();
	pd->i_abilityStats[0][5] = pdbGetInt();
	pd->i_abilityStats[1][5] = pdbGetInt();
	pd->i_skills[SKILL_SEARCH] = pdbGetInt();
	pd->i_skills[SKILL_SOH] = pdbGetInt();
	pd->i_skills[SKILL_SPOT] = pdbGetInt();
	pd->i_skills[SKILL_OPENLOCKS] = pdbGetInt();
	pd->i_skills[SKILL_MOVESILENTLY] = pdbGetInt();
	pd->i_skills[SKILL_LISTEN] = pdbGetInt();
	pd->i_skills[SKILL_HIDE] = pdbGetInt();
	pd->i_skills[SKILL_CONCENTRATION] = pdbGetInt();
	pd->i_spellResistance = pdbGetInt();
	pd->i_damageReduction[0] = pdbGetInt();
	pd->i_damageReduction[1] = pdbGetInt();
	pd->i_damageReduction[2] = pdbGetInt();
	pd->i_size = pdbGetInt();
	pd->i_xp[MAX] = pdbGetInt();
	pd->i_xp[TMP] = pdbGetInt();
	pd->i_inGroup = pdbGetInt();
	strcpy(pd->c_hitDice, pdbGetString());
	pd->flags.f_isStabilised = pdbGetInt();
	pd->flags.f_disabled = pdbGetInt();
	pd->flags.f_ignoreCriticals = pdbGetInt();
	pd->flags.f_autoStablise = pdbGetInt();
	pd->flags.f_evasion = pdbGetInt();
	
	pd->spellsAvailable.i_maxLevel0 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel1 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel2 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel3 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel4 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel5 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel6 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel7 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel8 = pdbGetInt();
	pd->spellsAvailable.i_maxLevel9 = pdbGetInt();
	pd->spellsAvailable.i_curLevel0 = pdbGetInt();
	pd->spellsAvailable.i_curLevel1 = pdbGetInt();
	pd->spellsAvailable.i_curLevel2 = pdbGetInt();
	pd->spellsAvailable.i_curLevel3 = pdbGetInt();
	pd->spellsAvailable.i_curLevel4 = pdbGetInt();
	pd->spellsAvailable.i_curLevel5 = pdbGetInt();
	pd->spellsAvailable.i_curLevel6 = pdbGetInt();
	pd->spellsAvailable.i_curLevel7 = pdbGetInt();
	pd->spellsAvailable.i_curLevel8 = pdbGetInt();
	pd->spellsAvailable.i_curLevel9 = pdbGetInt();

	pd->i_playerID = pdbGetInt();
	pd->i_stdMonsterID = pdbGetInt();
	if (i_monsterType < 1) {
		pd->i_stdMonsterID = qkMonster;
	}
	pd->flags.f_npc = pdbGetInt();
	pd->i_regeneration = pdbGetInt();
//	if (pd->i_regeneration > 0) { printf("%s = %d\n", pd->c_name, pd->i_regeneration); }
}

/********************************************************************************/
int readAllPlayerData(char *fx) {
int i, w;
char of[256], wf[256];
char key1[10], key2[10];

	sprintf(of, "%s.pty", fx);
	sprintf(wf, "%s.pwp", fx);

	for (i=0; i<MAX_PLAYERS; i++) {
		sprintf(key1, "%d", i);
		pdbKeySearch(of, readPlayerData, (void *)&players[i], 1, key1, NULL, NULL);
		for (w=0; w<MAX_WEAPONS; w++) {
			sprintf(key2, "%d", w);
			pdbKeySearch(wf, readPlayerWeaponData, (void *)&players[i].w_weapons[w], 1, key1, key2, NULL);
		}
	}

	return 0;
}

/********************************************************************************/
int readAllMonsterData(char *fx, ADMPplayer *p) {
int i, w;
char of[256], wf[256];
char key1[10], key2[10];

	sprintf(of, "%s.mon", fx);
	sprintf(wf, "%s.mwp", fx);

	for (i=0; i<MAX_MONSTERS; i++) {
		sprintf(key1, "%d", i+MAX_PLAYERS);
		pdbKeySearch(of, readPlayerData, (void *)&p[i], 1, key1, NULL, NULL);
		for (w=0; w<MAX_WEAPONS; w++) {
			sprintf(key2, "%d", w);
			pdbKeySearch(wf, readPlayerWeaponData, (void *)&p[i].w_weapons[w], 1, key1, key2, NULL);
		}
	}

	return 0;
}

/********************************************************************************/
int readQKMonsterData() {
	qkMonster++;
	pdbDataPointer = (void *)&qkmonsters[qkMonster];
	readPlayerData();
	qkmonsters[qkMonster].flags.f_srdMonster = i_monsterType;
	qkmonsters[qkMonster].i_stdMonsterID = (qkMonster - i_monsterCount) + i_monsterType;
//printf("loaded: %s (%d/%d)\n", qkmonsters[qkMonster].c_name, qkMonster, qkmonsters[qkMonster].i_stdMonsterID);

	return 0;
}

/********************************************************************************/
void readQKMonstWeaponData() {
	pdbDataPointer = (void *)&qkmonsters[qkMonster].w_weapons[qkMonsterWeapon];
	readPlayerWeaponData();
	qkMonsterWeapon++;
}

/********************************************************************************/
int readAllStdMonsterData(char *fx) {
char of[256], wf[256];
char key1[10];

	sprintf(of, "%s.std", fx);
	sprintf(wf, "%s.swp", fx);

	pdbLoadAll(of, readQKMonsterData);
	if (i_monsterType == 0) { i_monsterCount = qkMonster; }
//printf("i_monsterCount = %d\n", i_monsterCount);

	for (int i=0; i<MAX_QKMONSTERS; i++) {
		if (qkmonsters[i].c_name[0] != '\0') {
			if (i_monsterType == 0) { qkMonster = i; } else { qkMonster = i_monsterCount + i; }
			sprintf(key1, "%d", i);
			qkMonsterWeapon = 0;
			pdbKeySearch(wf, readQKMonstWeaponData, (void *)&qkmonsters[qkMonster].w_weapons[0], MAX_WEAPONS, key1, NULL, NULL);
		}
	}

	return 0;
}

/********************************************************************************/
int writeAllSRDMonsterData(char *fx) {
int i, w, r;
char of[256], wf[256];

	sprintf(of, "%s.std", fx);
	sprintf(wf, "%s.swp", fx);

	pdbClearFile(of);
	pdbClearFile(wf);

	for (i=0; i<MAX_QKMONSTERS; i++) {
		if (qkmonsters[i].c_name[0] != '\0' && qkmonsters[i].i_hp[HP_MAX] > 0 && qkmonsters[i].flags.f_srdMonster == 0) {
			pi_id = i;
			r = pdbAddRecord(of, writePlayerData, (void *)&qkmonsters[i]);
			for (w=0; w<MAX_WEAPONS; w++) {
				if (qkmonsters[i].w_weapons[w].c_description[0] != '\0' && qkmonsters[i].w_weapons[w].c_damage[0] != '\0') {
					wi_id = w;
					r = pdbAddRecord(wf, writePlayerWeaponData, (void *)&qkmonsters[i].w_weapons[w]);
				}
			}
		}
	}

	return 0;
}

/********************************************************************************/
int writeAllStdMonsterData(char *fx) {
int i, w, r, j;
char of[256], wf[256];

	sprintf(of, "%s.std", fx);
	sprintf(wf, "%s.swp", fx);

	pdbClearFile(of);
	pdbClearFile(wf);

	j=0;

	for (i=0; i<MAX_QKMONSTERS; i++) {
		if (qkmonsters[i].c_name[0] != '\0' && qkmonsters[i].i_hp[HP_MAX] > 0 && qkmonsters[i].flags.f_srdMonster != 0) {
			qkmonsters[i].i_stdMonsterID = pi_id = j++;		// TODO
			r = pdbAddRecord(of, writePlayerData, (void *)&qkmonsters[i]);
			for (w=0; w<MAX_WEAPONS; w++) {
				if (qkmonsters[i].w_weapons[w].c_description[0] != '\0' && qkmonsters[i].w_weapons[w].c_damage[0] != '\0') {
					wi_id = w;
					r = pdbAddRecord(wf, writePlayerWeaponData, (void *)&qkmonsters[i].w_weapons[w]);
				}
			}
		}
	}

	// TODO - write notes file

	return 0;
}

/********************************************************************************/
void loadMonsterDescription(char *name) {
char fname[256], tname[64], *words[3] = {NULL, NULL, NULL};
int i=4;
FILE *ifile;

	while (name[i] != '\0') {
		tname[i-4] = toupper(name[i]);
		if (tname[i-4] == ' ') { tname[i-4] = '_'; }
		i++;
	}

	tname[i-4] = '\0';
	sprintf(fname, "srd/monsters/%s.html", tname);

	if ((ifile = fopen(fname, "r")) != NULL) {
		fclose(ifile);
		a_CTMonsterHTMLDetails->load(fname);
		return;
	} else {
		if ((words[1] = strstr(tname, "_")) != NULL) {
			*words[1] = '\0'; words[1]++;
			if ((words[2] = strstr(words[1], "_")) != NULL) {
				*words[2] = '\0'; words[2]++;
			}
			sprintf(fname, "srd/monsters/%s.html", tname);
			if ((ifile = fopen(fname, "r")) != NULL) {
				fclose(ifile);
				a_CTMonsterHTMLDetails->load(fname);
				return;
			} else {
				sprintf(fname, "srd/monsters/%s_%s.html", tname, words[1]);
				if ((ifile = fopen(fname, "r")) != NULL) {
					fclose(ifile);
					a_CTMonsterHTMLDetails->load(fname);
					return;
				} else {
					sprintf(fname, "srd/monsters/%s.html", words[1]);
					if ((ifile = fopen(fname, "r")) != NULL) {
						fclose(ifile);
						a_CTMonsterHTMLDetails->load(fname);
						return;
					}
				}
			}
		}
	}
	a_CTMonsterHTMLDetails->load("");
}

/********************************************************************************/
int myalphasort(const void *a, const void *b) {
#ifndef VISUALC
	return (strcasecmp((*(const struct dirent **) a)->d_name, (*(const struct dirent **) b)->d_name));
#else
	return (strcmpi((*(const struct dirent **) a)->d_name, (*(const struct dirent **) b)->d_name));
#endif
}

/********************************************************************************/
void loadSRDspellEntries(char *dir) {
int num_entries, i=0, j=0;
char *cp, *cp1;
struct dirent **list;

	for (i=0; i<128; i++) { i_spellEntries[i] = -1; }
	a_CTSpellDetailsList->clear();

	num_entries = fl_filename_list(dir, &list, fl_casealphasort);
	if (num_entries < 3) { return; }
	list++; list++;
	for (i=2; i<num_entries; i++) {
		cp = (*list)->d_name;
			// remove the .html extension
		if ((cp1 = strstr(cp, ".")) != NULL) { *cp1 = '\0'; }
		j = tolower(cp[0]) - 'a';
		if (i_spellEntries[j] == -1) { i_spellEntries[j] = i; } //printf("lSSE: %d (%c) = %d\n", j, cp[0], i); }
		a_CTSpellDetailsList->add(cp);
		list++;
	}
}

/********************************************************************************/
void showSRDspellEntries() {
int i;

	strcpy(mybuf2, a_CTSpellDetailsSearch->value());
	i = tolower(mybuf2[0]) - 'a';

	if (i < 0 || i > 100) { return; }
	if (i_spellEntries[i] < 0) { return; }

	a_CTSpellDetailsList->topline(i_spellEntries[i]);
}

/********************************************************************************/
void selectSRDspellEntries() {
	sprintf(mybuf2, "srd/spells/%s.html", a_CTSpellDetailsList->text(a_CTSpellDetailsList->value()));
	a_CTSpellHTMLDetails->load(mybuf2);
}

/********************************************************************************/
int readAllStdWeaponData() {
char key1[10];

	for (int i=0; i<MAX_STDWEAPONS; i++) {
		sprintf(key1, "%d", i);
		pdbKeySearch("stdweapon.wpd", readPlayerWeaponData, (void *)&stdweapons[i], 1, key1, NULL, NULL);
	}

	return 0;
}

/********************************************************************************/
void qkMonsterAdd(int action) {
ADMPplayer plyrs;
int pid=0, ok=0, i, j;

	switch (action) {
			// show it
		case 0:  a_QkMonsterAdd->show();
					aQkAdd_Name->value("");
					aQkAdd_Count->value(1);
					aQkAdd_HPMax->value("");
					aQkAdd_ACNormal->value("");
					aQkAdd_ACTouch->value("");
					aQkAdd_ACFlatFoot->value("");
					aQkAdd_InitBonus->value("");
					aQkAdd_Speed->value("");
					aQkAdd_SaveFort->value("");
					aQkAdd_SaveReflex->value("");
					aQkAdd_SaveWisdom->value("");
					aQkAdd_NoAttacks->value("1");
					aQkAdd_CRating->value("");
					aQkAdd_SpellResistance->value("");
					aQkAdd_DamReductionHP->value("");
					aQkAdd_DamReductionWeapon->value("");
					aQkAdd_DamReductionType->value(0);
					break;

			// accept
		case 1:  a_QkMonsterAdd->hide();
					memset(plyrs.c_name, '\0', sizeof(ADMPplayer));
					sprintf(mybuf1, "Z - %s", aQkAdd_Name->value());
					mybuf1[29] = '\0'; strcpy(plyrs.c_name, mybuf1);
					plyrs.i_hp[HP_CUR] = plyrs.i_hp[HP_MAX] = atoi(aQkAdd_HPMax->value());
					if (plyrs.c_name[4] == '\0' && plyrs.i_hp[HP_MAX] == 0) { return; }
					plyrs.i_savingThrows[SAVE_FORTITUDE] = atoi(aQkAdd_SaveFort->value());
					plyrs.i_savingThrows[SAVE_REFLEX] = atoi(aQkAdd_SaveReflex->value());
					plyrs.i_savingThrows[SAVE_WISDOM] = atoi(aQkAdd_SaveWisdom->value());
					plyrs.i_armorClass[AC_NORMAL] = atoi(aQkAdd_ACNormal->value());
					plyrs.i_armorClass[AC_TOUCH] = atoi(aQkAdd_ACTouch->value());
					plyrs.i_armorClass[AC_FLATFOOTED] = atoi(aQkAdd_ACFlatFoot->value());
					plyrs.i_initiative[0] = atoi(aQkAdd_InitBonus->value());
					plyrs.i_speed = atoi(aQkAdd_Speed->value());
					plyrs.i_noAttacks = atoi(aQkAdd_NoAttacks->value());
				
					plyrs.f_challengeRating = (float )atof(aQkAdd_CRating->value());
					plyrs.i_spellResistance = atoi(aQkAdd_SpellResistance->value());
					plyrs.i_damageReduction[0] = aQkAdd_DamReductionType->value();
					plyrs.i_damageReduction[1] = atoi(aQkAdd_DamReductionHP->value());
					plyrs.i_damageReduction[2] = atoi(aQkAdd_DamReductionWeapon->value());

					plyrs.w_weapons[0].c_available=1;
					strncpy(plyrs.w_weapons[0].c_description, aQkAdd_WeaponName1->value(), 29);
					plyrs.w_weapons[0].i_attackBonus[0] = atoi(aQkAdd_WeaponAttkBonus11->value());
					plyrs.w_weapons[0].i_attackBonus[1] = atoi(aQkAdd_WeaponAttkBonus12->value());
					plyrs.w_weapons[0].i_attackBonus[2] = atoi(aQkAdd_WeaponAttkBonus13->value());
					plyrs.w_weapons[0].i_attackBonus[3] = atoi(aQkAdd_WeaponAttkBonus14->value());
					strncpy(plyrs.w_weapons[0].c_damage, aQkAdd_WeaponDamage1->value(), 19);
					strncpy(plyrs.w_weapons[0].c_critical, aQkAdd_WeaponCritical1->value(), 19);
					plyrs.w_weapons[0].i_magicalBonus = atoi(aQkAdd_WeaponMagicBonus1->value());
					plyrs.w_weapons[0].i_range = atoi(aQkAdd_WeaponRange1->value());
					plyrs.w_weapons[0].i_reach = atoi(aQkAdd_WeaponReach1->value());
					plyrs.w_weapons[0].i_noAttacks = atoi(aQkAdd_WeaponNoAttacks1->value());
					plyrs.w_weapons[0].c_description[29] = '\0';
					plyrs.w_weapons[0].c_damage[19] = '\0';
					plyrs.w_weapons[0].c_critical[19] = '\0';
					plyrs.w_weapons[0].c_message[29] = '\0';

					plyrs.w_weapons[1].c_available=1;
					strncpy(plyrs.w_weapons[1].c_description, aQkAdd_WeaponName2->value(), 29);
					plyrs.w_weapons[1].i_attackBonus[0] = atoi(aQkAdd_WeaponAttkBonus21->value());
					plyrs.w_weapons[1].i_attackBonus[1] = atoi(aQkAdd_WeaponAttkBonus22->value());
					plyrs.w_weapons[1].i_attackBonus[2] = atoi(aQkAdd_WeaponAttkBonus23->value());
					plyrs.w_weapons[1].i_attackBonus[3] = atoi(aQkAdd_WeaponAttkBonus24->value());
					strncpy(plyrs.w_weapons[1].c_damage, aQkAdd_WeaponDamage2->value(), 19);
					strncpy(plyrs.w_weapons[1].c_critical, aQkAdd_WeaponCritical2->value(), 19);
					plyrs.w_weapons[1].i_magicalBonus = atoi(aQkAdd_WeaponMagicBonus2->value());
					plyrs.w_weapons[1].i_range = atoi(aQkAdd_WeaponRange2->value());
					plyrs.w_weapons[1].i_reach = atoi(aQkAdd_WeaponReach2->value());
					plyrs.w_weapons[1].i_noAttacks = atoi(aQkAdd_WeaponNoAttacks2->value());
					plyrs.w_weapons[1].c_description[29] = '\0';
					plyrs.w_weapons[1].c_damage[19] = '\0';
					plyrs.w_weapons[1].c_critical[19] = '\0';
					plyrs.w_weapons[1].c_message[29] = '\0';

					plyrs.i_unconciousAt = -1;
					plyrs.flags.f_disabled = 0;
					for (j=0; j<MAX_D20ROLLS; j++) {
						plyrs.i_d20Rolls[j] = getRND(20);
						plyrs.i_whomAttacked[j][0] = -1;
		 			}
		 			plyrs.i_whomAttacked[0][0] = plyrs.i_whomAttacked[1][0] = plyrs.i_whomAttacked[2][0] = -1;
				
					reloadNames(0);
				
					if (config.flags.f_debug != 0) printf("Saving %d\n", pid);
				
					// do some quick sanity checks
					if (a_EORMessageList == NULL) { doEndOfRoundMessages(0); }
					a_EORMessageList->clear();
					a_EORMessageList->add("");
					a_EORMessageList->add("@B63@.The following are possible problems with this monster");
					a_EORMessageList->add("");
					if (plyrs.i_hp[HP_MAX] == 0 || plyrs.i_hp[HP_CUR] == 0) { ok = 1; a_EORMessageList->add("  HP's are zero"); }
					if (plyrs.i_armorClass[AC_NORMAL] == 0) { ok = 1; a_EORMessageList->add("  A/C is zero"); }
					if (plyrs.i_savingThrows[SAVE_FORTITUDE] == 0 && plyrs.i_savingThrows[SAVE_REFLEX] == 0 && plyrs.i_savingThrows[SAVE_WISDOM] == 0) { ok = 1; a_EORMessageList->add("  Saving Throws have issues"); }
					if (plyrs.i_noAttacks == 0) { ok = 1; a_EORMessageList->add("  #Attks is zero"); }
					if (plyrs.w_weapons[0].i_attackBonus[0] == 0 && plyrs.w_weapons[1].i_attackBonus[0] == 0) { ok = 1; a_EORMessageList->add("  Weapon BAB's dont seem to be set"); }
					if (plyrs.f_challengeRating == 0 && pid >= MAX_PLAYERS) { ok = 1; a_EORMessageList->add("  CR value is zero"); }
				
					if (ok == 1) {
						doEndOfRoundMessages(0);
					}
					j = (int )aQkAdd_Count->value();
					for (; j>0; j--) {
						for (i=MAX_PLAYERS; i<MAX_MEMBERS; i++) {
							if (players[i].c_name[0] == '\0' && players[i].i_hp[HP_MAX] == 0) {
								memcpy(&players[i].c_name[0], &plyrs.c_name[0], sizeof(ADMPplayer));
								players[i].c_name[0] = identifiers[i];
								break;
							}
						}
					}
					reloadNames(0);
					break;

			// cancel
		case 2:  a_QkMonsterAdd->hide();
					break;
	}
}

/********************************************************************************/
int writeSystemData(char *fn) {
FILE *ofile;
int i,j;

	if ((ofile = fopen(fn, "w+")) == NULL) {
		return -1;
	}
	pdbFilePointer = ofile; 

	fprintf(ofile, "0|");

	pdbPutFloat(ofile, config.f_version);
	pdbPutInt(ofile, config.i_roundCount);
	pdbPutInt(ofile, config.i_whenBleed);
	pdbPutInt(ofile, config.i_editID);
	pdbPutInt(ofile, config.i_actionInitID);
	pdbPutInt(ofile, config.i_actionRecvID);
	pdbPutInt(ofile, config.i_actionSelected);
	pdbPutInt(ofile, config.i_actionWeaponSelected);
	pdbPutInt(ofile, config.i_roundsSave);
	pdbPutString(ofile, config.c_partyFilename);
	pdbPutString(ofile, config.c_monstFilename);
	pdbPutInt(ofile, config.flags.f_debug);
	pdbPutInt(ofile, config.flags.f_rollAllRolls);
	pdbPutInt(ofile, config.flags.f_rollInitRolls);
	pdbPutInt(ofile, config.flags.f_logfile);
	pdbPutInt(ofile, config.flags.f_rollInitEveryRnd);
	pdbPutInt(ofile, config.flags.f_removeMonstersInit);
	pdbPutInt(ofile, config.flags.f_oneMissTwentyHits);

	pdbPutNewLine(ofile); fprintf(ofile, "0|");

	for (i=0; i<15; i++) {
		for (j=0; j<MAX_MEMBERS+2; j++) {
			pdbPutInt(ofile, config.i_idList[i][j]);
		}
	}

	pdbPutNewLine(ofile); fprintf(ofile, "0|");
	for (i=0; i<MAX_MODIFIERS; i++) {
		pdbPutString(ofile, config.system_mods[i].c_message);
		pdbPutInt(ofile, config.system_mods[i].i_idFollows);
		pdbPutInt(ofile, config.system_mods[i].i_noRounds);
		for (j=0; j<MAX_MEMBERS; j++) { pdbPutInt(ofile, config.system_mods[i].i_whom[j]); }
		for (j=0; j<5; j++) { 
			pdbPutInt(ofile, config.system_mods[i].i_data[j][0]); 
			pdbPutInt(ofile, config.system_mods[i].i_data[j][1]); 
			pdbPutInt(ofile, config.system_mods[i].i_data[j][2]); 
		}
	}

	pdbPutNewLine(ofile); fprintf(ofile, "0|");
	for (i=0; i<MAX_MODIFIERS; i++) {
		pdbPutString(ofile, config.action_mods[i].c_message);
		pdbPutInt(ofile, config.action_mods[i].i_idFollows);
		pdbPutInt(ofile, config.action_mods[i].i_noRounds);
		for (j=0; j<MAX_MEMBERS; j++) { pdbPutInt(ofile, config.action_mods[i].i_whom[j]); }
		for (j=0; j<5; j++) { 
			pdbPutInt(ofile, config.action_mods[i].i_data[j][0]); 
			pdbPutInt(ofile, config.action_mods[i].i_data[j][1]); 
			pdbPutInt(ofile, config.action_mods[i].i_data[j][2]); 
		}
	}

	pdbPutNewLine(ofile); fprintf(ofile, "0|");
	for (i=0; i<MAX_TRIGGERS; i++) {
		pdbPutString(ofile, config.action_msgs[i].c_message);
		pdbPutInt(ofile, config.action_msgs[i].i_noRounds);
		pdbPutInt(ofile, config.action_msgs[i].i_display);
		pdbPutString(ofile, config.action_msgs[i].c_every);
	}

	pdbPutNewLine(ofile); fprintf(ofile, "0|");
	for (i=0; i<MAX_AUTODAMAGE; i++) {
		pdbPutString(ofile, config.auto_damage[i].c_description);
		pdbPutInt(ofile, config.auto_damage[i].i_initiatorID);
		for (j=0; j<MAX_MEMBERS; j++) { pdbPutInt(ofile, config.auto_damage[i].i_whom[j]); }
		pdbPutString(ofile, config.auto_damage[i].c_damage);
		pdbPutInt(ofile, config.auto_damage[i].i_noRounds);
		pdbPutInt(ofile, config.auto_damage[i].i_doneRounds);
		pdbPutInt(ofile, config.auto_damage[i].i_type);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag1);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag2);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag3);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag4);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag5);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag6);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag7);
		pdbPutInt(ofile, config.auto_damage[i].flags.f_flag8);
	}

	fclose(ofile);

	return 0;
}

/********************************************************************************/
int readSystemData(char *fn) {
FILE *ofile;
int i,j;

	if ((ofile = fopen(fn, "r")) == NULL) {
		return -1;
	}
	pdbFilePointer = ofile;
	fgets(pdbDataLine, PDBLINESIZE, pdbFilePointer);
	pdbTokenize(pdbDataLine);

	config.f_version = pdbGetFloat();
	config.i_roundCount = pdbGetInt();
	config.i_whenBleed = pdbGetInt();
	config.i_editID = pdbGetInt();
	config.i_actionInitID = pdbGetInt();
	config.i_actionRecvID = pdbGetInt();
	config.i_actionSelected = pdbGetInt();
	config.i_actionWeaponSelected = pdbGetInt();
	config.i_roundsSave = pdbGetInt();
	strcpy(config.c_partyFilename, pdbGetString());
	strcpy(config.c_monstFilename, pdbGetString());
	config.flags.f_debug = pdbGetInt();
	config.flags.f_rollAllRolls = pdbGetInt();
	config.flags.f_rollInitRolls = pdbGetInt();
	config.flags.f_logfile = pdbGetInt();
	config.flags.f_rollInitEveryRnd = pdbGetInt();
	config.flags.f_removeMonstersInit = pdbGetInt();
	config.flags.f_oneMissTwentyHits = pdbGetInt();
	
	fgets(pdbDataLine, PDBLINESIZE, pdbFilePointer);
	pdbTokenize(pdbDataLine);
	for (i=0; i<15; i++) {
		for (j=0; j<MAX_MEMBERS+2; j++) {
			config.i_idList[i][j] = pdbGetInt();
		}
	}
	
	fgets(pdbDataLine, PDBLINESIZE, pdbFilePointer);
	pdbTokenize(pdbDataLine);
	for (i=0; i<MAX_MODIFIERS; i++) {
		strcpy(config.system_mods[i].c_message, pdbGetString());
		config.system_mods[i].i_idFollows = pdbGetInt();
		config.system_mods[i].i_noRounds = pdbGetInt();
		for (j=0; j<MAX_MEMBERS; j++) { config.system_mods[i].i_whom[j] = pdbGetInt(); }
		for (j=0; j<5; j++) { 
			config.system_mods[i].i_data[j][0] = pdbGetInt();
			config.system_mods[i].i_data[j][1] = pdbGetInt();
			config.system_mods[i].i_data[j][2] = pdbGetInt();
		}
	}
	
	fgets(pdbDataLine, PDBLINESIZE, pdbFilePointer);
	pdbTokenize(pdbDataLine);
	for (i=0; i<MAX_MODIFIERS; i++) {
		strcpy(config.action_mods[i].c_message, pdbGetString());
		config.action_mods[i].i_idFollows = pdbGetInt();
		config.action_mods[i].i_noRounds = pdbGetInt();
		for (j=0; j<MAX_MEMBERS; j++) { config.action_mods[i].i_whom[j] = pdbGetInt(); }
		for (j=0; j<5; j++) { 
			config.action_mods[i].i_data[j][0] = pdbGetInt();
			config.action_mods[i].i_data[j][1] = pdbGetInt();
			config.action_mods[i].i_data[j][2] = pdbGetInt();
		}
	}
	
	fgets(pdbDataLine, PDBLINESIZE, pdbFilePointer);
	pdbTokenize(pdbDataLine);
	for (i=0; i<MAX_TRIGGERS; i++) {
		strcpy(config.action_msgs[i].c_message, pdbGetString());
		config.action_msgs[i].i_noRounds = pdbGetInt();
		config.action_msgs[i].i_display = pdbGetInt();
		if (config.f_version > 1.12) {
			strncpy(config.action_msgs[i].c_every, pdbGetString(), 9); config.action_msgs[i].c_every[9] = '\0';
		}
	}
	
	fgets(pdbDataLine, PDBLINESIZE, pdbFilePointer);
	pdbTokenize(pdbDataLine);
	for (i=0; i<MAX_AUTODAMAGE; i++) {
		strcpy(config.auto_damage[i].c_description, pdbGetString());
		config.auto_damage[i].i_initiatorID = pdbGetInt();
		for (j=0; j<MAX_MEMBERS; j++) { config.auto_damage[i].i_whom[j] = pdbGetInt(); }
		strcpy(config.auto_damage[i].c_damage, pdbGetString());
		config.auto_damage[i].i_noRounds = pdbGetInt();
		config.auto_damage[i].i_doneRounds = pdbGetInt();
		config.auto_damage[i].i_type = pdbGetInt();
		config.auto_damage[i].flags.f_flag1 = pdbGetInt();
		config.auto_damage[i].flags.f_flag2 = pdbGetInt();
		config.auto_damage[i].flags.f_flag3 = pdbGetInt();
		config.auto_damage[i].flags.f_flag4 = pdbGetInt();
		config.auto_damage[i].flags.f_flag5 = pdbGetInt();
		config.auto_damage[i].flags.f_flag6 = pdbGetInt();
		config.auto_damage[i].flags.f_flag7 = pdbGetInt();
		config.auto_damage[i].flags.f_flag8 = pdbGetInt();
	}

	fclose(ofile);

	return 0;
}

/********************************************************************************/
void loadCombatNotes() {
char *cp;

	cp = pdbTokenize(pdbDataLine);	// get rid of the line no and key
	cp = pdbGetString();

	while ((cp = pdbGetString()) != NULL) {
		a_CTNotesDetails->add(cp);
	}
}

/********************************************************************************/
void loadEditNotes() {
char *cp;

	cp = pdbTokenize(pdbDataLine);	// get rid of the line no and key
	cp = pdbGetString();

	while ((cp = pdbGetString()) != NULL) {
		edit_notes_buffer->append(cp);
		edit_notes_buffer->append("\n");
	}
}

/********************************************************************************/
void doNotesAction(int action, int id) {
FILE *nf;
char *cp;

	if (id >= MAX_PLAYERS) {
		sprintf(mybuf1, "%s.mnt", config.c_monstFilename);
	} else {
		sprintf(mybuf1, "%s.pnt", config.c_partyFilename);
	}

if (config.flags.f_debug != 0) printf("dNA: %d %d %d\n", action, id, edit_notes_buffer->length());

	switch(action) {
		case 0:		// SAVE
					sprintf(mybuf2, "%d", id);
					pdbDeleteAllRecord(mybuf1, mybuf2, NULL, NULL);
					if ((nf = fopen(mybuf1, "a+")) != NULL) {
						strncpy(mybuf1, edit_notes_buffer->text(),2047);
						while ((cp = strstr(mybuf1, "\n")) != NULL) {
							*cp = '|';
						}
						fprintf(nf, "+%d|%d|%s|\n", id, id, mybuf1);
						fclose(nf);
					}
				  break;

		case 1:		// LOAD
				  edit_notes_buffer->select(0, edit_notes_buffer->length());
				  edit_notes_buffer->remove_selection();
				  sprintf(mybuf2, "%d", id);
				  pdbKeySearch(mybuf1, loadEditNotes, NULL, 1, mybuf2, NULL, NULL);
				  break;

		case 2:		// DISPLAY ON COMBAT SCREEN
					a_CTNotesDetails->clear();
				  sprintf(mybuf2, "%d", id);
				  pdbKeySearch(mybuf1, loadCombatNotes, NULL, 1, mybuf2, NULL, NULL);
				  break;
	}
}

/********************************************************************************/
void qkHealAction(int action) {
	int j=0, k;

	switch (action) {
		case 0:		// refresh
				  a_qkheal_ln00->deactivate(); a_qkheal_ln01->deactivate(); a_qkheal_ln01->value("");
				  a_qkheal_ln10->deactivate(); a_qkheal_ln11->deactivate(); a_qkheal_ln11->value("");
				  a_qkheal_ln20->deactivate(); a_qkheal_ln21->deactivate(); a_qkheal_ln21->value("");
				  a_qkheal_ln30->deactivate(); a_qkheal_ln31->deactivate(); a_qkheal_ln31->value("");
				  a_qkheal_ln40->deactivate(); a_qkheal_ln41->deactivate(); a_qkheal_ln41->value("");
				  a_qkheal_ln50->deactivate(); a_qkheal_ln51->deactivate(); a_qkheal_ln51->value("");
				  a_qkheal_ln60->deactivate(); a_qkheal_ln61->deactivate(); a_qkheal_ln61->value("");
				  a_qkheal_ln70->deactivate(); a_qkheal_ln71->deactivate(); a_qkheal_ln71->value("");
				  a_qkheal_ln80->deactivate(); a_qkheal_ln81->deactivate(); a_qkheal_ln81->value("");
				  a_qkheal_ln90->deactivate(); a_qkheal_ln91->deactivate(); a_qkheal_ln91->value("");
// TODO
				  for (int i=0; i<MAX_PLAYERS; i++) {
						if (players[i].c_name[0] != '\0' && players[i].i_hp[HP_MAX] > 0) {
							sprintf(mybuf1, "%s (%d/%d) = %d", players[i].c_name, players[i].i_hp[HP_CUR], players[i].i_hp[HP_MAX], (players[i].i_hp[HP_MAX] - players[i].i_hp[HP_CUR]));
							switch (j) {
								case 0: a_qkheal_ln00->value(mybuf1); a_qkheal_ln00->activate(); a_qkheal_ln01->activate(); break;
								case 1: a_qkheal_ln10->value(mybuf1); a_qkheal_ln10->activate(); a_qkheal_ln11->activate(); break;
								case 2: a_qkheal_ln20->value(mybuf1); a_qkheal_ln20->activate(); a_qkheal_ln21->activate(); break;
								case 3: a_qkheal_ln30->value(mybuf1); a_qkheal_ln30->activate(); a_qkheal_ln31->activate(); break;
								case 4: a_qkheal_ln40->value(mybuf1); a_qkheal_ln40->activate(); a_qkheal_ln41->activate(); break;
								case 5: a_qkheal_ln50->value(mybuf1); a_qkheal_ln50->activate(); a_qkheal_ln51->activate(); break;
								case 6: a_qkheal_ln60->value(mybuf1); a_qkheal_ln60->activate(); a_qkheal_ln61->activate(); break;
								case 7: a_qkheal_ln70->value(mybuf1); a_qkheal_ln70->activate(); a_qkheal_ln71->activate(); break;
								case 8: a_qkheal_ln80->value(mybuf1); a_qkheal_ln80->activate(); a_qkheal_ln81->activate(); break;
								case 9: a_qkheal_ln90->value(mybuf1); a_qkheal_ln90->activate(); a_qkheal_ln91->activate(); break;
							}
							config.i_idList[ID_QKHEAL][j] = i;
							j++;
							config.i_idList[ID_QKHEAL][j] = -1;
						}
				  }
				  break;

		case 1:		// accept
				  if ((k=atoi(a_qkheal_ln01->value())) != 0) { j = config.i_idList[ID_QKHEAL][0]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln11->value())) != 0) { j = config.i_idList[ID_QKHEAL][1]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln21->value())) != 0) { j = config.i_idList[ID_QKHEAL][2]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln31->value())) != 0) { j = config.i_idList[ID_QKHEAL][3]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln41->value())) != 0) { j = config.i_idList[ID_QKHEAL][4]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln51->value())) != 0) { j = config.i_idList[ID_QKHEAL][5]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln61->value())) != 0) { j = config.i_idList[ID_QKHEAL][6]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln71->value())) != 0) { j = config.i_idList[ID_QKHEAL][7]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln81->value())) != 0) { j = config.i_idList[ID_QKHEAL][8]; players[j].i_hp[HP_CUR] += k; }
				  if ((k=atoi(a_qkheal_ln91->value())) != 0) { j = config.i_idList[ID_QKHEAL][9]; players[j].i_hp[HP_CUR] += k; }
				  j = 0;
				  while (config.i_idList[ID_QKHEAL][j] != -1) {
						k = config.i_idList[ID_QKHEAL][j];
						if (players[k].i_hp[HP_CUR] > players[k].i_hp[HP_MAX]) { players[k].i_hp[HP_CUR] = players[k].i_hp[HP_MAX]; }
						j++;
				  }
				  a_qkheal_ln01->value(""); a_qkheal_ln11->value(""); a_qkheal_ln21->value("");
				  a_qkheal_ln31->value(""); a_qkheal_ln41->value(""); a_qkheal_ln51->value("");
				  a_qkheal_ln61->value(""); a_qkheal_ln71->value(""); a_qkheal_ln81->value("");
				  a_qkheal_ln91->value("");
				  break;
	}
}

/********************************************************************************/
